#ifndef STRING_DOT_H
#define STRING_DOT_H

#ifndef __const
#ifdef __cplusplus
#define __const const
#else
#define __const
#endif
#endif

#ifndef __cplusplus
overload read, write, fgets, gets, fputs, puts;
overload strcmp,strchr,strrchr,strpbrk,strspn,strcspn;
#endif

#include <string.h>
#include <osfcn.h>
#include <stream.h>
#include <Pool.h>
#include <memory.h>

#include <Objection.h>
extern Objection S_memory_error;      // out of memory error
extern Objection S_substr_error;      // substring domain error
extern Objection S_subchar_error;     // sub char domain error

int const MAXS = 8;
int const MAXL = 64;

//CLASSES
class Srep;
class Stringsize;
class String;
class Tmpstring;
class Substring;

extern Srep* nullrep;
extern Pool* Reelp;       // Pool for "reel" small strings
extern Pool* SPoolp;      // Pool for smallish strings

class Srep {         // String Representation
public:
    unsigned len;
    unsigned max;
    unsigned char refc;
    char str[MAXS];
    Srep()                      { len = 0; max = 0; refc = 0; }
    void rcdec()                { if(refc && --refc==0)  delete_srep(); }
    void rcinc()                { if(refc) refc++; }
    friend Srep *new_srep(unsigned);
    friend Srep *get_long(unsigned);
    void delete_srep();
};

class String_init {
    static int init_count;
public:
    String_init();
    ~String_init();
};

static String_init string_init;

class Stringsize {     // for declarations of large Strings
    unsigned int i;
    friend String;
public:
    Stringsize(unsigned int x)     { i = x; }
    ~Stringsize()                  { }
};
    
class String {        //  the String
    Srep *d;
    friend Tmpstring;
    friend Substring;
    String(Srep* r)                { d = r; }
public:
    String()                       { d = nullrep;}
    String(const char*);
    String(const char*,unsigned);
    String(const char c)           { d = new_srep(1); d->str[0]=c; }
    String(Stringsize s)           { d = new_srep(s.i); d->len=0;   }
    String(const String& s)        { d = s.d; d->rcinc(); }
    inline String(Tmpstring&);
    inline String(const Substring&);
    friend String int_to_str(int);
    operator __const char*() {
        if(d->len==0) return "";
        else {
            *this = Stringsize(d->len+1);
            d->str[d->len]='\0';
            return d->str;
        }
    }
    ~String()                       { d->rcdec(); }
    /* ASSIGNMENT */
    inline String& operator=(const String&);
    String& operator=(const char*);
    inline String& operator=(const char);
    String& operator=(Stringsize);
    /* APPENDING */
    String& operator+=(const String&);
    String& operator+=(const char*);
    String& operator+=(const char);
    String& put(const String& s)    { return *this += s; }
    String& put(const char* s)      { return *this += s; }
    String& put(const char c)       { return *this += c; }
    /* ADDITION */
    friend Tmpstring operator+(const String&,const String&);
    friend Tmpstring operator+(const String&,const char*);
    friend Tmpstring operator+(const String&,const char);
    friend Tmpstring operator+(const char*,const String&);
    friend Tmpstring operator+(const char,const String&);
    /* MISCELLANEOUS */
    unsigned length() __const       { return d->len; } 
    int is_empty() __const          { return d->len ? 0 : 1; } 
    inline char char_at(unsigned) __const;
    char& operator[](unsigned);
    Substring operator()(unsigned,unsigned); 
    Substring operator()(unsigned);
    String chunk(unsigned,unsigned) __const; 
    String chunk(unsigned) __const;
    int index(const String&) __const;
    int index(const char) __const;
    int hashval() __const;
    char* dump(char* s) __const     { memcpy(s,d->str,d->len); 
                                     s[d->len]='\0'; return s;}
    int getX(char&);
    int get()                       { char c; return getX(c); }
    int unputX(char&);
    int unput()                     { char c; return unputX(c); }
    String& unget(const char);
    String& unget(const String&);
    inline int firstX(char&) __const;            
    inline int lastX(char&) __const;
    /* COMPARISON OPERATORS */
    inline int compare(const String&) __const;
    inline int compare(const char*) __const;
    friend inline int operator==(const String&,const String&);
    friend inline int operator==(const String&,const char*);
    /* SYSTEM FUNCTIONS */
    friend int read(int, String&, int);
    friend int write(int fd, const String& s) {
#ifdef __cplusplus
        return write(fd, (const void*)s.d->str, s.d->len);
#else
        return write(fd, s.d->str, s.d->len);
#endif
    }
    friend puts(const String&);
    friend fputs(const String&, FILE*);
    /* STRING(3) FUNCTIONS */
    int	strchr(const char c) __const;
    int	strrchr(const char c) __const;
    int	strpbrk(const String&) __const;
    int	strspn(const String&) __const;
    int	strcspn(const String&) __const;
    /* MISCELLANEOUS */
    int match(const String&) __const;
    int match(const char *) __const;
    /* INPUT/OUTPUT */
#ifdef __cplusplus
    friend inline ostream& operator<<(ostream&,const String&);
#else
    friend ostream& operator<<(ostream&,const String&);
    friend ostream& operator<<(ostream&,const Substring&);
#endif
    friend istream& operator>>(istream&,String&);
    /* CASE FUNCTIONS */
    String upper() __const;
    String lower() __const;
};

class Tmpstring {  // Class of String temporaries for Addition
    Srep *d;
    char strung;
    friend String;
public:
    Tmpstring(Srep* r)                    { strung=0; d = r; }
    Tmpstring(const String& s)            { strung=0; d = s.d; s.d->rcinc(); }
    Tmpstring(const Tmpstring& t)         { strung=0; d = t.d; d->rcinc(); }
    Tmpstring& operator+(const String&);
    Tmpstring& operator+(const char*);
    Tmpstring& operator+(const char);
    ~Tmpstring()                          { if(!strung) d->rcdec(); }
#ifndef __cplusplus
    friend ostream& operator<<(ostream&,const Tmpstring&);
#endif
};

class Substring {
    friend String;
    Substring(const Substring&);
    String *ss;
    int oo;
    int ll;
    Substring(String &i,int o,int l) : ss(&i), oo(o), ll(l) { }
public:
    void operator=(const String&);
    void operator=(char* s);
    String *it() __const                   { return ss; }
    int offset() __const                   { return oo; }
    int length() __const                   { return ll; }
#ifndef __cplusplus
    friend ostream& operator<<(ostream&,const Substring&);
#endif
};

// inline functions

// CONSTRUCTOR STUFF

inline 
String::String(Tmpstring& t)   { t.strung=1; d = t.d; }

inline
String::String(const Substring& s)
{
    d = new_srep(s.ll);
    if(s.ll) memcpy(d->str,s.ss->d->str+s.oo,s.ll);
}

//ASSIGNMENT
inline String&
String::operator=(const String& r)
{
    r.d->rcinc();
    d->rcdec();
    d = r.d;
    return *this;
}

inline String&
String::operator=(const char c)
{
    if(d->refc!=1) {
        d->rcdec();
        d = new_srep(1);
    }
    d->str[0] = c;
    d->len = 1;
    return *this;
}

/* COMPARISON OPS */
inline int
String::compare(const String& s) __const
{
    register int dlen = d->len;
    register int slen = s.d->len;
    register int i = memcmp(d->str,s.d->str,dlen < slen ? dlen : slen);
    return i ? i : (dlen - slen);
}

inline int
String::compare(const char* s) __const
{
    register int dlen = d->len;
    register int ln = s ? strlen(s) : 0;
    register int i = memcmp(d->str,s, (dlen < ln ? dlen : ln));
    return i ? i : (dlen - ln);
}

inline int 
strcmp(const String& s,const String& t) {return s.compare(t);}
inline int 
strcmp(const String& s,const char* p) {return s.compare(p);}
inline int 
strcmp(const char* p,const String& t) {return -(t.compare(p));}

inline int 
compare(const String& s,const String& t) {return s.compare(t);}

inline int
operator==(const String& t,const String& s)
{
    register int dlen = t.d->len;
    return (dlen != s.d->len) ? 0 : !memcmp(t.d->str,s.d->str,dlen);
}
inline int
operator==(const String& t,const char* s)
{
    register int dlen = t.d->len;
    register slen = s ? strlen(s) : 0;
    return (dlen != slen) ? 0 : !memcmp(t.d->str,s,dlen);
}

inline int 
operator==(const char* s,const String& t)    { return t==s; }
inline int 
operator!=(const String& s,const String& t)  { return !(s==t); }
inline int 
operator!=(const String& t,const char* s)    { return !(t==s); }
inline int 
operator!=(const char* s,const String& t)    { return !(t==s); }
inline int 
operator>=(const String& s,const String& t)  { return s.compare(t)>=0; }
inline int 
operator>=(const String& t,const char* s)    { return t.compare(s)>=0; }
inline int 
operator>=(const char* s,const String& t)    { return t.compare(s)<=0; }
inline int 
operator>(const String& s,const String& t)   { return s.compare(t)>0; }
inline int 
operator>(const String& t,const char* s)     { return t.compare(s)>0; }
inline int 
operator>(const char* s,const String& t)     { return t.compare(s)<0; }
inline int 
operator<=(const String& s,const String& t)     { return s.compare(t)<=0; }
inline int 
operator<=(const String& t,const char* s)       { return t.compare(s)<=0; }
inline int 
operator<=(const char* s,const String& t)       { return t.compare(s)>=0; }
inline int 
operator<(const String& s,const String& t)      { return s.compare(t)<0; }
inline int 
operator<(const String& t,const char* s)        { return t.compare(s)<0; }
inline int 
operator<(const char* s,const String& t)        { return t.compare(s)>0; }

/* STRING(3) FUNCTIONS */
#ifdef __cplusplus
inline int 
strchr(const String& s,const char c)     { return s.strchr(c); }
inline int 
strrchr(const String& s,const char c)    { return s.strrchr(c); }
#endif
inline int 
strpbrk(const String& s,const String& t) { return s.strpbrk(t);}
inline int 
strpbrk(const char* s,const String& t)   { return String(s).strpbrk(t);}
inline int 
strpbrk(const String& s,const char* t)   { return s.strpbrk(t);}
inline int 
strspn(const String& s,const String& t)  { return s.strspn(t);}
inline int 
strspn(const char* s,const String& t)    { return String(s).strspn(t);}
inline int 
strspn(const String& s,const char* t)    { return s.strspn(t);}
inline int 
strcspn(const String& s,const String& t) { return s.strcspn(t);}
inline int 
strcspn(const char* s,const String& t)   { return String(s).strcspn(t);}
inline int 
strcspn(const String& s,const char* t)   { return s.strcspn(t);}

/* SUBCHAR STUFF */
inline char
String::char_at(unsigned i) __const
{
    if(i >= length()) {
        S_subchar_error.abort("i out of bounds in s.char_at(i)");
        return '\0';
    }
    else return d->str[i];
}

/* MISCELLANEOUS */
inline int
String::firstX(char &c) __const
{ 
    if(d->len >0 ) return (c=d->str[0],1);
    else return 0;
}
inline int
String::lastX(char &c) __const
{ 
    if(d->len >0 ) return (c=d->str[d->len - 1],1);
    else return 0;
}
inline int length(const String& s)       { return s.length(); }
inline int hashval(const String& s)      { return s.hashval(); }

/* I/O */
#ifdef __cplusplus
inline ostream&
operator<<(ostream& oo, const String& r)
{
    oo.write(r.d->str,r.d->len);
    return oo;
}
#endif
String sgets(istream&);
int fgets(String&, int, FILE*);
int gets(String&);

char* Memcpy(register char*,register const char*,int);

#endif
