#include "moods.h"
#include "memmng.h"

MemMng :: MemMng()
{
  segSize = Size = nSeg = 0;
  segments = NULL;
}

MemMng :: MemMng(void * buf, uint size, uint segs, OID * segOIDs, int bg)
{
  MemMng();
  Setup(buf, size, segs, segOIDs, bg);
}

MemMng :: ~MemMng()
{
  Destroy();
}

int MemMng :: Setup(byte * buf, uint size, uint segs, OID * segOIDs, int bg)
{
  int i;
  bufSeg * s;
  byte * p;

  if (segments)      // already allocated. First use Destroy().
    return ERROR;

  nSeg     = segs;
  segSize  = size / segs;
  Size     = size;
  bgIdx    = bg;
  segments = (bufSeg *) malloc(nSeg * sizeof(bufSeg));

  if (segments == NULL)     // short of memory
    return ERROR;

  for (i=0, p=buf, s=segments; i<nSeg; i++, p+=segSize, s++) {
    s->start = p;
    OID_Copy(s->oid, segOIDs[i]);
    s->curr  = 0;
    s->currInObj  = 0;
  }

  return OK;
}
 
void MemMng :: Destroy()
{
  if (!segments) return;
  free(segments);
  Size = nSeg = segSize = 0;
  segments = NULL;
}

int MemMng :: Write(uint segNo, byte *data, uint dSize)
{
  bufSeg *b;

  b = &segments[segNo];

  if (!isAvail(segNo, dSize)) {         // bucket full. Flush it on disk and continue.
     if(Flush(b) == ERROR)
       return ERROR;
  }

// space is available, just append it
  memcpy(b->start + b->curr, data, dSize);
  b->curr += dSize;
  return OK;
}

int MemMng :: Flush(bufSeg *b)
{
  USERDESC *uDesc;
  int e;

// Read the part of the object to be overwritten into the buffer pool.
/*
  e =  sm_ReadObject(bgIdx, &b->oid, b->currInObj, b->curr, &uDesc);
  if (e != esmNOERROR)
     return(ERROR);

// Write the data into the object. Starting offset is 0 since only the required part
// of the object is in the buffer.
  e =  sm_WriteObject(bgIdx, 0, b->curr, b->start, uDesc, TRUE);
  if (e != esmNOERROR)
     return(ERROR);
*/
  e = sm_AppendToObject(bgIdx,&b->oid,b->curr,b->start);
  if (e != esmNOERROR)
     return(ERROR);

  b->currInObj += b->curr;
  b->curr=0;
  return OK;

}

int MemMng :: FlushAll()
{
  int i;
  bufSeg * b;

  for (i=0, b=segments; i<nSeg; i++, b++)
    if (Flush(b) == ERROR)
       return ERROR;

  return OK;
}

int MemMng :: isAvail(uint segNo, uint freeSize)
{
  if (!segments) return false;
  return (segments[segNo].curr + freeSize <= segSize);
}

int MemMng :: oCount(uint segNo, uint oSize)
{
  return segments[segNo].currInObj / oSize;
}

