/*

    FUNCTIONS RELATED TO INDEX CREATION AND DELETION

*/
#include "smex.h"
#include "parse.h"
#include "system.h"
#include "query.h"
#include "error.h"
#include "allo.h"
#define INDGRPSIZE 4 

extern int e;
extern VOLID volid;
extern int queryGroup;

createIndex(char *treename, char *classname, char *attname)
/* 
   creates a b+-tree index by successive insertion
*/
{ 
	MOODSTYPE btype, atype;
	BOOL successfull;
	TYPEATT atts;
	OID   btreeoid;
	OBJHDR btreeobjhdr;
	BOOL eof, found = FALSE;
	USERDESC *userdesc;
	long attoffset, attlen;
	SMDATATYPE indtype;
	IID  indexid;
	KEY  akey;
	BTREEINFO anindex;
	int i, j, count=0;
	OID element;

	searchCatalog("BTREE", &btype, &successfull);
	if(!successfull){
		setError(INTERNAL_ERROR, 0, " there is no catalog object for BTREE");
		return(FALSE);
	}

	getAttributes(classname, &atype, &atts, &successfull);
	if(!successfull){
		setError(RUNTIME_ERROR, 0, "there is no class with this name");
		return(FALSE);
	}

	found = FALSE ;
	for(j=0; j<atts.no_of_atts ; j++) {
		if(!strcmp(attname, atts.contents[j].attname)){
			found = TRUE;
			i = j;
		}
	}

	if(!found){
		setError(RUNTIME_ERROR, 0, "there is no attribute of specified class with this name");
		return(FALSE);
	}


	if(atts.contents[i].constructor != NOCONS) {
		setError(RUNTIME_ERROR, 0, " the atribute is not a basic type ");
		return(FALSE);
	}
	else{
		attoffset = atts.contents[i].offset;
		attlen =    atts.contents[i].length;
	}

	if(!strcmp(atts.contents[i].classname,"String"))  indtype = SM_string;
	if(!strcmp(atts.contents[i].classname,"Integer")) indtype = SM_int;
	if(!strcmp(atts.contents[i].classname,"Float"))   indtype = SM_float;

	found = FALSE;
	if((e = sm_GetFirstOid(queryGroup, &btype.place, &btreeoid, &btreeobjhdr, &eof)) != esmNOERROR){
		setError(EXODUS_ERROR, e, " can not access BTREE file (createIndex)");
		return(FALSE);
	}

	while(!(eof || found)) {
		if((e = sm_ReadObject(queryGroup, &btreeoid, 0, READ_ALL, &userdesc)) != esmNOERROR) {
			setError(EXODUS_ERROR, e, "can not read object (createIndex) ");
			return(FALSE);
		}

		if(!strcmp(((BTREEINFO *) userdesc->basePtr)->treename, treename)) found = TRUE;

		if((e = sm_ReleaseObject(userdesc)) != esmNOERROR) {
			setError(EXODUS_ERROR, e, " can not release object(createIndex)");
			return(FALSE);
		}

		if((e = sm_GetNextOid(queryGroup, &btreeoid, &btreeoid, &btreeobjhdr, &eof)) != esmNOERROR){
			setError(EXODUS_ERROR, e, " can not access BTREE file(createIndex)");
			return(FALSE);
		}
	}

	if(found){
		setError(RUNTIME_ERROR, 0, " there exists an index with this name");
		return(FALSE);
	}

	if((e = sm_CreateIndex(volid, queryGroup, SM_BTREENDX, indtype, attlen, sizeof(OID), FALSE, &indexid)) != esmNOERROR) {
		setError(EXODUS_ERROR, e, " can not create index (createIndex)");
		return(FALSE);
	}

	akey.length = attlen;
	akey.valuePtr = (void *) myMalloc(attlen);
	if((e = sm_GetFirstOid(queryGroup, &atype.place, &btreeoid, &btreeobjhdr, &eof)) != esmNOERROR){
		setError(EXODUS_ERROR, e, " can not access class object(createIndex)");
		return(FALSE);
	}

	while(!eof) {

		if((e = sm_ReadObject(queryGroup, &btreeoid, 0, READ_ALL, &userdesc)) != esmNOERROR){
			setError(EXODUS_ERROR, e, " can not access class object (createIndex)");
			return(FALSE);
		}

		memcpy(akey.valuePtr, (void *) &userdesc->basePtr[attoffset], attlen);
		memcpy((void *) &element, (void *) &userdesc->oid, sizeof(OID));

		if((e = sm_InsertEntry(&indexid, queryGroup, &akey, (void *) &element)) != esmNOERROR) {
			setError(EXODUS_ERROR, e, " can not insert entry(createIndex)");
			return(FALSE);
		}

		if((e = sm_ReleaseObject(userdesc)) != esmNOERROR){
			setError(EXODUS_ERROR, e, " can not release object (createIndex) ");
			return(FALSE);
		}

		if((e = sm_GetNextOid(queryGroup, &btreeoid, &btreeoid, &btreeobjhdr, &eof)) != esmNOERROR){
			setError(EXODUS_ERROR, e, " can not access btree file (createIndex) ");
			return(FALSE);
		}

		count++;
	}

	strcpy(anindex.treename, treename);
	strcpy(anindex.classname, classname);
	strcpy(anindex.attname, attname);
	/* assuming that net page size for index is 3000 */
	anindex.leaves = count * (attlen + sizeof(OID)) / 3000;
	anindex.level = 2;
	anindex.v = 3000 / (2 *(attlen +sizeof(OID)));
	memcpy((void *) &anindex.indexid, (void *) &indexid, sizeof(IID));

	if((e = sm_CreateObject(queryGroup, &btype.place, NEAR_FIRST_PHYSICAL, NULL, NULL, sizeof(BTREEINFO), (void *) &anindex,
	    &btreeoid)) != esmNOERROR) {
		setError(EXODUS_ERROR, e, " can not create object (createIndex)");
		return(FALSE);
	}
	printf("\n index is created \n\n");
	return(TRUE);
}


getBtree(char *classname, char *attname, BTREEINFO *p, BOOL *success)
{
	MOODSTYPE btree;
	BOOL btreesuc, eof;
	SCANDESC *btreescanDesc;
	USERDESC *btreeuserDesc;
	*success = FALSE;
	int btreeGroup;
	searchCatalog("BTREE", &btree, &btreesuc);
	if(!btreesuc){
		setError(INTERNAL_ERROR, 0, " There is no catalog object for BTREE");
		return(FALSE);
	}

	if((e = sm_OpenBufferGroup(INDGRPSIZE, BF_MRU, &btreeGroup, NOFLAGS)) != esmNOERROR){
		setError(EXODUS_ERROR, e, "can not open buffer (getBtree)");
		return(FALSE);
	}

	if((e = sm_OpenScanWithGroup(&(btree.place), PHYSICAL_SCAN, btreeGroup, &btreescanDesc, NULL)) != esmNOERROR) {
		setError(EXODUS_ERROR, e, " BTREE file does not exist (getBtree)");
		return(FALSE);
	}

	if((e = sm_ScanNextObject(btreescanDesc, 0, READ_ALL, &btreeuserDesc, &eof)) != esmNOERROR){
		setError(EXODUS_ERROR, e, " can not get next object (getBtree) ");
		return(FALSE);
	}

	while( !(eof || *success)){
		if(!(strcmp( ( (BTREEINFO *) (btreeuserDesc->basePtr))->classname, classname) || 
		    strcmp( ( (BTREEINFO *) (btreeuserDesc->basePtr))->attname, attname))){
			memcpy((void *)p, (void *) btreeuserDesc->basePtr, sizeof(BTREEINFO));
			*success= TRUE;
		}
		else {
			if((e = sm_ScanNextObject(btreescanDesc, 0, READ_ALL, &btreeuserDesc, &eof)) != esmNOERROR){
				setError(EXODUS_ERROR, e, " can not get next object (getBtree)");
				return(FALSE);
			}
		}
	}

	if((e = sm_CloseScan(btreescanDesc)) != esmNOERROR) {
		setError(EXODUS_ERROR, e, "can not close scan (getBtree)");
		return(FALSE);
	}
	if((e= sm_CloseBufferGroup(btreeGroup)) != esmNOERROR) {
		setError(EXODUS_ERROR,  e, "can not close group (getBtree)");
		return(FALSE);
	}
	return(TRUE);
}

dropIndex(char *indexname)
{
	MOODSTYPE btree;
	BOOL btreesuc, eof;
	SCANDESC *btreescanDesc;
	USERDESC *btreeuserDesc;
	BOOL success = FALSE;
	OID boid;
	IID indexid;

	int btreeGroup;
	searchCatalog("BTREE", &btree, &btreesuc);
	if(!btreesuc){
		setError(INTERNAL_ERROR, 0, " There is no catalog object for BTREE");
		return(FALSE);
	}

	if((e = sm_OpenBufferGroup(INDGRPSIZE, BF_MRU, &btreeGroup, NOFLAGS)) != esmNOERROR){
		setError(EXODUS_ERROR, e, "can not open buffer (getBtree)");
		return(FALSE);
	}

	if((e = sm_OpenScanWithGroup(&(btree.place), PHYSICAL_SCAN, btreeGroup, &btreescanDesc, NULL)) != esmNOERROR) {
		setError(EXODUS_ERROR, e, " BTREE file does not exist (getBtree)");
		return(FALSE);
	}

	if((e = sm_ScanNextObject(btreescanDesc, 0, READ_ALL, &btreeuserDesc, &eof)) != esmNOERROR){
		setError(EXODUS_ERROR, e, " can not get next object (getBtree) ");
		return(FALSE);
	}

	while( !(eof || success)){
		if(!(strcmp( ( (BTREEINFO *) (btreeuserDesc->basePtr))->treename, indexname))){
			memcpy((void *) &boid, &btreeuserDesc->oid, sizeof(OID));
			memcpy((void *) &indexid,&(((BTREEINFO *)  btreeuserDesc->basePtr)->indexid), sizeof(IID));
			success= TRUE;
		}
		else {
			if((e = sm_ScanNextObject(btreescanDesc, 0, READ_ALL, &btreeuserDesc, &eof)) != esmNOERROR){
				setError(EXODUS_ERROR, e, " can not get next object (getBtree)");
				return(FALSE);
			}
		}
	}

	if(!success) {
		setError(RUNTIME_ERROR, 0, "there is no index with this name");
		return(FALSE);
	}

	if((e = sm_CloseScan(btreescanDesc)) != esmNOERROR) {
		setError(EXODUS_ERROR, e, "can not close scan (getBtree)");
		return(FALSE);
	}
	if((e= sm_CloseBufferGroup(btreeGroup)) != esmNOERROR) {
		setError(EXODUS_ERROR,  e, "can not close group (getBtree)");
		return(FALSE);
	}
	if(!success) {
		setError(RUNTIME_ERROR, 0, "there is no index with this name");
		return(FALSE);
	}

	if((e = sm_DestroyIndex(&indexid, queryGroup)) != esmNOERROR){
		setError(EXODUS_ERROR, e, "can not destroy index");
		return(FALSE);
	}

	if((e = sm_DestroyObject(queryGroup, &boid)) != esmNOERROR){
		setError(EXODUS_ERROR, e, "can not destroy object");
		return(FALSE);
	}

	printf("\n index is dropped \n\n");

	return(TRUE);
}



