#include "mood_catalog.h"
#include <lists.h>
#include <set.h>

extern ERTYPE ErrorCheck(int errcode, char *msg ); // prototype of error handling routine ....


//           ************** Set::Set *****************
//  Constructor for a known Set ( defined previously ) 
//  Just get the set in to memory if exists, otherwise 
//  Set the PStatus variable to Error Code ....
//  IN  >>>    PSetOid   Set object's OID to get the object ...
//  OUT <<<    PTypeId   The Object's Type ID 
//  OUT <<<    PSize     The Size of Object
//  OUT <<<    PStatus   
//                       if 0 OUT ->set exists & loaded.
//                       if 1 OUT ->set does not exist.
//          ******************************************

Set::Set(OID *PSetOid,TYPEID *PTypeID,SIZTYP *PSize, CatBOOLEAN *PStatus) {
     // Copy Some important values to member variables ,,,,,,

     int EsmError;

     BufGrp   = BufferGroup; 
     Tid      = TransactionId; 
     SetOid   = *PSetOid;
     FileId   = FileIdentifier;
     ListPos  = 0;
     
     //  Test if it Works !!!!!
     //  Read requested Set into memory....

     EsmError = sm_ReadObject(BufGrp,&SetOid,0,READ_ALL,&UserDesc);

     if(EsmError != esmNOERROR) { 
        *PStatus   = EsmError;
        *PSize     = 0;
        PTypeID    = NULL;
        ErrorCheck(EsmError,"Set consrtuctor, getting ...\n");
     } else { 
        *PStatus=0;
        Size      = UserDesc->objectSize;
        Buffer    = malloc(Size);
        memcpy(Buffer,UserDesc->basePtr,Size);
        ListHead  = (OID *) (UserDesc->basePtr + S_TypeId);
        PTypeID   = Buffer ; 
        Size      = Size - S_TypeId;
        Size      = Size / S_OID; 
        *PSize     = Size;
        //cout << "size of read set= " << Size << "\n" ;
        unpin();
     }
     TypeId = MOODSSET;
}

//           ************** Set::Set *****************
//  Constructor for Set creation 
//  the Set is created which has only a TypeId Field filled with MOODSSET value. 
//  Set the PStatus variable to Error Code ....
//  IN  >>>  PFid      File Id for the placement of the new object.  
//  IN  >>>  PPos      Position dependency to given PNearOid. 
//  OUT <<<  PStatus   returned from constructor . 0 for success, error# otherwise.
//           ******************************************

Set::Set(FID *PFid,NTYPE PPos,OID *PNearOid,CatBOOLEAN *PStatus) {
     // Copy Some important values to member variables ,,,,,,

     int EsmError;
     void * buffer;
     OBJHDR ObjHdr;
     TYPEID type  = MOODSSET;

     BufGrp   = BufferGroup; 
     Tid      = TransactionId; 

     // Create a Set object with no elements in file ................
     // Save   Set's Object Id in member variable for its lifetime...
     
     buffer =(void *) new TYPEID;
     memcpy(buffer,&type,S_TypeId);

     TypeId = MOODSSET;
     if (PNearOid)
        EsmError = sm_CreateObject(BufGrp,PFid,PPos,PNearOid,NULL, S_TypeId,buffer,&SetOid);
     else
        EsmError = sm_CreateObject(BufGrp,PFid,PPos,NULL,NULL, S_TypeId,buffer,&SetOid);


     if(EsmError != esmNOERROR) {
        *PStatus = EsmError ; 
        ErrorCheck(EsmError,"creating set...\n"); }
        else {
        *PStatus = 0;
       }
     Size = 0;
     ListHead = NULL;
     free(buffer);
}
/*
Set::Set()
{}

Set::createSet( (FID *PFid,NTYPE PPos,OID *PNearOid,CatBOOLEAN *PStatus )
{
     // Copy Some important values to member variables ,,,,,,
 
     int EsmError;
     void * buffer;
     TYPEID type  = MOODSSET;
 
     BufGrp   = BufferGroup;
     Tid      = TransactionId;
 
     // Create a Set object with no elements in file ................
     // Save   Set's Object Id in member variable for its lifetime...
 
     buffer =(void *) new TYPEID;
     memcpy(buffer,&type,S_TypeId);
 
     TypeId = MOODSSET;
     EsmError = sm_CreateObject(BufGrp,PFid,PPos,PNearOid,NULL, S_TypeId,buffer,&SetOid);
 
 
     if(EsmError != esmNOERROR) {
        *PStatus = EsmError ;
        ErrorCheck(EsmError,"creating set...\n"); }
        else {
        *PStatus = 0;
       }
     Size = 0;
}
*/


void Set::getNext(OID *POid ,CatBOOLEAN *PStatus) {

       if( (ListPos+1) < Size ) {
           memcpy(POid, &ListHead[ ++ListPos ],sizeof(OID));;
           *PStatus = 0;
       } else *PStatus = 1;
}

void Set::getPrev(OID *POid,CatBOOLEAN *PStatus) {

       if( ListPos > 0 ) {
         memcpy(POid , &ListHead[ --ListPos ],sizeof(OID));;
         *PStatus = 0;
       } else *PStatus = 1;
}


ERTYPE Set::insertToSet(OID *POid,CatBOOLEAN *PStatus) {

int ListIndex, Offset = 0;
int err;
OID *CurOid;

     BufGrp   = BufferGroup; 
     Tid      = TransactionId; 
     FileId   = FileIdentifier;

     // unpin();

     // Find Correct Place To Insert Oid ..............

     if ( Size != 0 ) { 
        for ( ListIndex = 0; ListIndex < Size; ListIndex++ ) {
              // display_Oid(ListHead[ ListIndex ]);
              if ( !Test_Oid( ListHead[ ListIndex ] , *POid , T_LT ) ) Offset = ListIndex + 1 ;
              if ( Test_Oid( ListHead[ ListIndex ] , *POid , T_EQ ) ) { 
                   *PStatus = 0;
                   return(0);
               }
        }


       Offset = Offset * S_OID + S_TypeId ;


       err    = sm_InsertInObject(BufGrp,&SetOid,Offset,S_OID,(void *)POid);
       if (err != esmNOERROR ) ErrorCheck(err,"Insert in Object \n");
       Size   = Size + 1;
       return(0);   
    } else {
       err    = sm_AppendToObject(BufGrp,&SetOid,S_OID,(void *)POid);
       if (err != esmNOERROR ) ErrorCheck(err,"Insert in Object \n");
       err    = sm_ReadObject(BufGrp,&SetOid,0,READ_ALL,&UserDesc);
       if (err != esmNOERROR ) ErrorCheck(err,"Insert in Object \n");
       ListHead  = (OID *) (UserDesc->basePtr + S_TypeId);
       Buffer    = UserDesc->basePtr ;
       unpin();
       Size   = Size + 1;
       return(0);   
   } 
}

ERTYPE Set::deleteFromSet(OID POid,CatBOOLEAN *PStatus) {
int ListIndex, Offset ;
int err;
OID *CurOid;

       //  Find Correct Place To delete Oid ..............

       unpin();
       
       Offset = -1; // set to -1 to check if the element already exists in the set....

       for ( ListIndex=0; ListIndex <= Size; ListIndex++ ) {
             if ( Test_Oid( ListHead[ListIndex] , POid , T_EQ ) ) Offset=ListIndex ;
       }        

       if(Offset == -1) {  // element not in the set .....
          *PStatus = 1; 
          return(0);       // Escape from routine.........
       } ; 

       Offset = Offset * S_OID + S_TypeId ; // Offset#of elements + TYPE of set.....
       err    = sm_DeleteFromObject(BufGrp,&SetOid,Offset,S_OID);
       Size   = Size - 1 ;

       ErrorCheck(err,"Delete From Object \n");

       // Delete below line please....

       *PStatus = 0; 
       return(0);   
}

ERTYPE Set::isIn(OID POid,CatBOOLEAN *PStatus) {
int ListIndex, Offset ;

       //  Find Object in set..............
    
       display_Oid(POid);
       *PStatus = 1 ;     // For the worst case !! .......

       for ( ListIndex=0; ListIndex < Size; ListIndex++ ) 
             if ( Test_Oid( ListHead[ListIndex] , POid , T_EQ ) ) *PStatus=0;

       //  Anything to ask ?...

       return(0);
}

ERTYPE Set::getElement(OID *POid , int SetIndex ,CatBOOLEAN *PStatus) {

       if ( SetIndex > Size ) {  // If wanted more than set  has ......
            *PStatus = 1 ;
            return (0) ;
       }

       memcpy(POid, &ListHead[ SetIndex ],sizeof(OID));;
       *PStatus = 0 ;            // Everything Seems ok................
       return(0);

}

ERTYPE Set::setUnion(Set PSet,Set USet,CatBOOLEAN *PStatus) { 

int Setdex;
OID Element;

    if ( USet.Size ) {                   // Union Set is not empty ..............
         *PStatus = 1 ;
          return(0);  
    }

    /*      
         Insert first the elements of current set 
                                                  */

    for (Setdex = 0 ; Setdex <  Size ; Setdex++ )  {


         getElement( &Element, Setdex, PStatus ) ;

         if ( !( *PStatus ) ) {                      // If element received from current set..
             display_Oid(Element);
             USet.insertToSet( &Element , PStatus );

         } 
         else {                                       // If not received ........
             *PStatus = 1;
         }
        

    }

    /* 
         Now Insert Other set's elements ...
                                                 */

    for (Setdex = 0 ; Setdex < PSet.Size ; Setdex++ )  {

         PSet.getElement( &Element, Setdex, PStatus)  ;

         if ( !( *PStatus ) ) {                    // If element received from second  set.....

             USet.insertToSet( &Element , PStatus ) ;

         } 
         else {
             *PStatus = 1 ;
         }
    }

return( 0 );

}
 
ERTYPE Set::isSubset(Set PSet,CatBOOLEAN *PStatus) {
int  SetIndex;
OID  Element;

     SetIndex=0;
    
     if ( Size > PSet.Size )  {       // If subset, how can it be larger !... 
        *PStatus = 1;
        return(0);
     }
                                      // Check all elements in current set if
    do {                              // They are in the target..........
        getElement(&Element,SetIndex++,PStatus);
        PSet.isIn(Element,PStatus);
    } while ( ( SetIndex < Size)  && ( ! (*PStatus)  ) );

    return(0);
}

ERTYPE Set::isContain(Set PSet, CatBOOLEAN *PStatus) {
int SetIndex ;
OID Element ;


    SetIndex = 0;

    if ( Size <= PSet.Size ) {    // If PSet is larger no need to check...
       *PStatus =  1;
       return(0);                 // Not an ESM error.....................
    }

    PSet.getFirst(&Element,PStatus);

    if ((*PStatus)) {             // If No elements in current set as can be seen !..
       return(0);                 // Not an ESM error.....................
    }

    isIn(Element,PStatus);

    for (SetIndex = 1; SetIndex < PSet.Size ; SetIndex++) {

        PSet.getNext(&Element,PStatus); 

        isIn(Element,PStatus);

        if (*PStatus) return(0);

     } 
    
return(0);                        // No way to get an ESM error !!!...

}

/*ERTYPE ErrorCheck(int e,char *routine) {

   if(e!=esmNOERROR) {                        // If there is not error do nothing..
     printf("Got a ESM error from %s \n",routine);
     printf("error= %s \n",sm_Error(sm_errno));
     exit(1);                                 // No need to to do this, but for test.
   }
   return(0);
}*/


OID Set::getSetOid(CatBOOLEAN *PStatus) {

    if ( OID_IS_INVALID(SetOid) )  // Test if program cheats......
       *PStatus = 1 ;
    else 
       *PStatus = 0;

    return(SetOid);               // May be changed to our standard.!
}

ERTYPE Set::getFirst(OID *POid,CatBOOLEAN *PStatus) { 

       if (!Size) {     // There is no set ! or at least not in memory....
          *PStatus = 1;
          POid=NULL;
          return(0);
       }
       memcpy(POid,ListHead,sizeof(OID));// Original point, elements start...(see set def.).
       *PStatus = 0;
       return(0);

}

ERTYPE Set::getLast(OID *POid,CatBOOLEAN *PStatus) {

       memcpy(POid , &ListHead[Size],sizeof(OID));
       *PStatus  = 0;           // Think why need  this ?....
       return(0);               // Tell me if you find out !!...

}

void Set::intersect(Set PSet,Set ISet,CatBOOLEAN *PStatus) {
       
int Setindex;
OID Element;

        if ( Size <= PSet.Size ) {
             for ( Setindex = 0 ; Setindex < Size ; Setindex ++ ) {
                   getElement ( &Element , Setindex , PStatus ) ;
                   PSet.isIn  ( Element  , PStatus );
                   if ( ! (*PStatus ) ) 
                        ISet.insertToSet ( &Element , PStatus );
             }
             *PStatus = 0;
        } 
        else {
             for ( Setindex = 0 ; Setindex < PSet.Size ; Setindex ++ ) {
                   PSet.getElement ( &Element , Setindex , PStatus ) ;
                   isIn  ( Element  , PStatus );
                   if ( ! (*PStatus ) ) 
                        ISet.insertToSet ( &Element , PStatus );
             }
             *PStatus = 0;
        }
}
