/**********************************************************************
* MoodView ,The Graphical User Interface To MOODS
* Copyright (c) 1994 Scientific and Technical Research Council of Turkiye
*                    Software Research and Development Center, Ankara
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE SOFTWARE RESEARCH AND DEVELOPMENT CENTER ALLOWS FREE USE OF THIS
* SOFTWARE IN ITS "AS IS" CONDITION.
* THE CENTER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The MOODS Project Group requests users of this software to return
* any improvements or extensions that they make to:
*
*   MOODS Project Group
*   TUBITAK Software Research and Development Center
*   Department of Computer Engineering
*   Middle East Technical University
*   Ankara, 06531
*
*        or budak@karakoy.srdc.metu.edu.tr
*
* In addition, the MOODS Project Group requests that users grant the
* Software Research and Development Center rights to redistribute these
* changes.
**********************************************************************/

#include "mood_catalog.h"
#include "userreq.h"
#include <stdio.h>
#include <stdlib.h>
#include <Xm/MainW.h>
#include <Xm/DrawingA.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/CascadeB.h>
#include <Xm/Frame.h>
#include <Xm/SeparatoG.h>
#include <Xm/MessageB.h>
#include <Xm/Separator.h>
#include <Mrm/MrmAppl.h>
#include <X11/cursorfont.h>
#include <malloc.h>
#include "libMv.h"
#include "chb.h"
#include "wid.h"
#include "m_class.h"

extern int    language;
extern Widget main_window,drawing_area;
extern Widget **button;
extern vertex **tree;
extern UserRequest *userRequest;
extern char UidPath[100];

Widget              toplevel;
Display             *dpy;
Window              win;
XmStringCharSet     charset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
static char         *uid_file[1];
static MrmHierarchy Mrmdata;
static MrmType      *dummy_class;
typedef struct mine *Pmine;

void        recreate_drawing_area();
static void NewdbAcceptCB(Widget,Widget,XtPointer);
static void NewdbCancelCB(Widget,Widget,XtPointer);
void        create_help();
Widget      CreateHelp(Widget parent);
Widget      CreateVersion(Widget parent);


/*-------------------------------------------------------------
** init_screen - Make initialization on the screen
*------------------------------------------------------------*/
void init_screen(int argc,char *argv[], XtAppContext *app)
{
  Arg       args[5];
  GC        gc;
  XGCValues gcv;
  int       n;
  Widget    main_w, separator, rowcol, main_window2;

  /* Get uil path */
  uid_file[0] = (char *) malloc(100);
  strcpy( uid_file[0], UidPath);
  if (language == ENGLISH)
     strcat( uid_file[0],"/bro.uid");
  if (language == GERMAN) 
     strcat( uid_file[0],"/bro2.uid");

  /* Prepare widgets */
  MrmInitialize();

   toplevel = XtVaAppInitialize(app,"Demos",NULL,0,&argc,argv,NULL,
             XmNwidth,730,XmNheight,530,NULL); 

  rowcol = XtVaCreateWidget("rowcol", xmRowColumnWidgetClass,
        toplevel,
        XmNorientation, XmVERTICAL,
        NULL);

  MrmOpenHierarchy( XtNumber(uid_file), uid_file,
                     NULL, &Mrmdata );
  MrmFetchWidget(Mrmdata,"main_widg",
       rowcol,
       &main_w,
       (void *) &dummy_class );
  XtManageChild(main_w);
  free(uid_file[0]);
 
  separator = XtVaCreateWidget ("sep1",xmSeparatorWidgetClass,
                rowcol,
                XmNorientation , XmHORIZONTAL,
                NULL);
  XtManageChild(separator);

  /* Create main window.*/
  n = 0;
  XtSetArg(args[n],XmNwidth,600);n++;
  XtSetArg(args[n],XmNheight,400);n++;
  XtSetArg(args[n],XmNshadowThickness,5);n++;
  XtSetArg(args[n],XmNshowSeparator, True);  n++;
  XtSetArg(args[n],XmNscrollingPolicy, XmAUTOMATIC);  n++;
  XtSetArg(args[n], XmNtopAttachment,  XmATTACH_WIDGET); n++; 
  XtSetArg(args[n], XmNtopWidget, separator); n++;
  XtSetArg(args[n], XmNbottomAttachment,  XmATTACH_FORM); n++;
  main_window = XmCreateMainWindow (rowcol, "main", args, n);

  create_help();

  n =0;
  XtSetArg(args[n],XmNwidth,3900);n++;
  XtSetArg(args[n],XmNheight,3900);n++;
  XtSetArg(args[n],XmNresizePolicy, XmNONE);n++;
  // XtSetArg(args[n],XmNresizePolicy, XmNONE);n++;
  XtSetArg(args[n],XmNbackground,GetPixel("LightSlateBlue"));n++;
  XtSetArg(args[n],XmNforeground,GetPixel("red"));n++;
  drawing_area = XmCreateDrawingArea(main_window,"drawing_area",args,n);
  XtManageChild(drawing_area);

  /* When scrolled, the drawing area will get expose events
     add callback for all mouse and keyboard input events */

  if (argc == 3) //Db selected
      XtAddCallback(drawing_area,XmNexposeCallback,(XtCallbackProc) draw_lines,NULL);
  gcv.foreground = BlackPixelOfScreen(XtScreen(drawing_area));
  gc = XCreateGC(XtDisplay(drawing_area),RootWindowOfScreen(XtScreen(drawing_area)),
                 GCForeground, &gcv);

  XtVaSetValues(drawing_area, XmNuserData, gc, NULL);

  XtManageChild(main_window);
  XtManageChild(rowcol);
}

/*-------------------------------------------------------------
** draw_onscreen - Creates frames and buttons and puts on drawing area
*------------------------------------------------------------*/
void draw_onscreen()
{
   Widget frame;
   Arg args[8];
   XmString label;
   int i,j,n,x,y;

   for(i=0;tree[i][0].id;i++)
      for(j=0;tree[i][j].id;j++)
      {
         if(strcmp(tree[i][j].name,"du"))
         {
            n = 0;
            XtSetArg(args[n],XmNshadowType,XmSHADOW_OUT);n++;
            XtSetArg(args[n],XmNshadowThickness,4);n++;
            XtSetArg(args[n],XmNx,tree[i][j].x);n++;
            XtSetArg(args[n],XmNy,tree[i][j].y);n++;

            frame = XmCreateFrame(drawing_area,"frame",args,n);
            XtManageChild(frame);

            label=XmStringCreateLtoR(tree[i][j].name,charset);

            n = 0;
            XtSetArg(args[n],XmNbackground,GetPixel("LightSteelBlue"));n++;
            XtSetArg(args[n],XmNforeground,GetPixel("red"));n++;
            XtSetArg(args[n],XmNhighlightColor,GetPixel("red"));n++;
            XtSetArg(args[n],XmNshadowThickness,3);n++;
            XtSetArg(args[n],XmNlabelString,label);n++;
            XtSetArg(args[n],XmNborderWidth,1);n++;
            XtSetArg(args[n],XmNheight,25);n++;

            button[i][j] = XmCreatePushButton(frame,tree[i][j].name,args,n);
            XtManageChild(button[i][j]);
            tree[i][j].wid = (unsigned) button[i][j];
            XtAddCallback(button[i][j],XmNactivateCallback,
                          (XtCallbackProc) buttonCB,button[i][j]);
       }
   }
   XmStringFree(label);
}


/*-------------------------------------------------------------
** buttonCB - Creates buttons on the drawing area.
*------------------------------------------------------------*/
void buttonCB(Widget w,Widget client_data,XmDrawingAreaCallbackStruct *cbs)
{
  Widget buttong,popup; 
  vertex *ver;
  unsigned wid;
  Pmine pass;

  ChangeCursor(toplevel, XC_gumby);
  popup = XmCreatePopupMenu(drawing_area, "popup", NULL, 0);
  
  if (language == ENGLISH)
     XtCreateManagedWidget("Update Class",xmLabelGadgetClass,popup,NULL,0);
  if (language == GERMAN)
     XtCreateManagedWidget("Klasse Aendern",xmLabelGadgetClass,popup,NULL,0);

  XtCreateManagedWidget("Sep",xmSeparatorGadgetClass,popup,NULL,0);

  wid = (unsigned)client_data;
  ver = find_vertex(wid);
  
  pass = (Pmine) malloc(sizeof(struct mine));
  strcpy(pass->clname, ver->name);
  pass->flag = 0; 

  if (language == ENGLISH) {
    buttong = XtCreateManagedWidget("Show Class",xmPushButtonGadgetClass,popup,NULL,0);
    XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) ClassCB,pass);
    buttong = XtCreateManagedWidget("Add a Subclass",xmPushButtonGadgetClass,popup,NULL,0);
    XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) AddSubClassCB,client_data);
    buttong = XtCreateManagedWidget("Drop This Class",xmPushButtonGadgetClass,popup,NULL,0);
    XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) DropClassCB,client_data);
    buttong = XtCreateManagedWidget("Change Name",xmPushButtonGadgetClass,popup,NULL,0);
    XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) RenameClassCB,client_data);
  }
  if (language == GERMAN) {
    buttong = XtCreateManagedWidget("Klasse Anzeigen",xmPushButtonGadgetClass,popup,NULL,0);
    XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) ClassCB,pass);
    buttong = XtCreateManagedWidget("Subklasse Hinzufuegen",xmPushButtonGadgetClass,popup,NULL,0);
    XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) AddSubClassCB,client_data);
    buttong = XtCreateManagedWidget("Diese Klasse entfernen ",xmPushButtonGadgetClass,popup,NULL,0);
    XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) DropClassCB,client_data);
    buttong = XtCreateManagedWidget("Namen Aendern",xmPushButtonGadgetClass,popup,NULL,0);
    XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) RenameClassCB,client_data);
  }


  XmMenuPosition(popup,(XButtonPressedEvent *)(cbs->event));
  XtManageChild(popup); 
  ChangeCursor(toplevel, XC_hand2);
}

/*-------------------------------------------------------------
**  GetPixel - Convert string to pixel
*------------------------------------------------------------*/
XtArgVal GetPixel(char *colorstr)
{
   XrmValue from, to;

   from.size = strlen(colorstr) +1;

   if (from.size < sizeof(String)) from.size = sizeof(String);

   from.addr = colorstr;
   to.addr = NULL;
   XtConvert(toplevel, XmRString, &from, XmRPixel, &to);

   return ((XtArgVal) *((XtArgVal *) to.addr));
}

/*-------------------------------------------------------------
**  create_help - Creates help menu bar on the screen
*------------------------------------------------------------*/

void create_help()
{
   Widget menu_bar,menu_pane,buttong,cascade;
   Arg args[4];
   int n;

  /*
   *  Create menu bar in main window.
   */
   menu_bar = XmCreateMenuBar (main_window, "menu_bar", NULL, 0);
   XtManageChild (menu_bar);

  /*
   *  Create "Actions" pulldown menu.
   */
   menu_pane = XmCreatePulldownMenu (menu_bar,"menu_pane",NULL,0);

   if (language == ENGLISH) {
     buttong = XtCreateManagedWidget("Open",xmPushButtonWidgetClass,menu_pane,NULL,0);
     XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) opendbCB, NULL);
     buttong = XtCreateManagedWidget("New",xmPushButtonWidgetClass,menu_pane,NULL,0);
     XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) newdbCB, NULL);
     buttong = XtCreateManagedWidget("Delete",xmPushButtonWidgetClass,menu_pane,NULL,0);
     XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) deletedbCB, NULL);
     buttong = XtCreateManagedWidget("Quit",xmPushButtonWidgetClass,menu_pane,NULL,0);
     XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) quitCB, NULL);
   }
   if (language == GERMAN) {
     buttong = XtCreateManagedWidget("Oeffnen",xmPushButtonWidgetClass,menu_pane,NULL,0);
     XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) opendbCB, NULL);
     buttong = XtCreateManagedWidget("Neu",xmPushButtonWidgetClass,menu_pane,NULL,0);
     XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) newdbCB, NULL);
     buttong = XtCreateManagedWidget("Loeschen",xmPushButtonWidgetClass,menu_pane,NULL,0);
     XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) deletedbCB, NULL);
     buttong = XtCreateManagedWidget("Beenden",xmPushButtonWidgetClass,menu_pane,NULL,0);
     XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) quitCB, NULL);
   }

   n = 0;
   XtSetArg (args[n], XmNsubMenuId, menu_pane);  n++;
   if (language == ENGLISH)
      cascade = XmCreateCascadeButton (menu_bar, "Database ", args, n);
   if (language == GERMAN)
      cascade = XmCreateCascadeButton (menu_bar, "Datenbank", args, n);
   XtManageChild (cascade);

  /*
   *  Create "Help" pulldown menu.
   */
   menu_pane = XmCreatePulldownMenu (menu_bar, "menu_pane", NULL, 0);

   if (language == ENGLISH)
      buttong = XmCreatePushButton (menu_pane, "Help", NULL, 0);
   if (language == GERMAN)
      buttong = XmCreatePushButton (menu_pane, "Hilfe", NULL, 0);
   XtManageChild (buttong);
   XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) HelpCB, NULL);
   if (language == ENGLISH)
     buttong = XtCreateManagedWidget("Program Version",xmPushButtonWidgetClass,
                                   menu_pane,NULL,0);
   if (language == GERMAN)
     buttong = XtCreateManagedWidget("Programmversion",xmPushButtonWidgetClass,
                                   menu_pane,NULL,0);
   XtAddCallback (buttong, XmNactivateCallback, (XtCallbackProc) VersionCB, NULL);

   n = 0;
   XtSetArg (args[n], XmNsubMenuId, menu_pane);  n++;
   if (language == ENGLISH)
      cascade = XmCreateCascadeButton (menu_bar, "Help", args, n);
   if (language == GERMAN)
      cascade = XmCreateCascadeButton (menu_bar, "Hilfe", args, n);
   XtManageChild (cascade);

   n = 0;
   XtSetArg (args[n], XmNmenuHelpWidget, cascade);  n++;
   XtSetValues (menu_bar, args, n);

}

/*-------------------------------------------------------------
**      HelpCB                  - callback for help buttong
*-------------------------------------------------------------*/
void HelpCB (Widget w, XtPointer client_data, XtPointer call_data)
{
   Widget message_box;     /*  message box*/

  /*
   * Create help window.
   */
   message_box = CreateHelp (w);

  /* Display help window.
   */
   XtManageChild (message_box);
}

/*-------------------------------------------------------------
**      CreateHelp              - create help window
*------------------------------------------------------------*/
Widget CreateHelp(Widget parent)
{
   Widget          buttong;
   Widget          message_box;    /*  Message Dialog      */
   Arg             args[20];       /*  arg list            */
   static char     message[512];   /*  help text           */
   XmString        title_string = NULL;
   XmString        message_string = NULL;
   XmString        button_string = NULL;
   int n;

  /*      Generate message to display.
  */
   if (language == ENGLISH)
      sprintf (message, "\n\n\n Not available with this version. \n\0");
   if (language == GERMAN)
      sprintf (message, "\n\n\n In dieser version nicht verfuegbar. \n\0");

   message_string = XmStringCreateLtoR (message, charset);
   button_string = XmStringCreateLtoR ("Close", charset);
   title_string = XmStringCreateLtoR ("Class Hiearchy Browser Help", charset);


   /*      Create message box dialog.
   */
   n = 0;
   XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
   XtSetArg (args[n], XmNokLabelString, button_string);  n++;
   XtSetArg (args[n], XmNmessageString, message_string);  n++;
   message_box = XmCreateMessageDialog (parent, "helpbox", args, n);
   buttong = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
   XtUnmanageChild (buttong);
   buttong = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
   XtUnmanageChild (buttong);

  /*   Free strings and return message box.
   */
   if (title_string) XtFree ((char *)title_string);
   if (message_string) XtFree ((char *)message_string);
   if (button_string) XtFree ((char *)button_string);

   return (message_box);
}

/*-------------------------------------------------------------
**  VersionCB      - callback for help button
*------------------------------------------------------------*/
void VersionCB (Widget w,XtPointer client_data,XtPointer call_data)
{
  Widget    message_box;    /*  message box    */


  /*  Create message window. */
  message_box = CreateVersion (w);


  /*  Display version window. */
  XtManageChild (message_box);
}

/*-------------------------------------------------------------
**  CreateVersion    - create version window
*------------------------------------------------------------*/
Widget CreateVersion (Widget parent)
{
  Widget    buttong;
  Widget    message_box;  /*  Message Dialog   */
  XmString  title_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;
  Arg       args[6];
  static char  message[]="Class Hierachy Browser, Version 1.0, 1994";
  static char  message2[]="Klassenhierarchie Browser, Version 1.0, 1994";
  int n;

  /* Create the compound strings */
  message_string = XmStringCreateLtoR (message, charset);
  if (language == ENGLISH) {
     button_string = XmStringCreateLtoR ("Close", charset);
     title_string = XmStringCreateLtoR ("Class Hiearchy Browser Version", charset);
  }
  if (language == GERMAN) {
     button_string = XmStringCreateLtoR ("Schliessen", charset);
     title_string = XmStringCreateLtoR ("Klassenhierarchie Browser Version", charset);
  }
  

  /*  Create message box dialog. */
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNokLabelString, button_string);  n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (parent, "versionbox", args, n);

  buttong = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild (buttong);
  buttong = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (buttong);


  /*  Free strings and return message box. */
  if (title_string) XtFree ((char *)title_string);
  if (message_string) XtFree ((char *)message_string);
  if (button_string) XtFree ((char *)button_string);
  return (message_box);
}

/*-------------------------------------------------------------
** opendbCB - Use a database
*------------------------------------------------------------*/
void opendbCB (Widget w,XtPointer client_data,XtPointer call_data)
{
 /* List database names, 0=Display Mode */
 listdb(0);
}

/*-------------------------------------------------------------
** newdbCB - Collect information for a new database
*------------------------------------------------------------*/
void newdbCB (Widget w,XtPointer client_data,XtPointer call_data)
{
 XmString title_string, mes_string;
 int n=0;
 Arg args[2];
 Widget prompt;

 /* Dialog for a new database */
 if (language ==ENGLISH) {
  title_string=XmStringCreateLtoR("New Database",charset);
  mes_string=XmStringCreateLtoR("New Database Name: ",charset);
 }
 if (language == GERMAN) {
  title_string=XmStringCreateLtoR("Neuer Datenbank",charset);
  mes_string=XmStringCreateLtoR("Neuer Name: ",charset);
 }

 n = 0;
 XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
 XtSetArg (args[n], XmNselectionLabelString, mes_string);  n++;
 prompt = XmCreatePromptDialog (w, "NewDatabase", args, n);
 XtManageChild (prompt);
 XtAddCallback (prompt, XmNokCallback,(XtCallbackProc)NewdbAcceptCB,client_data);
 XtAddCallback (prompt, XmNcancelCallback, (XtCallbackProc)NewdbCancelCB, NULL);
 n = 0;
 XtSetArg (args[n], XmNx, 120);  n++;
 XtSetArg (args[n], XmNy, 150);  n++;
 XtSetValues (XtParent(prompt), args, n);
 XmStringFree(title_string);
 XmStringFree(mes_string);
}

/*-------------------------------------------------------------
** NewdbAcceptCB - OK for a new database
*------------------------------------------------------------*/
static void NewdbAcceptCB(Widget w,Widget client_data,XtPointer call_data)
{
 char *newname;
 char sqlstm[30];

 XmSelectionBoxCallbackStruct *scb = (XmSelectionBoxCallbackStruct *)call_data;
 XmStringGetLtoR(scb->value, charset,&newname);

 /* Popdown the new database dialog*/
 XtDestroyWidget(w);  

 /* SQL... Create a new database */
 strcpy(sqlstm, "create database ");
 strcat(sqlstm, newname);
 userRequest->executeMSQL(sqlstm);
 if ((userRequest->operationType == HANDLE_ERROR) &&
     (userRequest->userError.errorType != NO_ERROR)) {
        char *msg = userRequest->getErrorText();
        Display_Error(w, msg);
        free(msg);
  } else { /* Success */

        /* SQL... Use the new database */
        strcpy(sqlstm, "use database ");
        strcat(sqlstm, newname);
        userRequest->executeMSQL(sqlstm);

        /* Refresh the screen */
        update_chb();
        dispose();
        dag_algorithm();
        recreate_drawing_area();
        draw_onscreen();
        XtRealizeWidget(toplevel);
        dpy = XtDisplay(drawing_area);
        win = XtWindow(drawing_area);

 //       char *msg = userRequest->getMessageText();
 //       strcpy(msg, "Database created succesfully.");
 //       Display_Warning(toplevel, msg);
 //       free(msg);

  }
  userRequest->reset();
}

/*-------------------------------------------------------------
** NewdbCancelCB - Cabcel new db creation
*------------------------------------------------------------*/
static void NewdbCancelCB(Widget w,Widget client_data,XtPointer call_data)
{
 /* Popdown the new database dialog */
 XtDestroyWidget(w);  
}

/*-------------------------------------------------------------
** deletedbCB - Delete db
*------------------------------------------------------------*/
void deletedbCB (Widget w,XtPointer client_data,XtPointer call_data)
{
 /* List database names */
 listdb(1);
}

/*-------------------------------------------------------------
**  quitCB - callback for quit button
*------------------------------------------------------------*/
void quitCB (Widget w,XtPointer client_data,XtPointer call_data)
{
  /* Quit from browser */
  dispose();
  userRequest->closeESM();
  delete userRequest;
  exit (0);
}

