#include "mood_catalog.h"
#include "mood_parser.h"
#include <stdio.h>
#include <Xm/PushBG.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/DrawingA.h>
#include <Xm/SelectioB.h>
#include "chb.h"
#include "wid.h" 

#define GERMAN  0
#define ENGLISH 1

extern int language; 
extern FID CatalogID;
extern UserRequest *userRequest;
extern XmStringCharSet charset;
extern Widget toplevel;
extern Display  *dpy;
extern Window    win;
extern Widget drawing_area;
extern void recreate_drawing_area();

/*-------------------------------------------------------------
** DialogRenameClassAcceptCB - Rename the Class
*------------------------------------------------------------*/
void DialogRenameClassAcceptCB (Widget w,Widget client_data,XtPointer call_data)
{
    XmString label;
    Arg  args[2];
    vertex *ver;
    unsigned wid;
    char tmp[10];
    char *newname;
    int n;
 
    XmSelectionBoxCallbackStruct *scb = (XmSelectionBoxCallbackStruct *)call_data;
    /* Get the New Class Name string from the Rename Class prompt box*/
    XmStringGetLtoR(scb->value, charset,&newname);
    XtDestroyWidget(w);  /* Popdown the file selection box*/

    n = 0;
    label = XmStringCreateLtoR (newname, charset);
    XtSetArg(args[n],XmNlabelString,label);n++;
    XtSetValues(client_data,args,n);
  
   /* Get the Vertex that holds the class information
      and change the class name */ 
    wid = (unsigned)client_data;
    ver = find_vertex(wid);

   /*
    * CAT... Rename Class.
    */
    MoodsTypes my;
    userRequest->startXAct();
    userRequest->parseText();
    userRequest->handleOperation();
    my.changeNameOfClass(ver->name, newname);
    userRequest->commitXAct();
    strcpy(ver->name,newname);
   
    dispose();
    dag_algorithm();
    recreate_drawing_area();
    draw_onscreen();
    XtRealizeWidget(toplevel);
    dpy = XtDisplay(drawing_area);
    win = XtWindow(drawing_area);

}

/*-------------------------------------------------------------
**  DialogRenameClassHelpCB    - create dialog help window
*------------------------------------------------------------*/
void DialogRenameClassHelpCB (Widget w, XtPointer client_data, XtPointer call_data)
{
  Widget    button;
  Widget    message_box;  /*  Message Dialog   */
  Arg       args[4];  /*  Arg list    */
  int n = 0;

  static char  opendlog_help[100];  /*  Help text for new dialog */
  XmString  title_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;

  /*  Generate help message for new dialog */
  sprintf (opendlog_help, "\
You must move the pointer into the text window and click BSelect \n\
before you enter the New Class Name.  After you enter the New Class Name \n\
press Return or move the pointer to 'OK' and click BSelect.  \0");

  message_string = XmStringCreateLtoR (opendlog_help, charset);
  button_string = XmStringCreateLtoR ("Close", charset);
  title_string  = XmStringCreateLtoR ("Rename Class Help", charset);

  /*  Create message box dialog.
  */
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNokLabelString, button_string);  n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (w, "helpbox", args, n);
  XtManageChild (message_box);

  button = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtDestroyWidget (button);
  button = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtDestroyWidget (button);

  /*  Free strings and return message box.
  */
  if (title_string) XmStringFree (title_string);
  if (message_string) XmStringFree (message_string);
  if (button_string) XmStringFree (button_string);
}

/*-------------------------------------------------------------
**  DialogRenameClassCancelCB
**  Process callback from Dialog cancel actions.
*------------------------------------------------------------*/
void DialogRenameClassCancelCB (Widget w,Widget client_data,XtPointer call_data)
{
  /* No action Needed */
}

/*-------------------------------------------------------------
**  RenameClassCB  - Used for Renaming of Classes
*------------------------------------------------------------*/
void RenameClassCB (Widget w,Widget client_data,XtPointer call_data)
{
   char *name;
    Arg args[6];
    Widget prompt;
    extern XmStringCharSet charset;
    XmString title_string;
    XmString mes_string;
    vertex *ver;
    int n=0;
    unsigned wid;

    if (language == ENGLISH) {
      title_string=XmStringCreateLtoR("Rename Class",charset);
      mes_string=XmStringCreateLtoR("New Class Name : ",charset);
    }
    if (language == GERMAN) {
      title_string=XmStringCreateLtoR("Namen Aendern",charset);
      mes_string=XmStringCreateLtoR("Neuer Klassenname: ",charset);
    }

    n = 0;
    XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
    XtSetArg (args[n], XmNselectionLabelString, mes_string);  n++;
    prompt = XmCreatePromptDialog (client_data, "RenameClass", args, n);
    XtManageChild (prompt);

    XtAddCallback (prompt, XmNokCallback,(XtCallbackProc)DialogRenameClassAcceptCB,client_data);
    XtAddCallback (prompt, XmNokCallback,(XtCallbackProc)clearCB,client_data);
    XtAddCallback (prompt, XmNcancelCallback, (XtCallbackProc)DialogRenameClassCancelCB, NULL);
    XtAddCallback (prompt, XmNhelpCallback, (XtCallbackProc)DialogRenameClassHelpCB,NULL);

    n = 0;
    XtSetArg (args[n], XmNx, 120);  n++;
    XtSetArg (args[n], XmNy, 150);  n++;
    XtSetValues (XtParent(prompt), args, n);

    XmStringFree(title_string);
    XmStringFree(mes_string);
}


/*-------------------------------------------------------------
**  DialogWarningDropClassHelpCB    - create dialog help window
*------------------------------------------------------------*/
void DialogWarningDropClassHelpCB (Widget w,XtPointer client_data,XtPointer call_data)
{
  Widget    button;
  Widget    message_box;  /*  Message Dialog   */
  Arg       args[4];  /*  Arg list    */
  int n = 0;

  static char  opendlog_help[56];  /*  Help text for new dialog */
  XmString  title_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;

  /*  Generate help message for new dialog */
  sprintf (opendlog_help, "\
 If you want to drop this class and its all sub classes,\n\
You must move the pointer to the 'OK' and click BSelect.\n\
Otherwise move the pointer to the 'Cancel' and click Bselect.\0");

  message_string = XmStringCreateLtoR (opendlog_help, charset);
  button_string = XmStringCreateLtoR ("Close", charset);
  title_string  = XmStringCreateLtoR ("Drop Class Help", charset);

  /*  Create message box dialog.
  */
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNokLabelString, button_string);  n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (w, "helpbox", args, n);
  XtManageChild (message_box);

  button = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild (button);
  button = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (button);

  /*  Free strings and return message box.
  */
  if (title_string) XmStringFree (title_string);
  if (message_string) XmStringFree (message_string);
  if (button_string) XmStringFree (button_string);
}

/*-------------------------------------------------------------
**  DialogWarningDropClassCancelCB
**    Process callback from Dialog cancel actions.
*------------------------------------------------------------*/
void DialogWarningDropClassCancelCB (Widget w,Widget client_data,XtPointer call_data)
{
   /* No action Needed */
}


/*-------------------------------------------------------------
**  control_drop_level_accept - 
*------------------------------------------------------------*/
void control_drop_level_acceptCB(Widget w,XtPointer client_data,XtPointer call_data)
{
  /* No action */
}

/*-------------------------------------------------------------
**  control_drop_level_helpCB    - create dialog help window
*------------------------------------------------------------*/
void control_drop_level_helpCB (Widget w,XtPointer client_data,XtPointer call_data)
{
  Widget    button;
  Widget    message_box;  /*  Message Dialog   */
  Arg       args[4];  /*  Arg list    */
  int n = 0;

  static char  opendlog_help[256];  /*  Help text for new dialog */
  XmString  title_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;

  /*  Generate help message for new dialog */
  sprintf (opendlog_help, "\
 You cannot Drop Classes more then 4 level.\n  \n\
 If You are sure to drop more than 4 level deep,\n\
then drop classes recursively.Now, move the \n\
pointer to the 'OK' and click BSelect.\n\
Otherwise move the pointer to the 'Cancel' and click Bselect.\0");

  message_string = XmStringCreateLtoR (opendlog_help, charset);
  button_string = XmStringCreateLtoR ("Close", charset);
  title_string  = XmStringCreateLtoR ("Drop Class Help", charset);

  /*  Create message box dialog.
  */
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNokLabelString, button_string);  n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (w, "helpbox", args, n);
  XtManageChild (message_box);

  button = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild (button);
  button = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (button);

  /*  Free strings and return message box.
  */

  if (title_string) XmStringFree (title_string);
  if (message_string) XmStringFree (message_string);
  if (button_string) XmStringFree (button_string);
}

/*------------------------------------------------------------
** control_drop_level
** Make warning message for dropping of more than 4 level deep. 
*------------------------------------------------------------*/
void control_drop_level(Widget client_data,vertex *ver1,vertex *ver2)
{
 int n,i,k;
  Arg args[4];
  XmString title_string;
  XmString mes_string;
  Widget warn;
  
      mes_string=XmStringCreateLtoR(" You cannot Drop Classes more than 4 Level Deep !" ,charset);
      title_string=XmStringCreateLtoR("Warning",charset);

     n = 0;
     XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
     XtSetArg (args[n], XmNmessageString, mes_string);  n++;
     warn = XmCreateWarningDialog(client_data, "Warning", args, n);
     draw_lines(); 
     XtManageChild (warn);

     XtAddCallback(warn,XmNokCallback,control_drop_level_acceptCB,NULL);
     XtAddCallback(warn,XmNcancelCallback,(XtCallbackProc) DialogWarningDropClassCancelCB,NULL);
     XtAddCallback(warn,XmNhelpCallback,control_drop_level_helpCB,NULL);

     n = 0;
     XtSetArg (args[n], XmNx, 320);  n++;
     XtSetArg (args[n], XmNy, 150);  n++;
     XtSetValues (XtParent(warn), args, n);

     XmStringFree(title_string);
     XmStringFree(mes_string);
}

/*-------------------------------------------------------------
** DialogWarningDropClassAcceptCB - Make warning Dialog
*------------------------------------------------------------*/
void DialogWarningDropClassAcceptCB(Widget w,Widget client_data,XtPointer call_data)
{
  vertex *ver;
  int cnt;
  unsigned wid = (unsigned) client_data;

  ver = find_vertex(wid);
  cnt = ver->id;

   while(ver->no_of_child){
      vertex *ver1;
      int cnt1 = ver->child[0];

      ver1 = find_vertex_id(cnt1);

         while(ver1->no_of_child){
           vertex *ver2;
           int  cnt2 = ver1->child[0];

           ver2 = find_vertex_id(cnt2);

              while(ver2->no_of_child){
                 vertex *ver3;  
                 int cnt3 = ver2->child[0];

                  ver3 = find_vertex_id(cnt3);

                     while(ver3->no_of_child){
                        vertex *ver4;  
                        int cnt4 = ver3->child[0];
                         
                        ver4 = find_vertex_id(cnt4);
 
                           while(ver4->no_of_child){
                              vertex *ver5; 
                              int cnt5 = ver4->child[0];
                              int lev ; 
                              ver5 = find_vertex_id(cnt5); 
                              lev = ver5->level - ver->level;
                              if(lev>4)
                                {
                                control_drop_level(client_data,ver,ver5);
                                goto exit_loop;
                                }
                              else del_button(cnt5);  
                           } 
                         del_button(cnt4);
                      }
                  del_button(cnt3);
              }
         del_button(cnt2);
         } 
   del_button(cnt1);
   }
   del_button(cnt);
  
  exit_loop:;
}

/*-------------------------------------------------------------
** DropClassCB - Drop Class Dialog 
*------------------------------------------------------------*/
void DropClassCB(Widget w,Widget client_data,XtPointer call_data)
{
  XmString title_string;
  XmString mes_string;
  Widget warn;
  Arg args[4];
  vertex *ver; 
  unsigned id,wid = (unsigned) client_data;
  int n;

  ver = find_vertex(wid);

  /* If the pressed class has no child, drop that class; otherwise, 
     create Warning Dialog to ask drop all subclasses or not */

  
    title_string=XmStringCreateLtoR("Warning",charset);

    if (language == ENGLISH) {
    if(!ver->no_of_child) 
        mes_string=XmStringCreateLtoR(" Are You Sure to Drop this Class ?",charset); 
    else
        mes_string=XmStringCreateLtoR("  This Class has Subclasses!\n             \n Are you sure to drop all Subclasses ?",charset);
    }
    if (language == GERMAN) {
    if(!ver->no_of_child)
        mes_string=XmStringCreateLtoR(" Wollen Sie diese Klasse wirklich loeschen?",charset);
    else
        mes_string=XmStringCreateLtoR("  Diese Klasse hat Subklassen!\n             \n 
Wollen Sie wirklich alle Subklassen loeschen?",charset);
    }
  
     n = 0;
     XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
     XtSetArg (args[n], XmNmessageString, mes_string);  n++;
     warn = XmCreateWarningDialog(client_data, "Warning", args, n);
     XtManageChild (warn);
    
    XtAddCallback(warn,XmNokCallback,(XtCallbackProc) DialogWarningDropClassAcceptCB,client_data);
    XtAddCallback(warn, XmNokCallback,(XtCallbackProc) clearCB,NULL);
    XtAddCallback(warn, XmNokCallback,(XtCallbackProc) draw_lines,NULL);
    XtAddCallback(warn, XmNcancelCallback,(XtCallbackProc) DialogWarningDropClassCancelCB,NULL);
    XtAddCallback(warn,XmNhelpCallback,(XtCallbackProc) DialogWarningDropClassHelpCB,NULL);
 
    n = 0;
    XtSetArg (args[n], XmNx, 320);  n++;
    XtSetArg (args[n], XmNy, 150);  n++;
    XtSetValues (XtParent(warn), args, n);

    XmStringFree(title_string);
    XmStringFree(mes_string);
}

