/**********************************************************************
* MoodView ,The Graphical User Interface To MOODS
* Copyright (c) 1994 Scientific and Technical Research Council of Turkiye
*                    Software Research and Development Center, Ankara
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE SOFTWARE RESEARCH AND DEVELOPMENT CENTER ALLOWS FREE USE OF THIS
* SOFTWARE IN ITS "AS IS" CONDITION.
* THE CENTER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The MOODS Project Group requests users of this software to return
* any improvements or extensions that they make to:
*
*   MOODS Project Group
*   TUBITAK Software Research and Development Center
*   Department of Computer Engineering
*   Middle East Technical University
*   Ankara, 06531
*
*        or budak@karakoy.srdc.metu.edu.tr
*
* In addition, the MOODS Project Group requests that users grant the
* Software Research and Development Center rights to redistribute these
* changes.
**********************************************************************/

#include "mood_catalog.h"
#include "mood_parser.h"
#include "userreq.h"
#include <stdio.h>
#include <sys/types.h>
#include <Mrm/MrmAppl.h>
#include <Xm/RowColumn.h>
#include <Xm/Frame.h>
#include <Xm/LabelG.h>
#include <Xm/XmP.h>
#include <Xm/Separator.h>
#include <Xm/MainW.h>
#include <X11/cursorfont.h>
#include "first.h"
#include "libMv.h"

char 		UidPath[100];
UserRequest     *ur;
UserRequest     *userRequest;
char            opendb[100];
Widget          toplevel, toplist;
XmStringCharSet charset = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
char            language[50];
static char     * uid_filenames1[1];
static char     * uid_filenames2[1];


void        list_db();
static void dblist_proc(Widget list_w, XtPointer client_data, XmListCallbackStruct *cbs);

main(int argc,char *argv[])
{
   char *ResPath;

    Widget first_rowcol;
    Widget frame_rowcol, f_frame1, f_frame2, f_frame3, f_frame4;
    Widget f_frame5, f_frame6, f_frame7;
    Widget main_w1, main_w2, separator;
    WidgetClass mclass = xmLabelGadgetClass;

    if (argc < 2) {
        printf("Unsifficient arguments!\n");
        printf("Please Enter Language Mode: german or english \n");
        exit(0);
    }
  
    strcpy(language, argv[1]);
    strcpy(opendb, "\0");

     ResPath = getenv("MOOD_BIN");
     if (!ResPath) {
        printf("\nMOOD_BIN environment variable is not set !\n");
        exit(1);
     }

    sprintf(UidPath, "%s/gui/uil",ResPath);
    uid_filenames1[0] = (char *) malloc(100);
    uid_filenames2[0] = (char *) malloc(100);
    strcpy( uid_filenames1[0], UidPath);
    strcpy( uid_filenames2[0], UidPath);
    if (strcmp (language, "english") == 0)
        strcat( uid_filenames1[0],"/first.uid");
    if (strcmp (language, "german") == 0)
        strcat( uid_filenames1[0],"/first2.uid");
    strcat( uid_filenames2[0],"/buttons.uid");

    MrmInitialize();

    toplevel = XtVaAppInitialize(&app, "Demos",
        NULL, 0, &argc, argv, NULL, NULL);
    
    first_rowcol = XtVaCreateWidget("first", xmRowColumnWidgetClass,
        toplevel,
        XmNorientation, XmVERTICAL,
        NULL);

    MrmOpenHierarchy( XtNumber(uid_filenames1), uid_filenames1,
                     NULL, &Mrmdata );
    MrmFetchWidget(Mrmdata,"main_w1",
       first_rowcol,
       &main_w1,
       (short int *)&dummy_class );
    XtManageChild(main_w1);

    separator = XtVaCreateWidget ("sep",xmSeparatorWidgetClass,
                first_rowcol,
                XmNorientation , XmHORIZONTAL,
                XmNleftAttachment , XmATTACH_FORM,
                XmNrightAttachment , XmATTACH_FORM,
                NULL);
    XtManageChild(separator);
    
    MrmOpenHierarchy( XtNumber(uid_filenames2), uid_filenames2,
                     NULL, &Mrmdata );
    /*
     *  Register callback routines
     */

    if (MrmRegisterNames (callbackvec, callbacknum)
                        != MrmSUCCESS)
                            printf("can't register callback names\n");

    MrmFetchWidget(Mrmdata,"main_w2",
       first_rowcol,
       &main_w2,
       (short int *)&dummy_class );
    XtManageChild(main_w2);

   /* Make a RowColumn to contain all the Frames */
   frame_rowcol = XtVaCreateWidget("framerowcolumn",
        xmRowColumnWidgetClass, first_rowcol,
        XmNorientation, XmHORIZONTAL,
        NULL);
   f_frame1 = XtVaCreateManagedWidget("f_frame1",
        xmFrameWidgetClass, frame_rowcol,
        XmNshadowType,      XmSHADOW_ETCHED_IN,
        NULL);
        XtVaCreateManagedWidget("Browser", mclass, f_frame1, NULL);
   f_frame2 = XtVaCreateManagedWidget("f_frame2",
        xmFrameWidgetClass, frame_rowcol,
        XmNshadowType,      XmSHADOW_ETCHED_IN,
        NULL);
   if (strcmp(language, "english") == 0)
        XtVaCreateManagedWidget("Queries", mclass, f_frame2, NULL);
   if (strcmp(language, "german")  == 0)
        XtVaCreateManagedWidget("Abfragen", mclass, f_frame2, NULL);
   f_frame3 = XtVaCreateManagedWidget("f_frame3",
        xmFrameWidgetClass, frame_rowcol,
        XmNshadowType,      XmSHADOW_ETCHED_IN,
        NULL);
        XtVaCreateManagedWidget("DBS", mclass, f_frame3, NULL);
   f_frame4 = XtVaCreateManagedWidget("f_frame4",
        xmFrameWidgetClass, frame_rowcol,
        XmNshadowType,      XmSHADOW_ETCHED_IN,
        NULL);
   if (strcmp(language, "english") == 0)
        XtVaCreateManagedWidget("Utilities", mclass, f_frame4, NULL);
   if (strcmp(language, "german")  == 0)
        XtVaCreateManagedWidget("Diensetpr.", mclass, f_frame4, NULL);
   f_frame5 = XtVaCreateManagedWidget("f_frame5",
        xmFrameWidgetClass, frame_rowcol,
        XmNshadowType,      XmSHADOW_ETCHED_IN,
        NULL);
   if (strcmp(language, "english") == 0)
        XtVaCreateManagedWidget("Help", mclass, f_frame5, NULL);
   if (strcmp(language, "german")  == 0)
        XtVaCreateManagedWidget("Hilfe", mclass, f_frame5, NULL);
   f_frame6 = XtVaCreateManagedWidget("f_frame6",
        xmFrameWidgetClass, frame_rowcol,
        XmNshadowType,      XmSHADOW_ETCHED_IN,
        NULL);
        XtVaCreateManagedWidget("Rtree", mclass, f_frame6, NULL);
        XtManageChild(frame_rowcol);
   f_frame7 = XtVaCreateManagedWidget("f_frame7",
        xmFrameWidgetClass, frame_rowcol,
        XmNshadowType,      XmSHADOW_ETCHED_IN,
        NULL);
   if (strcmp(language, "english") == 0)
        XtVaCreateManagedWidget("Exit", mclass, f_frame7, NULL);
   if (strcmp(language, "german")  == 0)
        XtVaCreateManagedWidget("Beenden", mclass, f_frame7, NULL);
    XtManageChild(frame_rowcol);
    XtManageChild(first_rowcol);
    XtRealizeWidget(toplevel);
    ChangeCursor(toplevel, XC_hand2);
    XtAppMainLoop(app);
}

void list_db()
{
   char        *DbNameList[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
   ListItem    *p;
   int         i=0;
   int         j;
   Widget      rowcol, list_w, text_w, label;
   Widget      main_wid;
   XtAppContext  app;
   XmString      str_list[20];

   ur = new UserRequest();
   ur->initESM("mood");
   
   /*
    * CAT... Get database list from Kernel.
    */
    ur->executeMSQL("list database");
    memset(DbNameList, 0, sizeof(char *)*20);
    for (p = ((ListDatabase *)ur->operation)->dbList->firstItem; p; p=p->next){
         DbNameList[i] = (char *) malloc(20);
         sprintf(DbNameList[i++], "%s", (char *)p->item);
    }

   /*
    * Prepare widgets
    */
    toplist = XtVaCreatePopupShell(NULL, topLevelShellWidgetClass,toplevel,
            XmNallowShellResize, True,
            XmNdeleteResponse,   XmUNMAP,
            NULL);
    main_wid = XtVaCreateManagedWidget("main_wid", xmMainWindowWidgetClass, toplist,
        NULL);
    rowcol = XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, main_wid, NULL);
    if (strcmp (language, "english") == 0)
        label = XtVaCreateManagedWidget("Databases:", xmLabelGadgetClass, rowcol,
        XmNrightAttachment,  XmATTACH_FORM,
        NULL);
    if (strcmp (language, "german") == 0)
        label = XtVaCreateManagedWidget("Datenbank:", xmLabelGadgetClass, rowcol,
        XmNrightAttachment,  XmATTACH_FORM,
        NULL);

    for (j = 0; j < i; j++)
        str_list[j] = (XmString)XmStringCreateLtoR(DbNameList[j], charset);
    list_w = XmCreateScrolledList(rowcol, "list", NULL, 0);
    XtVaSetValues(list_w,
      XmNitems,            str_list,
      XmNitemCount,        i,
      XmNvisibleItemCount, 7,
      XmNselectionPolicy,  XmEXTENDED_SELECT,
      NULL);
    XtAddCallback(list_w, XmNextendedSelectionCallback,(XtCallbackProc)dblist_proc, NULL);
    XtAddCallback(list_w, XmNdefaultActionCallback,(XtCallbackProc)dblist_proc, NULL);
    XtManageChild(list_w);
    for (j = 0; j < i; j++)
        XmStringFree(str_list[j]);
    XtFree((char *)str_list);
    XtManageChild(rowcol);
    XtPopup(toplist, XtGrabNone);

    ur->closeESM(); 
}

static void dblist_proc(Widget list_w, XtPointer client_data, XmListCallbackStruct *cbs)
{
  char *text;

  XmStringGetLtoR(cbs->selected_items[0], charset, &text);
  strcpy(opendb, text);
  XtDestroyWidget(toplist);
}  

static void execute_prog(Widget widget, char *tag, 
                           XmAnyCallbackStruct *callback_data )
{
  char browstm[150];

  ChangeCursor(toplevel, XC_gumby);
  switch(*tag) {
         case 'B':
             if (strcmp(opendb, "\0") == 0)  // No db selected
               Display_Warning(toplevel, "No database selected yet!");
             sprintf(browstm, "%s/",getenv("MOOD_BIN"));
             strcat(browstm, "browser ");
             if (strcmp(opendb, "\0") != 0) { // db selected
                 strcat(browstm, opendb);
                 strcat(browstm, " ");
             }
             strcat(browstm, language);
             strcat(browstm, " &"); 
             system(browstm);
             break; 
         case 'Q':
             if (strcmp(opendb, "\0") == 0)  // No db selected
               Display_Warning(toplevel, "No database selected yet!");
             sprintf(browstm, "%s/",getenv("MOOD_BIN"));
             strcat(browstm, "query ");
             if (strcmp(opendb, "\0") != 0) { // db selected
               strcat(browstm, opendb);
               strcat(browstm, " ");
             }            
             strcat(browstm, language);
             strcat(browstm, " &");
             system(browstm);
             break; 
         case 'G':
             list_db();        
             break; 
         case 'U':
             break; 
         case 'R':
             break; 
         case 'H':
             break; 
         case 'E':
             free(opendb);
             exit(0);
  } 
  ChangeCursor(toplevel, XC_hand2);
}
