/**********************************************************************
* MoodView ,The Graphical User Interface To MOODS
* Copyright (c) 1994 Scientific and Technical Research Council of Turkiye
*                    Software Research and Development Center, Ankara
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE SOFTWARE RESEARCH AND DEVELOPMENT CENTER ALLOWS FREE USE OF THIS
* SOFTWARE IN ITS "AS IS" CONDITION.
* THE CENTER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The MOODS Project Group requests users of this software to return
* any improvements or extensions that they make to:
*
*   MOODS Project Group
*   TUBITAK Software Research and Development Center
*   Department of Computer Engineering
*   Middle East Technical University
*   Ankara, 06531
*
*        or budak@karakoy.srdc.metu.edu.tr
*
* In addition, the MOODS Project Group requests that users grant the
* Software Research and Development Center rights to redistribute these
* changes.
**********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <Cursor.h>
#include <Xm/BulletinB.h>
#include <Xm/CascadeB.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/MainW.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrollBar.h>
#include <Xm/ScrolledW.h>
#include "me.h"
#include <userreq.h>


/*-------------------------------------------------------------
**  Global Variables
*------------------------------------------------------------*/
extern Widget fn_main_window;
extern Widget fn_toplevel,fn_frame,fn_BBoard;
extern Widget fn_menu_bar;
int AttributeShowed;

int find_len(BULK_SCHEMA *bs)
{
  int i,len = 0;
 
  AttributeShowed = 0;
  for (i = 0 ; i < bs->arity; i++) {
      if ((strstr(bs->attr[i]->name,"typeId")) || 
            (strstr(bs->attr[i]->name,"self")))
           continue;
      AttributeShowed ++;
      switch(bs->attr[i]->attr_type) {
        case REAL : if (strlen(bs->attr[i]->name)<15)
                       len += 15;
                    else
                       len += strlen(bs->attr[i]->name);
                    break;
        case INTEGER: if (strlen(bs->attr[i]->name)<8)
                         len += 8;
                      else
                         len += strlen(bs->attr[i]->name);
                    break;
        case TEXT :
                    len += 20;
                    break;
        case REF :
        case SET :
        case LIST :
        case SIGN_VOID :
        case DB_TYPE :
                    len += 20;
       }
  }
   return (len);
}


/*-------------------------------------------------------------
**  fn_GetPixel - Convert string to pixel
*------------------------------------------------------------*/
XtArgVal fn_GetPixel(char *colorstr)
{
   XrmValue from, to;

   from.size = strlen(colorstr) +1;

       if (from.size < sizeof(String))
           from.size = sizeof(String);

   from.addr = colorstr;
   to.addr = NULL;
   XtConvert(fn_toplevel, XmRString, &from, XmRPixel, &to);

   return ((XtArgVal) *((XtArgVal *) to.addr));
}

/*-------------------------------------------------------------
**  create_help - Creates help menu bar on the screen
*------------------------------------------------------------*/

void fn_create_help()
{
   Widget menu_pane,buttong,cascade;
   Arg args[4];
   int n;

   //Create menu bar in main window.

   fn_menu_bar = XmCreateMenuBar (fn_main_window, "fn_menu_bar", NULL, 0);
   XtManageChild (fn_menu_bar);

   // Create "Actions" pulldown menu.
   menu_pane = XmCreatePulldownMenu (fn_menu_bar,"menu_pane",NULL,0);

   buttong = XtCreateManagedWidget("Quit",xmPushButtonWidgetClass,menu_pane,NULL,0);
   XtAddCallback (buttong, XmNactivateCallback, fn_quitCB, NULL);

   n = 0;
   XtSetArg (args[n], XmNsubMenuId, menu_pane);  n++;
   cascade = XmCreateCascadeButton (fn_menu_bar, "Exit", args, n);
   XtManageChild (cascade);

   // Create "Help" pulldown menu.
   menu_pane = XmCreatePulldownMenu (fn_menu_bar, "menu_pane", NULL, 0);

   buttong = XmCreatePushButton (menu_pane, "Help", NULL, 0);
   XtManageChild (buttong);
   XtAddCallback (buttong, XmNactivateCallback, fn_HelpCB, NULL);

   buttong = XtCreateManagedWidget("Program Version",xmPushButtonWidgetClass,menu_pane,NULL,0);
   XtAddCallback (buttong, XmNactivateCallback, fn_VersionCB, NULL);

   n = 0;
   XtSetArg (args[n], XmNsubMenuId, menu_pane);  n++;
   cascade = XmCreateCascadeButton (fn_menu_bar, "Help", args, n);
   XtManageChild (cascade);

   n = 0;
   XtSetArg (args[n], XmNmenuHelpWidget, cascade);  n++;
   XtSetValues (fn_menu_bar, args, n);
}

/*-------------------------------------------------------------
**      HelpCB                  - callback for help buttong
*-------------------------------------------------------------*/
void fn_HelpCB (Widget w,XtPointer client_data,XtPointer call_data)
{
   Widget message_box; //message box

   // Create help window.
   message_box = fn_CreateHelp (w);

   //      Display help window.
   XtManageChild (message_box);
}

/*-------------------------------------------------------------
**      CreateHelp              - create help window
*------------------------------------------------------------*/
Widget fn_CreateHelp(Widget parent)
{
   Widget          buttong;
   Widget          message_box;
   Arg             args[20];
   static char     message[512];
   XmString        title_string = NULL;
   XmString        message_string = NULL;
   XmString        button_string = NULL;
   int n;

   //      Generate message to display.
    sprintf (message, "\n\n\n Not available with this version. \n\0");
    message_string = XmStringCreateLtoR (message, charset);
    button_string = XmStringCreateLtoR ("Close", charset);
    title_string = XmStringCreateLtoR ("MOODSQL Displayer Help", charset);

   //      Create message box dialog.
   n = 0;
   XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
   XtSetArg (args[n], XmNokLabelString, button_string);  n++;
   XtSetArg (args[n], XmNmessageString, message_string);  n++;
   message_box = XmCreateMessageDialog (parent, "helpbox", args, n);
   buttong = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
   XtUnmanageChild (buttong);
   buttong = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
   XtUnmanageChild (buttong);

   //   Free strings and return message box.
   if (title_string) XtFree ((char *)title_string);
   if (message_string) XtFree ((char *)message_string);
   if (button_string) XtFree ((char *)button_string);

   return (message_box);
}

/*-------------------------------------------------------------
**  VersionCB      - callback for help button
*------------------------------------------------------------*/
void fn_VersionCB (Widget w,XtPointer client_data,XtPointer call_data)
{
  Widget    message_box; // message box


  //  Create message window.
  message_box = fn_CreateVersion (w);


  // Display version window.
  XtManageChild (message_box);
}

/*-------------------------------------------------------------
**  CreateVersion    - create version window
*------------------------------------------------------------*/
Widget fn_CreateVersion (Widget parent)
{
  Widget    buttong;
  Widget    message_box;
  XmString  title_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;
  Arg args[6];
  static char  message[]="MOODSQL Displayer, Version 1.0 July, 1993";
  int n;

  // Create the compound strings
  message_string = XmStringCreateLtoR (message, charset);
  button_string = XmStringCreateLtoR ("Close", charset);
  title_string = XmStringCreateLtoR ("MOODSQL Displayer Version", charset);

  //  Create message box dialog.
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);n++;
  XtSetArg (args[n], XmNokLabelString, button_string);n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (parent, "versionbox", args, n);

  buttong = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild (buttong);
  buttong = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (buttong);

  // Free strings and return message box.
  if (title_string) XtFree ((char *)title_string);
  if (message_string) XtFree ((char *)message_string);
  if (button_string) XtFree ((char *)button_string);
  return (message_box);
}


/*-------------------------------------------------------------
**  quitCB - callback for quit button
*------------------------------------------------------------*/
extern UserRequest *userRequest;

void fn_quitCB (Widget w,XtPointer client_data,XtPointer call_data)
{
Widget Pwid;
  Pwid = XtParent(w);
  Pwid = XtParent(Pwid);
  Pwid = XtParent(Pwid);
  Pwid = XtParent(Pwid);
  Pwid = XtParent(Pwid);
  XtDestroyWidget(Pwid);
}

