/**********************************************************************
* MoodView ,The Graphical User Interface To MOODS
* Copyright (c) 1994 Scientific and Technical Research Council of Turkiye
*                    Software Research and Development Center, Ankara
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE SOFTWARE RESEARCH AND DEVELOPMENT CENTER ALLOWS FREE USE OF THIS
* SOFTWARE IN ITS "AS IS" CONDITION.
* THE CENTER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The MOODS Project Group requests users of this software to return
* any improvements or extensions that they make to:
*
*   MOODS Project Group
*   TUBITAK Software Research and Development Center
*   Department of Computer Engineering
*   Middle East Technical University
*   Ankara, 06531
*
*        or budak@karakoy.srdc.metu.edu.tr
*
* In addition, the MOODS Project Group requests that users grant the
* Software Research and Development Center rights to redistribute these
* changes.
**********************************************************************/

#include <stdio.h>
#include <Xm/MessageB.h>
#include "me.h"
extern XmStringCharSet charset;

/*-------------------------------------------------------------
**  DialogHelpCB     create dialog help window
*------------------------------------------------------------*/
void DialogHelpCB (Widget w, XtPointer client_data,XtPointer  call_data)
{
  Widget    button;
  Widget    message_box; 
  Arg    args[10]; 
  register int  n; 

  static char  newdlog_help[BUFSIZ];  /*  Help text for new dialog */
  static char  opendlog_help[BUFSIZ];  /*  Help text for open dialog */
  XmString  title_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;


  if ((int)client_data == (int)NEW_HELP) {
  /*  Generate help message for new dialog */
  sprintf (newdlog_help, "\
You must move the pointer into the text window and click BSelect \n\
before you enter the new query name.  After you enter the query \n\
name, press Return or move the pointer to 'OK' and click BSelect.  \0");

  message_string = XmStringCreateLtoR (newdlog_help, charset);
  title_string = XmStringCreateLtoR ("New Query Help", charset);
  }
  else   if ((int)client_data == (int)OPEN_HELP) {
  /*  Generate help message for open dialog */
  sprintf (opendlog_help, "\
Select a query to edit by moving the pointer into the 'Existing Queries' box\n\
and over the name of the query you want.  Click BSelect and then move the \n\
pointer to 'OK' and click BSelect.  The text of the query you selected \n\
will appear in the text edit window. \n\
  \n\
To change the directory, move the pointer into the 'Directories' box and \n\
over the name of the directory you want.  Click BSelect and then move the \n\
pointer to 'Filter' and click BSelect.  You can also enter a new directory \n\
in the 'File Filter' box by moving the pointer into that box and deleting \n\
any or all of the text therein and entering the desired directory name. \n\
Then, either press Return or move the pointer to 'Filter' and click \n\
BSelect.  You can then select a query as described above. \0");

  message_string = XmStringCreateLtoR (opendlog_help, charset);
  title_string = XmStringCreateLtoR ("Open an Existing Query Help", charset);
  }
else{
  /*  Generate help message for new dialog */
  sprintf (newdlog_help, "\
You must move the pointer into the text window, write new name \n\
for your query and  click BSelect.  After you enter the new query \n\
name, press Return or move the pointer to 'OK' and click BSelect.  \0");

  message_string = XmStringCreateLtoR (newdlog_help, charset);
  title_string = XmStringCreateLtoR ("Save As .. Help", charset);
    }

  button_string = XmStringCreateLtoR ("Close", charset);


  /*  Create message box dialog.
  */
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNokLabelString, button_string);  n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (w, "helpbox", args, n);
  XtManageChild (message_box);

  button = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild (button);
  button = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (button);


  /*  Free strings and return message box.
  */
  if (title_string) XmStringFree (title_string);
  if (message_string) XmStringFree (message_string);
  if (button_string) XmStringFree (button_string);
}

/*-------------------------------------------------------------
**  CreateHelpQ    - create help window
*------------------------------------------------------------*/
Widget CreateHelpQ (Widget parent) 
{
  Widget    button;
  Widget    message_box;
  Arg    args[10]; 
  register int  n;

  static char  message[BUFSIZ];  /*  Help text  */
  XmString  title_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;

  /*  Generate message to display.
  */
  sprintf (message, "\
Before you can enter text you must open an existing query or create a new \n\
query.  Move the pointer to 'File' and click BSelect, then select 'Open' \n\
or 'New'.  In either case, dialogs will appear that will allow you to \n\
choose an existing query or enter the name of a new query.  If you choose \n\
a new query, you must move the pointer into the text window and click \n\
BSelect before you enter the new query name.  After you enter the query\n\
name, press Return or move the pointer to 'OK' and click BSelect.  \n\
   \n\
The 'Edit' menu has 'Cut', 'Copy', 'Paste', and 'Clear' buttons.  You \n\
must first make the text to be acted upon the primary selection.  The \n\
'Cut' button deletes or cuts the selected text, but saves it on the \n\
clipboard so you can paste it elsewhere.  'Copy' copies the selected \n\
text to the clipboard but does not delete it.  'Paste' pastes the text \n\
currently in the clipboard to the location of the insertion cursor.  \n\
'Clear' deletes the selected text but does not copy it to the clipboard. \0");

  message_string = XmStringCreateLtoR (message, charset);
  button_string = XmStringCreateLtoR ("Close", charset);
  title_string = XmStringCreateLtoR ("Help", charset);


  /*  Create message box dialog.
  */
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNokLabelString, button_string);  n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (parent, "helpbox", args, n);

  button = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild (button);
  button = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (button);


  /*  Free strings and return message box.
  */
  if (title_string) XmStringFree (title_string);
  if (message_string) XmStringFree (message_string);
  if (button_string) XmStringFree (button_string);
  return (message_box);
}

/*-------------------------------------------------------------
**  HelpCBQ      - callback for help button
*------------------------------------------------------------*/
void HelpCBQ (Widget w, XtPointer client_data,XtPointer  call_data) 
{
  Widget    message_box;   /*  Message box            */

  /*  Create help window.
  */
  message_box = CreateHelpQ (w);

  /*  Display help window.
  */
  XtManageChild (message_box);
}

/*-------------------------------------------------------------
**  CreateVersionQ    - create version window
*------------------------------------------------------------*/
Widget CreateVersionQ (Widget parent) 
{
  Widget    button;
  Widget    message_box;
  Arg    args[10]; 
  register int  n;

  static char  message[BUFSIZ]; 
  XmString  title_string = NULL;
  XmString  message_string = NULL;
  XmString  button_string = NULL;
  char text4[50];

  strcpy (text4, "MOODSQL Editor, Version 1.0, July,1993");
  /*  Generate message to display. */
  sprintf (message, "This is %s \0",text4);

  /* Create the compound strings */
  message_string = XmStringCreateLtoR (message, charset);
  button_string = XmStringCreateLtoR ("Close", charset);
  title_string = XmStringCreateLtoR ("xmeditor help", charset);

  /*  Create message box dialog. */
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNokLabelString, button_string);  n++;
  XtSetArg (args[n], XmNmessageString, message_string);  n++;
  message_box = XmCreateMessageDialog (parent, "versionbox", args, n);

  button = XmMessageBoxGetChild (message_box, XmDIALOG_CANCEL_BUTTON);
  XtUnmanageChild (button);
  button = XmMessageBoxGetChild (message_box, XmDIALOG_HELP_BUTTON);
  XtUnmanageChild (button);

  /*  Free strings and return message box. */
  if (title_string) XmStringFree (title_string);
  if (message_string) XmStringFree (message_string);
  if (button_string) XmStringFree (button_string);
  return (message_box);
}
