/**********************************************************************
* MoodView ,The Graphical User Interface To MOODS
* Copyright (c) 1994 Scientific and Technical Research Council of Turkiye
*                    Software Research and Development Center, Ankara
* All Rights Reserved.
*
* Permission to use, copy, modify and distribute this software and its
* documentation is hereby granted, provided that both the copyright
* notice and this permission notice appear in all copies of the
* software, derivative works or modified versions, and any portions
* thereof, and that both notices appear in supporting documentation.
*
* THE SOFTWARE RESEARCH AND DEVELOPMENT CENTER ALLOWS FREE USE OF THIS
* SOFTWARE IN ITS "AS IS" CONDITION.
* THE CENTER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
* WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
*
* The MOODS Project Group requests users of this software to return
* any improvements or extensions that they make to:
*
*   MOODS Project Group
*   TUBITAK Software Research and Development Center
*   Department of Computer Engineering
*   Middle East Technical University
*   Ankara, 06531
*
*        or budak@karakoy.srdc.metu.edu.tr
*
* In addition, the MOODS Project Group requests that users grant the
* Software Research and Development Center rights to redistribute these
* changes.
**********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <Xm/Text.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/SelectioB.h>
#include <Xm/CascadeB.h>
#include <Xm/FileSB.h>
#include "me.h"

extern int language;
extern Widget cut_button;           /* Clipboard cut button          */
extern Widget copy_button;          /* Clipboard copy button         */
extern Widget copy_by_name_button;  /* Clipboard copy by name button */
extern Widget paste_button;         /* Clipboard paste button        */
extern Widget paste_by_name_button; /* Clipboard paste by name button*/
extern XmStringCharSet charset;
Widget open_dialog;          /* File selection dialog                */
Widget new_dialog;           /* File name prompt dialog              */
Widget close_warning;        /* Special internal selection dialog    */
Widget exit_warning;         /* Special internal selection dialog    */
Widget general_warning;      /* Warning dialog                       */
Widget save_dialog;   
Widget menu_pane;

char *path;

// Bits for exclamation point in dialog
char warningBits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00,
   0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00,
   0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00,
   0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00,
   0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00,
   0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00,
   0x00, 0xe0, 0x07, 0x00, 0x00, 0xe0, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x03, 0x00,
   0x00, 0xe0, 0x07, 0x00, 0x00, 0xf0, 0x0f, 0x00, 0x00, 0xf0, 0x0f, 0x00,
   0x00, 0xf0, 0x0f, 0x00, 0x00, 0xf0, 0x0f, 0x00, 0x00, 0xe0, 0x07, 0x00,
   0x00, 0xc0, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00};

/*-------------------------------------------------------------
**  MenuCB
**    Process callback from PushButtons in PulldownMenus.
*------------------------------------------------------------*/
void MenuCB (Widget w, XtPointer client_data,XtPointer  call_data)
{
  register int  n; 
  Arg           args[10];
  XmString      text_string; 

  switch ((int)client_data)
  {
    case MENU_OPEN:
      XtManageChild (open_dialog);
      break;

    case MENU_NEW:
      XtManageChild (new_dialog);
      break;

    case MENU_CLOSE:
      if (!file_saved) // Display the 'save' message dialog
         XtManageChild (close_warning);
      else
         CloseFile();
      break;

    case MENU_SAVE:
      SaveFile();
      break;

    case MENU_SAVE_AS:
      text_string = XmStringCreateLtoR (filename, charset);
      n = 0;
      XtSetArg(args[n], XmNtextString,text_string);  n++;
      XtSetValues(save_dialog, args, n);
      XtManageChild (save_dialog);
      XmStringFree (text_string);
      break;

    case MENU_EXIT:
      // Exit if there is no file open 
      if (!file_saved) // Display the 'save' message dialog
         XtManageChild (exit_warning);
      else {
         CloseFile();
         exit (0);
      }
      break;

    case MENU_CUT:
      CopyFileToClipboard(((XmPushButtonCallbackStruct *) call_data)->event-> xbutton.time);

      XmTextRemove(text);
      break;

    case MENU_COPY:
      CopyFileToClipboard(((XmPushButtonCallbackStruct *) call_data)->event->xbutton.time);
      break;
    case MENU_COPY_BY_NAME:
      by_name = True;
      /* Call routine to copy selection to clipboard */
      CopyFileToClipboard(((XmPushButtonCallbackStruct *)
                            call_data)->event->xbutton.time);
      break;

    case MENU_PASTE:
      PasteItemFromClipboard();
      break;

    case MENU_PASTE_BY_NAME:
      PasteByName();
      break;

    case MENU_CLEAR:
      XmTextRemove(text);
      break;

    default:
      fprintf(stderr, "Warning: in menu callback\n");
      break;
  }
}

/**------------------------------------------------------------
**  CreateDefaultImage - create a default image for warning symbol.
*------------------------------------------------------------*/
XImage * CreateDefaultImage (char *bits, int width,int  height)
{
    XImage *image;

    image = (XImage *) XtMalloc (sizeof (XImage));
    image->width = width;
    image->height = height;
    image->data = bits;
    image->depth = 1;
    image->xoffset = 0;
    image->format = XYBitmap;
    image->byte_order = LSBFirst;
    image->bitmap_unit = 8;
    image->bitmap_bit_order = LSBFirst;
    image->bitmap_pad = 8;
    image->bytes_per_line = (width+7)/8;
    return (image);
}

/*-------------------------------------------------------------
**  CreateMenuBar
**    Create menu bar in main window
*------------------------------------------------------------*/
Widget CreateMenuBar (Widget parent)
{
  Widget        menu_bar;
  Widget        cascade;
  Widget        button; 
  XImage        *image;
  Arg           args[10];
  register int  n;
  XmString      label_string,title_string;
  XmString      acc_string,file_st,dir_st;

  /*  Create MenuArea.
  */
  n = 0;
  menu_bar = XmCreateMenuBar (parent, "menu_bar", args, n);

  /*  Create "Options" pulldown menu.
  */
  n = 0;
  menu_pane = XmCreatePulldownMenu (menu_bar, "menu_pane", args, n);

  image = CreateDefaultImage (warningBits, 32, 32);
  XmInstallImage (image, "warning_image");

  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'O'); n++;
  if (language == ENGLISH) 
    button = XmCreatePushButton (menu_pane, "Open", args, n);
  if (language == GERMAN)
    button = XmCreatePushButton (menu_pane, "Oeffnen", args, n);

  XtAddCallback (button, XmNactivateCallback, MenuCB, (XtPointer)MENU_OPEN);
  XtManageChild (button);

  title_string = XmStringCreateLtoR("Open an Existing Query",charset);
  if (language == ENGLISH)
    file_st = XmStringCreateLtoR("Existing Queries",charset);
  if (language == GERMAN)
    file_st = XmStringCreateLtoR("Existierende Abfragen",charset);

  dir_st = XmStringCreateLtoR("*.msql",charset);
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNfileListLabelString, file_st);  n++;
  XtSetArg (args[n], XmNdirMask,dir_st);  n++;
  open_dialog = XmCreateFileSelectionDialog(menu_pane,"file selection dialog",args,n);

  XtAddCallback (open_dialog, XmNokCallback,DialogAcceptCB, (XtPointer)DIALOG_FSELECT);
  XtAddCallback (open_dialog, XmNcancelCallback,DialogCancelCB, (XtPointer) DIALOG_FSELECT);
  XtAddCallback (open_dialog, XmNhelpCallback,DialogHelpCB, (XtPointer) OPEN_HELP);

  if (language == ENGLISH) {
  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'N'); n++;
  button = XmCreatePushButton (menu_pane, "New", args, n);
  }
  if (language == GERMAN) {
  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'N'); n++;
  button = XmCreatePushButton (menu_pane, "Neu", args, n);
  }

  XtAddCallback (button, XmNactivateCallback, MenuCB,  (XtPointer)MENU_NEW);
  XtManageChild (button);

  label_string = XmStringCreateLtoR ("Enter Name of New Query :", charset);
  title_string=XmStringCreateLtoR("New Query",charset);

  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg (args[n], XmNselectionLabelString, label_string); n++;
  new_dialog = XmCreatePromptDialog(menu_pane,"new file dialog", args, n);
  XtAddCallback (new_dialog, XmNokCallback, DialogAcceptCB,  (XtPointer)DIALOG_NEW);
  XtAddCallback (new_dialog, XmNcancelCallback,DialogCancelCB,  (XtPointer)DIALOG_NEW);
  XtAddCallback (new_dialog, XmNhelpCallback,DialogHelpCB,  (XtPointer)NEW_HELP);
  XmStringFree (label_string);

  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'C'); n++;
  button = XmCreatePushButton (menu_pane, "Close", args, n);
  XtAddCallback (button, XmNactivateCallback, MenuCB,  (XtPointer)MENU_CLOSE);
  XtManageChild (button);

  title_string=XmStringCreateLtoR("Save Warning !",charset);
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  close_warning = CreateSpecialWarningDialog(menu_pane, "save_warning",
              "warning_image", "Save Changes?", args, n);

  XtAddCallback(close_warning, XmNapplyCallback,DialogApplyCB,  (XtPointer)DIALOG_CWARNING);
  XtAddCallback(close_warning, XmNokCallback,DialogAcceptCB,  (XtPointer)DIALOG_CWARNING);

  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'S'); n++;
  button = XmCreatePushButton (menu_pane, "Save", args, n);
  XtAddCallback (button, XmNactivateCallback, MenuCB,  (XtPointer)MENU_SAVE);
  XtManageChild (button);

  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'A'); n++;
  button = XmCreatePushButton (menu_pane, "Save As ...", args, n);
  XtAddCallback (button, XmNactivateCallback, MenuCB,  (XtPointer)MENU_SAVE_AS);

  XtManageChild (button);

  label_string = XmStringCreateLtoR ("Save As ...", charset);
  title_string=XmStringCreateLtoR("Save As ... ",charset);
  n = 0;
  XtSetArg (args[n], XmNdialogTitle, title_string);  n++;
  XtSetArg(args[n], XmNselectionLabelString, label_string); n++;
  save_dialog = XmCreatePromptDialog(menu_pane, "Save As ...", args, n);
  XtAddCallback (save_dialog, XmNokCallback,DialogAcceptCB,  (XtPointer)DIALOG_SAVE);
  XtAddCallback (save_dialog, XmNhelpCallback,DialogHelpCB, (XtPointer) DIALOG_SAVE);

  acc_string = XmStringCreateLtoR ("F3", charset);
  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'E'); n++;
  XtSetArg(args[n], XmNacceleratorText, acc_string); n++;
  XtSetArg(args[n], XmNaccelerator, "<Key>F3:"); n++;
  button = XmCreatePushButton (menu_pane, "Exit", args, n);
  XtAddCallback (button, XmNactivateCallback, MenuCB,  (XtPointer)MENU_EXIT);
  XtManageChild (button);
  XmStringFree (acc_string);

  exit_warning = CreateSpecialWarningDialog(menu_pane, "exit warning",
              "warning_image", "Save Changes?", args, n);
  XtAddCallback (exit_warning, XmNapplyCallback,DialogApplyCB,  (XtPointer)DIALOG_XWARNING);
  XtAddCallback (exit_warning, XmNokCallback,DialogAcceptCB,  (XtPointer)DIALOG_XWARNING);

  n = 0;
  XtSetArg (args[n], XmNsubMenuId, menu_pane);  n++;
  XtSetArg(args[n], XmNmnemonic, 'Q'); n++;
  cascade = XmCreateCascadeButton (menu_bar, "Query   ", args, n);
  XtManageChild (cascade);

  /*  Create "Edit" pulldown   menu.
  */
  n = 0;
  menu_pane = XmCreatePulldownMenu (menu_bar, "menu_pane", args, n);

  acc_string = XmStringCreateLtoR ("Shift+Del", charset);
  n = 0;
  XtSetArg(args[n], XmNmnemonic, 't'); n++;
  XtSetArg(args[n], XmNacceleratorText, acc_string); n++;
  XtSetArg(args[n], XmNaccelerator, "Shift<Key>DeleteChar:"); n++;
  cut_button = XmCreatePushButton (menu_pane, "Cut", args, n);
  XtAddCallback (cut_button, XmNactivateCallback, MenuCB,  (XtPointer)MENU_CUT);
  XtManageChild (cut_button);
  XtSetSensitive(cut_button, False);
  XmStringFree (acc_string);

  acc_string = XmStringCreateLtoR ("Ctrl+Ins", charset);
  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'C'); n++;
  XtSetArg(args[n], XmNacceleratorText, acc_string); n++;
  XtSetArg(args[n], XmNaccelerator, "Ctrl<Key>InsertChar:"); n++;
  copy_button = XmCreatePushButton (menu_pane, "Copy", args, n);
  XtAddCallback (copy_button, XmNactivateCallback, MenuCB,  (XtPointer)MENU_COPY);
  XtManageChild (copy_button);
  XtSetSensitive(copy_button, False);
  XmStringFree (acc_string);

  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'o'); n++;
  copy_by_name_button = XmCreatePushButton (menu_pane, "Copy By Name", args, n);
  XtAddCallback (copy_by_name_button, XmNactivateCallback, MenuCB,  (XtPointer)MENU_COPY_BY_NAME);
  XtManageChild (copy_by_name_button);
  XtSetSensitive(copy_by_name_button, False);

  acc_string = XmStringCreateLtoR ("Shift+Ins", charset);
  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'P'); n++;
  XtSetArg(args[n], XmNacceleratorText, acc_string); n++;
  XtSetArg(args[n], XmNaccelerator, "Shift<Key>InsertChar:"); n++;
  paste_button = XmCreatePushButton (menu_pane, "Paste", args, n); n++;
  XtAddCallback (paste_button, XmNactivateCallback, MenuCB,  (XtPointer)MENU_PASTE);
  XtManageChild (paste_button);
  XtSetSensitive(paste_button, False);
  XmStringFree (acc_string);

  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'a'); n++;
  paste_by_name_button = XmCreatePushButton (menu_pane,"Paste By Name",args,n);
 n++;
  XtAddCallback (paste_by_name_button,XmNactivateCallback,MenuCB, (XtPointer)MENU_PASTE_BY_NAME);
  XtManageChild (paste_by_name_button);
  XtSetSensitive(paste_by_name_button, False);

  acc_string = XmStringCreateLtoR ("Del", charset);
  n = 0;
  XtSetArg(args[n], XmNmnemonic, 'e'); n++;
  XtSetArg(args[n], XmNacceleratorText, acc_string); n++;
  if (language == ENGLISH)
    clear_button = XmCreatePushButton (menu_pane, "Clear", args, n);
  if (language == GERMAN)
    clear_button = XmCreatePushButton (menu_pane, "Loeschen", args, n);
  XtAddCallback (clear_button, XmNactivateCallback, MenuCB,  (XtPointer)MENU_CLEAR);
  XtManageChild (clear_button);
  XtSetSensitive(clear_button, False);
  XmStringFree (acc_string);

  n = 0;
  XtSetArg (args[n], XmNsubMenuId, menu_pane);  n++;
  XtSetArg(args[n], XmNmnemonic, 'E'); n++;
  cascade= XmCreateCascadeButton (menu_bar, "Edit", args, n);
  XtManageChild (cascade);


  //Create "Help" pulldown menu.
  n = 0;
  menu_pane = XmCreatePulldownMenu (menu_bar, "menu_pane", args, n);

  n = 0;
  button = XmCreatePushButton (menu_pane, "Help", args, n);
  XtManageChild (button);
  XtAddCallback (button, XmNactivateCallback, HelpCBQ, NULL);

  n = 0;
  button = XmCreatePushButton (menu_pane, "Program Version", args, n);
  XtManageChild (button);
  XtAddCallback (button, XmNactivateCallback, VersionCBQ, NULL);

  n = 0;
  XtSetArg (args[n], XmNsubMenuId, menu_pane);  n++;
  cascade = XmCreateCascadeButton (menu_bar, "Help", args, n);
  XtManageChild (cascade);

  n = 0;
  XtSetArg (args[n], XmNmenuHelpWidget, cascade);  n++;
  XtSetValues (menu_bar, args, n);

    XtManageChild (menu_bar);
  return menu_bar;
}
