
/************************************************************************* 

			ABOUT THIS FILE 



Subject	:

	Definition of linked list structure which is used in implementaion 
	of MOODS.

Author 	: 

	Cetin Ertan Ozkan

Date	: 

	18/02/1993

Last Test Date :

	18/02/1993

Last Revision Date :

	No revision

Related Files :

	../parser/linklist.c --> related member functions

*************************************************************************/

#ifndef __LINKLIST_H
#define __LINKLIST_H

#include <string.h>
#include "reqroot.h"
#include "keywords.h"
#include "opt_defs.h"
#include "dbi_gen.h"
#include "opt.h"
#include "dbi.h"
#include "global.h"

// classes in this header file

class ListItem;
class LinkedList;
class AndTerms;
class Stack ;
class SortedStringList;
class DictItem;
class Dictionary;
class AndTerm;
class SubTreeItem;
class SubTreeList;
class Expression;
class BindListItem;
class BindExpression;
class ProjectionListItem;
class ProjectionExpression;
class PathItem;
class PathExpression;


class ListItem
{
public :
	unsigned	itemType; // type of the item
	ListItem	*next,
			*prev,
			*left, 
			*right,
			*lastTerm; 
	void		*item; // item itself

			ListItem();
			ListItem(unsigned itype, void *i);
	void		printOn();
			~ListItem();
};


class LinkedList
{
public :
	unsigned int	size; // size of the list
	ListItem	*firstItem; // pointer to the first item
	ListItem	*lastItem; // pointer to the last item
	ListItem	*curItem; // pointr to current item at a time

			LinkedList();
	void 		insertItem(ListItem *listItem);
	int		removeItem(ListItem *listItem);
	int		removeItem(unsigned index);
	void		deepDelete();
	void		deleteItems();	
	ListItem* 	getLastItem();
	void*           getFirstEntry();
        void*           getNextEntry();
	void 		insert(unsigned itype, void *i);
	void		traverseList();
	ListItem* 	listAt(unsigned int i);
	void		printOn();
	void		bprintOn();
	void		traverseAsTree( ListItem * );
	LinkedList*	subList( ListItem *stItem, ListItem *endItem );
	LinkedList*	extractVariables();
	void		generateMAT( SubTreeList *subTree);
	void		generateSELECT( SubTreeItem *place);
	JOIN_ARG*	convertToIlker( ListItem * );
			~LinkedList();
};


// Implementation of a Stack as a link list
class Stack : public LinkedList
{
public:
	void 		push(unsigned, void *);
	ListItem*	pop();
        void		extractAndedTerms( AndTerms *, ListItem *);
        ListItem*       stack2tree( ListItem *);
	void		printOn()
	{
		LinkedList::printOn();
	}
	
			Stack();
			~Stack() ;
};


class SortedStringList: public LinkedList
{
public :
        void            insertSorted(unsigned itype, void *i);
        ListItem*       findPlace(char *i);
        int             compare( SortedStringList* cmpKey);
	void		printOn();
			contains( SortedStringList * );
			hasMember( char *);
	SortedStringList *merge( SortedStringList * );
			~SortedStringList();
};


class DictItem
{
public :
        SortedStringList	*key;
        LinkedList		*exp;
	char			isProcessed;
	QUERY			*currentTree;


			DictItem();
        void            setKey( SortedStringList *);
        void            addExp( LinkedList      *anExp);
	void		printOn();
	void		generateSELECT( SubTreeList *subTree);
	JOIN_ARG	*generateJoinPred();
			~DictItem();
	
};

class Dictionary: public LinkedList
{
public :
        void		add( SortedStringList *inKey, LinkedList *andTerm );
        ListItem*       findPlace( SortedStringList *inKey );
	void		printOn();
	void		generateSELECT( SubTreeList *subTree);
	QUERY		*generateJOIN( SubTreeList *subTree, UserRequest *);
			~Dictionary();
};


class AndTerm
{
public :
        LinkedList      *andedExp; // ANDed terms
        Dictionary      *dictItems; // dictionary wrt to bind variables
	QUERY		*top;

			AndTerm();
        void            extractTerms( ListItem *inItem);    // extract AndedTerms
        void            normalizeTerms();  // leave variable on left side
	void		setExp( LinkedList *anExp);
	void		printOn();
	QUERY*		generateTree( SubTreeList *subTree, ProjectionExpression *pList, UserRequest * );
	QUERY*		generateJOIN( SubTreeList *subTree, UserRequest * );
			~AndTerm();
};

class AndTerms:public LinkedList
{
public :
	QUERY		*top;
	void insert(void *item)
	{
		LinkedList::insert( OR_TERM, item );
	}
			AndTerms();
	QUERY*		generateTree( SubTreeList *, ProjectionExpression *, UserRequest *);
			~AndTerms();

};

class SubTreeItem
{
public :
        char    	bindVar[80];
        QUERY 		*top;

        QUERY*		isGeneratedMAT( char *pathStr );
	void		printOn();
			SubTreeItem();
			~SubTreeItem();
};

class SubTreeList : public LinkedList
{
public :
        void            insert( void *i );
        SubTreeItem*    findPlace( char *bindVar );
        void            generate( BindExpression *, SortedStringList *);
	void		printOn();
			SubTreeList();
			~SubTreeList();
};


inline void SubTreeList::insert( void *i)
{
        LinkedList::insert( SUBTREE_ITEM, i);
}


/**************************************************************************

			EXPRESSION CLASSES

**************************************************************************/


class Expression : public Stack, public RequestRoot
{

private :
        int		complexExpression();
        int		compExpPrime();
        int		expression();
        int		expressionPrime();
        int		term();
        int		termPrime();
        int		factor();
        int		identifier();

public :

			Expression(UserRequest *userReq);
        int		parse();
	void 		push(int itype, void *item);
	void		printOn();
	void		normalize( AndTerms *orTerms);
	int		checkSemantic( BindExpression *, ListItem *, char * );
	//int		evaluateType();
			findUseSelves( SortedStringList *);
			~Expression();

};


/**************************************************************************

		CLASS FOR FROM CLAUSE EXPRESSIONS

**************************************************************************/

class BindListItem
{
public : 
        Expression      *bindExpression;
        char            alias[80];
	RELATION_SCHEMA classInfo;
 
        		BindListItem( UserRequest *userReq);
        char 		*setBindVariable( char *aName);
	int		checkSemantic( UserRequest *userReq );
			~BindListItem();
	void		printOn();
};


class BindExpression : public LinkedList, public RequestRoot
{
public :
        		BindExpression( UserRequest *userReq) ;
	int		parse();
	void		printOn();
	BindListItem*	contains( char *bindVar );
	char*		getClassName( char *bindVar );
	RELATION_SCHEMA *isDefineClass( char * );
			checkSemantic();
			~BindExpression();

};



/************************************************************************

		CLASS FOR PROJECTION LISTs

************************************************************************/



class ProjectionListItem : public RequestRoot
{
public :
	Expression 	*projExpression;
	int		specifier;

			ProjectionListItem( UserRequest *userReq);
			parse();
	void		printOn();
			~ProjectionListItem();
};


class ProjectionExpression : public LinkedList, public RequestRoot
{
public :
        		ProjectionExpression( UserRequest *userReq);
        int             parse(); 
	void		printOn();
	int		checkSemantic( BindExpression * bexp );
	JOIN_ARG	**convertToMehmet();
			~ProjectionExpression();
 
};

class PathItem
{
public :
        char            itemName[MAXSTRLEN];
	char		itemSign[MAXSTRLEN]; // attribute name or function sign.
        char            itemType; // FUNCTION or ATTRIBUTE 
	char		dbType; // Database Type or others
	char		isSetList;
	int		returnType; // only for function
	char		retTypeName[MAXSTRLEN]; // only for function
	RELATION_SCHEMA	attClass, *prevSchema;
	
        LinkedList      *parameters; //if it is an function parameter list 

			PathItem(); 
	char 		*setItemName( char *item );
	void		setItemType( char t );
	void		stack2tree();
	int		fillPathItem( BindExpression *, ListItem *, UserRequest *);
	void		generateMAT( SubTreeItem *place, char *pathStr );
	void		printOn();
	void		convertParameters( FCN_INPUT ** );
			~PathItem();
};

class PathExpression :  public RequestRoot
{
public :
        LinkedList      *pathItems;
	char		pathExpAsString[90];// string representation of path exp 
	char		finalType[30]; // for strict type checking

        		PathExpression(UserRequest *userReq);
	int		parse(); // parses a path expression
	int		parseParamList( PathItem * ); //parses parameter list inside path exp
	int		checkSemantic( BindExpression * ); // check semantic
	void		stack2tree(); // convert postfix stack to tree
	char*		findString(); // find string representation of path exp
	void		generateMAT( SubTreeList *subTree );// generates MAT tree for path exp
	void		printOn(); // output function
			~PathExpression();
};

#endif // __LINKLIST_H
