#include <stdio.h>
#include "mood_catalog.h"
#include <linklist.h>
#include <keywords.h>
#include "mood_parser.h"
#include <classddl.h>
#include "decldml.h"

extern ListItem        undefinedItem;



/**************************************************************************

                        ListItem Methods

**************************************************************************/


void freeAttribute( RELATION_SCHEMA *rs );
void freeFCN_INPUT( FCN_INPUT *funcIn );
void freeJoinArg( JOIN_ARG *pred );
void freeQUERY( QUERY * aplan );

#define DALLOC_WRITE( _dallocString, _deallocatedItem );\

/*	dealloc = fopen( "deallocate.dat", "a" );\
        fprintf( dealloc,  _dallocString , _deallocatedItem );\
        fclose( dealloc );\*/

#define MALLOC_WRITE( _mallocString, _allocatedItem);\

/*        myalloc = fopen( "allocate.dat", "a" );\
        fprintf(myalloc, _mallocString, _allocatedItem );\
        fclose( myalloc );\*/



// print out a list item
ListItem::~ListItem()
{
        if(!this) return;
	DALLOC_WRITE( "%p ListItem\n", this );
	switch( itemType )
	{
		/* Logical Operators */
		case EQ : case GT : case GE : case LT : case LE : case NE :
		/* Boolean Operators */
		case NOT : case OR  : case AND :
		/* arithmetic operators */
		case ADD : case SUB : case MUL : case DIV : case MOD :
		/* unary operators */
		case UNARYPLUS  : case UNARYMINUS : case EVERY      :
		break;
		case NULLVALUE : case MS_TRUE : case MS_FALSE :
		break;
		/* constants */
		case CHRCONS :
		case STRCONS :  
			myFree( (char *) item);
			break;

		case UNSIGNEDREAL:
			myFree( (char  *) item);
			break;

		case UNSIGNEDINT :
			myFree( (char *) item);
			break;
	
		/* Bind Item */
		case BIND_ITEM : 
			delete (BindListItem *)item; break;

		case PROJ_ITEM :
			delete (ProjectionListItem *)item; break;

		case PATH_ITEM :
			delete ( PathItem *) item; break;

		case PATH_EXP  : 
			delete ( PathExpression *) item; break;

		case UPDATE_EXP: 
			delete ( Expression *) item; break;

		case EXP_PTR   : 
			delete ( Expression *) item;	 break;

		case FUNCTION_PARAM : 
			delete ( Expression *) item;	 break;

		case QRY_PTR   : 
			delete ( Query *) item ; break;

		case NEW_PTR   : 
			delete (NewClause *) item; break;

		case ATR_PTR   :
			delete (AttrList *) item; break;

		case OID_PTR   :
			free( (OID *)item ); break;

		case LST_PTR   :
		case SET_PTR   : 
			delete ( LinkedList *) item; break;

		case OR_TERM   : 
			delete ( AndTerm * ) item; break;

		case BIND_AND_TERM : 
			delete (DictItem *) item; break;

		case AND_TERM  : 
			((LinkedList *) item)->deleteItems(); 
			delete (LinkedList *) item;
			break;

		case SUBTREE_ITEM :
			delete (SubTreeItem *) item; break;

		case CATALOG_TYPE_ITEM :
			delete (CatalogTypeItem *) item; 
			break;

		case FUNCTION_TYPE_ITEM :
			delete (FunctionType * ) item; break;

		case CLSCONS :
			delete (ClassCreate *) item; break;

		default : 
			 //printf(" Undefined Item Type in delete : %d", itemType );
			break;
	}
	item 	= NULL;
	next	= NULL;
	prev	= NULL;
	left	= NULL;
	right	= NULL;
	lastTerm= NULL;
	itemType= 0;
}

DictItem::~DictItem()
{
	if( !this ) return;
	DALLOC_WRITE( "%p DictItem\n", this );
	delete key;
	delete exp;
	delete currentTree;
	key = NULL;
	exp = NULL;
	currentTree = NULL;
}

BindListItem::~BindListItem()
{
	if( !this ) return;
	DALLOC_WRITE( "%p BindListItem\n", this );
        delete bindExpression;
        bindExpression = NULL;
	freeAttribute( &classInfo );
}

ProjectionListItem::~ProjectionListItem()
{
	if( !this ) return;
	DALLOC_WRITE( "%p ProjectionListItem\n", this );
        delete projExpression;
        projExpression = NULL;
}

PathItem::~PathItem()
{
	if( !this ) return;
	DALLOC_WRITE( "%p PathItem\n", this );
	freeAttribute( &attClass );
	delete parameters;
	parameters = NULL;
}

// deconstructor for a list
LinkedList::~LinkedList()
{
	if(!this) return;
	DALLOC_WRITE( "%p LinkedList\n", this );
        ListItem *p, *q;
        p = firstItem;
        while( p ){
                q = p->next;
                delete p;
                p = q;
        }
	size = 0;
	firstItem = NULL;
	lastItem  = NULL;
	curItem   = NULL;

}

void LinkedList::deepDelete()
{
       if(!this) return;
        ListItem *p, *q;
        p = firstItem;
        while( p ){
                q = p->next;
                delete p;
                p = q;
        }
	size = 0;
	firstItem = NULL;
	lastItem = NULL;
}

void LinkedList::deleteItems()
{
       if(!this) return;
        ListItem *p, *q;
        p = firstItem;
        while( p ){
                q = p->next;
                myFree( (char *) p);
                p = q;
        }
	size = 0;
	firstItem = NULL;
	lastItem = NULL;
}

Stack::~Stack()
{
	if( !this ) return;
	DALLOC_WRITE( "%p LinkedList\n", this );
}
SortedStringList::~SortedStringList()
{
	if( !this ) return;
	DALLOC_WRITE( "%p SortedStringList\n", this );
}
Dictionary::~Dictionary()
{
	if( !this ) return;
	DALLOC_WRITE( "%p Dictionary\n", this );
}
AndTerms::~AndTerms()
{
	if( !this ) return;
	DALLOC_WRITE( "%p AndTerms\n", this );
	top = NULL;
}
SubTreeItem::~SubTreeItem()
{
	if( !this ) return;
	DALLOC_WRITE( "%p SubTreeItem\n", this );
	top = NULL;
}
SubTreeList::~SubTreeList()
{
	if( !this ) return;
	DALLOC_WRITE( "%p SubTreeList\n", this );
}
Expression::~Expression()
{
	if( !this ) return;
	DALLOC_WRITE( "%p Expression\n", this );
}
BindExpression::~BindExpression()
{
	if( !this ) return;
	DALLOC_WRITE( "%p BindExpression\n", this );
}
ProjectionExpression::~ProjectionExpression()
{
	if( !this ) return;
	DALLOC_WRITE( "%p ProjectionExpression\n", this );
}

AndTerm::~AndTerm()
{
	if( !this ) return;
	DALLOC_WRITE( "%p AndTerm\n", this );
	//delete andedExp;
	andedExp->deleteItems();
	delete andedExp;
	delete dictItems;
	// delete	top; comment out , do not remove this
	andedExp = NULL;
	dictItems = NULL;
	top = NULL;
}

PathExpression::~PathExpression()
{
	if( !this ) return;
	DALLOC_WRITE( "%p PathExpression\n", this );
	delete pathItems;
	pathItems = NULL;
}

CreateDatabase::~CreateDatabase()
{
	if( !this ) return;
	DALLOC_WRITE( "%p CreateDatabase\n", this );
}

UseDatabase::~UseDatabase()
{
	if( !this ) return;
	DALLOC_WRITE( "%p UseDatabase\n", this );
}

DropDatabase::~DropDatabase()
{
	if( !this ) return;
	DALLOC_WRITE( "%p DropDatabase\n", this );
}

ListDatabase::~ListDatabase()
{
	if( !this ) return;
	DALLOC_WRITE( "%p ListDatabase\n", this );
	delete dbList;
}

ListType::~ListType()
{
	if( !this ) return;
	DALLOC_WRITE( "%p ListType\n", this );
	if( typeList ) delete typeList;
}

Query::~Query()
{
	if( !this ) return;
	DALLOC_WRITE( "%p Query\n", this );
	if( whereP )    	delete whereP;
	if( startWithP) 	delete startWithP;
	if( havingP )   	delete havingP;
	if( projectionList) 	delete projectionList;
	if( groupByList ) 	delete groupByList;
	if( orderByList )	delete orderByList;
	if( bindList) 		delete bindList;
	if( recurseAttribute ) 	delete recurseAttribute;
	if( parentQuery ) 	delete parentQuery;
	if( subQueries ) 	delete subQueries;
	if( ORTerms )		delete ORTerms;
	if( queryTree ) 	delete queryTree;
	if( solutionFile ){
		freeAttribute( solutionFile );
		myFree( (char *) solutionFile ) ;
	}
	if( useSelfVariables )	delete useSelfVariables;
}

ClassCreate::~ClassCreate()
{
	if( !this ) return;
	DALLOC_WRITE( "%p ClassCreate\n", this );
        if( !inheritList ) delete inheritList;
        deleteMembers( members );
}

UpdateClause::~UpdateClause()
{
	if( !this ) return;
	DALLOC_WRITE( "%p UpdateClause\n", this );
	if( whereP ) 		delete whereP;
	if( bindList ) 		delete bindList;
	if( updateExpression ) 	delete updateExpression;
}

DescribeClause::~DescribeClause()
{
	if( !this ) return;
	DALLOC_WRITE( "%p DescribeClause\n", this );
	if( att ) 	delete att;
	if( superCls )	delete superCls;
	if( subCls )	delete subCls;
}

DeleteClause::~DeleteClause()
{
	if( !this ) return;
	DALLOC_WRITE( "%p DeleteClause\n", this );
        if(!whereP)          delete whereP;
        if(!projectionList)  delete projectionList;
        if(!bindList)        delete bindList;
}


UserRequest::~UserRequest()
{
	if( !this ) return;
	DALLOC_WRITE( "%p UserRequest\n", this );
	reset();
	if( text )	delete text;
}

DisplayHierarchy::~DisplayHierarchy()
{
	if( !this ) return;
	DALLOC_WRITE( "%p DisplayHierarchy\n", this );
        delete hier;
}

DropFunction::~DropFunction()
{
	if( !this ) return;
	DALLOC_WRITE( "%p DropFunction\n", this );
}

ListFunction::~ListFunction()
{
	if( !this ) return;
	DALLOC_WRITE( "%p ListFunction\n", this );
        delete functionList;
}

DeclStmnt::~DeclStmnt()
{
	if( !this ) return;
	DALLOC_WRITE( "%p DeclStmnt\n", this );
}

ClassDrop::~ClassDrop()
{
	if( !this ) return;
	DALLOC_WRITE( "%p ClassDrop\n", this );
	if( clsList ) delete clsList;
}

CreateFunction::~CreateFunction()
{
	if( !this ) return;
	DALLOC_WRITE( "%p CreateFunction\n", this );
}

CatalogTypeItem::~CatalogTypeItem()
{
	if( !this ) return;
	DALLOC_WRITE( "%p CatalogTypeItem\n", this );
}

FunctionType::~FunctionType()
{
	if( !this ) return;
	DALLOC_WRITE( "%p FunctionType\n", this );
}

ClassNames::~ClassNames()
{
	if( !this ) return;
	if( next ) delete next;
	DALLOC_WRITE( "%p ClassNames\n", this );
}

AttrList::~AttrList()
{
	if( !this ) return;
	if( next ) delete next;
	if( nextsub ) delete nextsub;
	DALLOC_WRITE( "%p AttrList\n", this );
}

LevelInfo::~LevelInfo()
{
	if( !this ) return;
	if( next ) delete next;
	DALLOC_WRITE( "%p LevelInfo\n", this );
}

Hierarchy::~Hierarchy()
{
	if( !this ) return;
	if( nextlevel ) delete nextlevel;
	if( thislevel ) delete thislevel;
	DALLOC_WRITE( "%p Hierarchy\n", this );
}

/************************************************************************/

//  All constructors used by the MOOD
AttrList::AttrList()
{
	next = NULL;
	nextsub = NULL;
	MALLOC_WRITE( "%p AttrList\n", this );
}

Hierarchy::Hierarchy()
{
	MALLOC_WRITE( "%p Hierarchy\n", this );
	nextlevel = NULL;
	thislevel = NULL;
}

LevelInfo::LevelInfo()
{
	MALLOC_WRITE( "%p LevelInfo\n", this );
	next = NULL;
}

FunctionType::FunctionType()
{
	CatRoot::__typeId = TYPE_FUNCTION;
	MALLOC_WRITE( "%p FunctionType\n", this );
}

ClassNames::ClassNames()
{
	MALLOC_WRITE( "%p ClassNames\n", this );
	next = NULL;
}

DeclStmnt::DeclStmnt( UserRequest *p)
{ 
	MALLOC_WRITE( "%p DeclStmnt\n", this );
	ur = p;
}

ClassCreate::ClassCreate( UserRequest *userReq )
{
	MALLOC_WRITE( "%p ClassCreate\n", this );
        ur = userReq;
        strcpy(className, "");
        members = NULL;
        inheritList = NULL;
        //attributeList = NULL;
}
 
ClassCreate::ClassCreate( UserRequest *userReq, int flag )
{
	MALLOC_WRITE( "%p ClassCreate\n", this );
        typeFlag = flag;
        ur = userReq;
        strcpy(className, "");
        members = NULL;
        inheritList = NULL;
        //attributeList = NULL;
}

LinkedList::LinkedList()
{
	MALLOC_WRITE( "%p LinkedList\n", this );
	size = 0;
	firstItem = NULL;
	lastItem  = NULL;
	curItem   = NULL;
}

Stack::Stack() : LinkedList()
{
	MALLOC_WRITE( "%p Stack\n", this );
}

DictItem::DictItem() 
{
	MALLOC_WRITE( "%p DictItem\n", this );
	exp = new LinkedList;
	isProcessed = FALSE;
	currentTree = NULL;
	key = NULL;
}

AndTerm::AndTerm()
{
	MALLOC_WRITE( "%p AndTerm\n", this );
	dictItems = new Dictionary; 
	andedExp = NULL;
	top = NULL; 
}

AndTerms::AndTerms()
{
	MALLOC_WRITE( "%p AndTerms\n", this );
	top = NULL;
}

SubTreeList::SubTreeList()
{
	MALLOC_WRITE( "%p SubTreeList\n", this );
}

SubTreeItem::SubTreeItem()
{
        MALLOC_WRITE( "%p SubTreeItem\n", this );
        top = NULL;
}

PathItem::PathItem() 
{ 
        MALLOC_WRITE( "%p PathItem\n", this );
	parameters = NULL; 
}

DescribeClause::DescribeClause( UserRequest *userReq ) 
{ 
        MALLOC_WRITE( "%p DescribeClause\n", this );
	ur = userReq; 
}

CreateDatabase::CreateDatabase( UserRequest * userReq)
{
        MALLOC_WRITE( "%p CreateDatabase\n", this );
        RequestRoot::ur = userReq;
}


UseDatabase::UseDatabase( UserRequest * userReq)
{
        MALLOC_WRITE( "%p UseDatabase\n", this );
        RequestRoot::ur = userReq;
}

DropDatabase::DropDatabase( UserRequest *userReq )
{
        MALLOC_WRITE( "%p DropDatabase\n", this );
	RequestRoot::ur = userReq;
}

ClassDrop::ClassDrop( UserRequest * userReq)
{
        MALLOC_WRITE( "%p ClassDrop\n", this );
        ur = userReq;
	clsList = new LinkedList();
}

ListDatabase::ListDatabase( UserRequest *userReq )
{
        MALLOC_WRITE( "%p ListDatabase\n", this );
	RequestRoot::ur = userReq;
}

ListType::ListType(UserRequest *userReq )
{
        MALLOC_WRITE( "%p ListType\n", this );
	typeList = NULL;
	RequestRoot::ur = userReq;
}


CreateFunction::CreateFunction( UserRequest *userReq, char funcSpec )
{
        MALLOC_WRITE( "%p CreateFunction\n", this );
	RequestRoot::ur = userReq;
	fSpec = funcSpec;
}

ListFunction::ListFunction( UserRequest *userReq )
{
        MALLOC_WRITE( "%p ListFunction\n", this );
	RequestRoot::ur = userReq;
}

DisplayHierarchy::DisplayHierarchy( UserRequest *userReq )
{
        MALLOC_WRITE( "%p DisplayHierarchy\n", this );
	RequestRoot::ur = userReq;
}

StartFile::StartFile( UserRequest *userReq )
{
        MALLOC_WRITE( "%p StartFile\n", this );
	RequestRoot::ur = userReq;
}

DeleteClause::DeleteClause( UserRequest *userReq )
{
        MALLOC_WRITE( "%p DeleteClause\n", this );
        whereP           = NULL;
        projectionList   = NULL;
        bindList         = NULL;
        ur = userReq;
}

PathExpression::PathExpression(UserRequest *userReq)
{
        MALLOC_WRITE( "%p PathExpression\n", this );
        ur = userReq;
        pathItems = new LinkedList();
}

Expression::Expression(UserRequest *userReq) : Stack()
{
        MALLOC_WRITE( "%p Expression\n", this );
        ur       = userReq;
}
 
BindListItem::BindListItem( UserRequest *userReq)
{
        MALLOC_WRITE( "%p BindListItem\n", this );
        bindExpression = new Expression(userReq);
        strcpy( alias, UNDEFINED_STRING );
}

BindExpression::BindExpression( UserRequest *userReq) : LinkedList()
{
        MALLOC_WRITE( "%p BindExpression\n", this );
        ur = userReq;
}

ProjectionListItem::ProjectionListItem( UserRequest *userReq )
{
        MALLOC_WRITE( "%p ProjectionListItem\n", this );
        ur              = userReq;
        projExpression  = new Expression( userReq );
}

ProjectionExpression::ProjectionExpression( UserRequest *userReq) : LinkedList()
{
        MALLOC_WRITE( "%p ProjectionExpression\n", this );
        //projectionList = new LinkedList();
        ur = userReq;
}

DropFunction::DropFunction( UserRequest *userReq )
{
        MALLOC_WRITE( "%p DropFunction\n", this );
        RequestRoot::ur = userReq;
        strcpy( delStr, UNDEFINED_STRING );
        strcpy( className, UNDEFINED_STRING );
}



// default constructor for a list item
ListItem::ListItem()
{
	if( this != &undefinedItem )
        	MALLOC_WRITE( "%p ListItem\n", this );
        next = NULL;
        prev = NULL;
        left = NULL;
        right = NULL;
	lastTerm = NULL;
        item = NULL;
        itemType = 0;
}


// ListItem constructor with item type and item
ListItem::ListItem(unsigned itype, void *i)
{
        MALLOC_WRITE( "%p ListItem\n", this );
        next	= NULL;
	prev 	= NULL;
        left 	= NULL;
        right 	= NULL;
        lastTerm = NULL;
        item     = i;
        itemType = itype;
}

NewClause::NewClause(UserRequest *userReq)
{
        MALLOC_WRITE( "%p NewClause\n", this );
        newExpList = new LinkedList();
        attrList   = new LinkedList();
        ur = userReq;
}

Query::Query( UserRequest *userReq )
{
        MALLOC_WRITE( "%p Query\n", this );
        whereP           = NULL;
        startWithP       = NULL;
        havingP          = NULL;
        projectionList   = NULL;
        groupByList      = NULL;
        orderByList      = NULL;
        bindList         = NULL;
        recurseAttribute = NULL;
	parentQuery	 = NULL;
	subQueries	 = NULL;
        ORTerms          = NULL;
	queryTree	 = NULL;
	solutionFile     = NULL;
	useSelfVariables = NULL;
        ur               = userReq;
}


UpdateClause::UpdateClause( UserRequest *userReq )
{
        MALLOC_WRITE( "%p UpdateClause\n", this );
        whereP           = NULL;
        bindList         = NULL;
        updateExpression    = new LinkedList();
        ur = userReq;
}

UserRequest::UserRequest()
{
        if(myalloc ) MALLOC_WRITE( "%p UserRequest\n", this );
        text     = (char *) myAlloc(MAXTEXTLENGTH);
        if(myalloc ) MALLOC_WRITE( "%p char\n", text );
        maxTextLength = MAXTEXTLENGTH;
        curpos  = 0;
        prevpos = 0;
        userError.setError( NO_ERROR, 0, "");
        operation = NULL;
}

UserRequest::UserRequest(char *inText)
{
        MALLOC_WRITE( "%p UserRequest\n", this );
        text     = (char *) myAlloc(MAXTEXTLENGTH);
        MALLOC_WRITE( "%p char\n", text );
        maxTextLength = MAXTEXTLENGTH;
        curpos = 0;
        prevpos = 0;
        strcpy(text, inText);
        userError.setError( NO_ERROR, 0, "");
        operation = NULL;
}

void freeAttribute( RELATION_SCHEMA *rs )
{
	int  i;
	if( !rs ) return;
        for(i=0; i<rs->arity; i++ ) {
		myFree((char *) rs->attr[i] );
		rs->attr[i] = NULL;
	}
	rs->arity = 0;
}

void freeFCN_INPUT( FCN_INPUT *funcIn )
{
	int i;
	if( !funcIn ) return;

	// delete function parameters
	for( i = 0; i<funcIn->argnum; i++ )
		freeJoinArg( funcIn->ParExpr[i] );

	// delete myself
	myFree( (char *) funcIn );
}

void freeJoinArg( JOIN_ARG *pred )
{
	if( !pred ) return;

	// delete if a function exists
	if( pred->fcn ) freeFCN_INPUT( pred->fcn );

	// delete left
	freeJoinArg( pred->left );
	myFree( (char *) pred->left );

	//delete right
	freeJoinArg( pred->right );
	myFree( (char *) pred->right );

}

void freeQUERY( QUERY * aplan )
{
        if(!aplan) return;
        switch(aplan->my_operator )
        {
                case JOIN :
                        freeJoinArg( &aplan->argument.join);
                        freeQUERY( (QUERY *) aplan->input[0] );
                        freeQUERY( (QUERY *) aplan->input[1] );
                        break;
                case SELECT :
                        freeJoinArg( aplan->argument.select.cond[0] );
                        myFree((char *) aplan->argument.select.cond[0] );
                        freeQUERY( (QUERY *) aplan->input[0] );
                        break;
                case GET_SET :
                        freeAttribute( &aplan->argument.get_set.input );
                        freeQUERY( (QUERY *) aplan->input[0] );
                        break;
                case MAT :
                        freeAttribute( &aplan->argument.mat.input[0] );
                        freeQUERY( (QUERY *) aplan->input[0] );
                        break;
                case UNNEST :
                        freeQUERY( (QUERY *) aplan->input[0] );
                        break;
                default :
			break;
                        //printf(" Unknown Operator in freeQUERY\n ");
        }
}


void freePLAN( PLAN *plan )
{
	int	i;
	ALGORITHM_ARGUMENT	*alArg;

	if( !plan ) return;

	alArg =  &plan->algorithm_argument;

	switch( plan->algorithm )
	{

	case FILE_SCAN :
		freeAttribute( &alArg->file_scan.input );
		freePLAN( plan->input[0] );
		break;

	case FILTER :
		for( i = 0; i < alArg->filter.terms; i++ )
			freeJoinArg( alArg->filter.cond[i] );
		freePLAN( plan->input[0] );
		break;

	case TRAVERSE :
		for( i = 0; i < alArg->traverse.num; i++ ) 
			freeFCN_INPUT( alArg->traverse.fcn[i] );
		freeAttribute( &alArg->traverse.input[0] );
		freePLAN( plan->input[0] );
		break;

	case HASH_JOIN : 
		freeJoinArg( &alArg->hash_join );
		freePLAN( plan->input[0] );
		freePLAN( plan->input[1] );
		break;

	case NESTED_LOOP_JOIN :
		freeJoinArg( &alArg->nested_loop_join );
		freePLAN( plan->input[0] );
		freePLAN( plan->input[1] );
                break;
	
	case MERGE_JOIN :
		freeJoinArg( &alArg->merge_join );
		freePLAN( plan->input[0] );
		freePLAN( plan->input[1] );
		break;

	case BTREE_SELECT:
          	freeAttribute( &alArg->btree_select.input );
		freeJoinArg( alArg->btree_select.cond );
		myFree( (char *)  alArg->btree_select.cond );
		freePLAN( plan->input[0] );
		break;

	case PTR_HH_JOIN:
		freeAttribute( &alArg->ptr_hh_join.input );
		freeFCN_INPUT( alArg->ptr_hh_join.fcn );
		freePLAN( plan->input[0] );
		break;

	case PTR_SM_JOIN :
		freeAttribute( &alArg->ptr_sm_join.input );
		freeFCN_INPUT( alArg->ptr_sm_join.fcn );
		freePLAN( plan->input[0] );
		break;

	case UNNEST_ALG :
		freeAttribute( &alArg->unnest_alg.input );
		freePLAN( plan->input[0] );
		break;

	case FIX_ALG :
		freeAttribute( &alArg->fix_alg.input );
		freePLAN( plan->input[0] );
		break;

        case UNION_ALG :
	case SORT :
	default : 
		break;
	//	printf(" Undefined algorithm in freePLAN\n");
	}

	freeAttribute((RELATION_SCHEMA *) &plan->logical_property );
	myFree( (char *) plan );
}

void freeExecVars( PLAN *plan1, PLAN *plan2, JOIN_ARG ** prList, int pNum)
{
        int i;
        if( plan1 == plan2 ) freePLAN( plan1 );
        else {
                if( plan2 ) freeAttribute( &plan2->logical_property );
                myFree( (char *) plan2 );
                freePLAN( plan1 );
        }
        for( i = 0; i<pNum ; i++ )
                myFree( (char *) prList[i] );
        myFree( (char *) prList );
}

