#include <Cursor.h>
#include "mood_parser.h"
#include <lists.h>
#include <set.h>
#include <mood_catalog.h>
#include <memory.h>

extern int DecRefCount(OID Oid, int *RefCount, int Self);
extern int PrintRefCounts();

int DeleteClause::execute()
{
	Query		*query;
	char		*QryClName;
	MOODSCursor	ResCur;
	char		Flag;
	OID		oid;
	int		i, RefCount;
        char            DeletedBefore;

	// Convert DELETE Clause to a SELECT Clause and execute it...
	query = new Query(ur);

	query->projectionList = projectionList;
	query->bindList = bindList;
	query->whereP = whereP;

	// Check type of the query 
	QryClName = query->checkAltintas();

	if (! query->generateAllTree()) return FALSE;
	if (! query->generatePlan()) return FALSE;

	OpenCursor(query->solutionFile,&ResCur);

	if (!GetFirstObject(&ResCur,&Flag))
		return(FALSE);

        memset(&oid,0,12);
	while (!Flag) {
                DeletedBefore = 0;

                if (!memcmp((char *)&oid,(char *)ResCur.CurUserdesc->basePtr,12))
                   DeletedBefore = 1;
              
		memcpy((char *)&oid, (char *)ResCur.CurUserdesc->basePtr, sizeof(OID));

                if (!DeletedBefore) {
			if (!DeleteObject(&oid,QryClName,TRUE))
				return FALSE;
                }

		// Get next object from the cursor...
		if (!GetNextObject(&ResCur,&Flag))
			return(FALSE);

	} // while
 
	if (!CloseCursor(&ResCur))
		return(FALSE);

	PrintRefCounts();

	return TRUE;

} // execute


int DeleteClause::DeleteObject( OID *oid, char *className, int Self )
{
	int	RefCount;

	if (!DecRefCount(*oid,&RefCount,Self)) {
		ur->userError = catalogError;
		ur->operationType = HANDLE_ERROR;
		return FALSE;
	}

	if (RefCount == 0) {
		if (!DeleteRSL(className,*oid)) return FALSE;
		if (!DeleteObj(*oid)) return FALSE;
	} // if

	return TRUE;
}

int DeleteClause::DeleteSet( OID *oid, char *className )
{
	CatBOOLEAN	status = FALSE;
	int		s_type, s_size, i;
	OID		element;
	Set		*set;

	set = new Set( oid, &s_type, &s_size, &status );
	if( status ) return FALSE;

	for( i = 0; i < s_size; i++ ){
		set->getElement( &element, i, &status );
		if( status ) return FALSE;
		if( !DeleteObject( &element, className, FALSE ) ) return FALSE;
	}

	return TRUE;
}

int DeleteClause::DeleteList( OID *oid, char *className )
{
        CatBOOLEAN      status = FALSE;
        int             l_type, l_size, i;
        OID             element;
        List		*list;

	list = new List( oid, &l_type, &l_size, &status );
        if( status ) return FALSE;

        for( i = 0; i < l_size; i++ ){
                list->getElement( &element, i, &status );
                if( status ) return FALSE;
                if( !DeleteObject( &element, className, FALSE ) ) return FALSE;
        }

        return TRUE;
}

int DeleteClause::DeleteObj(OID oid)
{
	int GrNdx, err;

	err = sm_OpenBufferGroup(4,BF_MRU,&GrNdx,NOFLAGS);
	if (err != esmNOERROR) {
		ur->setError( EXODUS_ERROR, 1, "" );
		return FALSE;
	}

	err = sm_DestroyObject(GrNdx,&oid);
	if (err != esmNOERROR) {
		ur->setError( EXODUS_ERROR, 1, "" );
		return FALSE;
	}

	err = sm_CloseBufferGroup(GrNdx);
	if (err != esmNOERROR) {
		ur->setError( EXODUS_ERROR, 1, "" );
		return FALSE;
	}

	return TRUE;

} // deleteObj

int DeleteClause::DeleteRSL(char *ClassName, OID oid)
{
	LinkedList	*attrList;
	ListItem	*currAItem;
	int		i, err, GrNdx;
        CatBOOLEAN      status;
        MoodsTypes      catMngr;
        Catalog         classInfo;
        AttrList        *attributes, *currAttr;
	char		*Object;
	USERDESC	*UserDesc;

	err = sm_OpenBufferGroup(4,BF_MRU,&GrNdx,NOFLAGS);
	if (err != esmNOERROR) {
		ur->setError( EXODUS_ERROR, 1, "" );
		return FALSE;
	}

	// Get class information from the catalog...
	catMngr.getClassInfo(ClassName,&classInfo,&attributes,&status);
	if (status != 0) {
		ur->userError = catalogError;
		ur->operationType = HANDLE_ERROR;
		return FALSE;
	}

	attrList = new LinkedList;

	// Convert attribute tree to a linked list...
	convertToList(attributes, attrList);

        Object = (char *)myAlloc(classInfo.Size);

	err = sm_ReadObject(GrNdx, &oid, 0, READ_ALL, &UserDesc);
	if (err != esmNOERROR) {
		ur->setError( EXODUS_ERROR, 1, "" );
		return FALSE;
	}

	memcpy(Object, (char *)UserDesc->basePtr, classInfo.Size);

	err = sm_ReleaseObject(UserDesc);
	if (err != esmNOERROR) {
		ur->setError( EXODUS_ERROR, 1, "" );
		return FALSE;
	}

	// for each REF, SET, LIST attribute,  decrement refcounts...

	i = 1;
	currAItem = attrList->firstItem->next;

	while (currAItem) {

		currAttr  = (AttrList *) (currAItem->item);

		switch (currAttr->entry.AttributeConstructor)  {
		case 'N': {
			if (!strcmp(currAttr->entry.AttrTypeName,"OID")) {
				OID Oid;

				Oid = *((OID *)(Object+currAttr->entry.AttributeOffset));
				if (!OID_IS_INVALID(Oid)) {
					DeleteObj(Oid);
				}
			}
			break;
		}
		case 'R': {
			OID	RefOid;

			RefOid = *((OID *)(Object+currAttr->entry.AttributeOffset));

			if (!OID_IS_INVALID(RefOid)) {
				DeleteObject(&RefOid,(currAttr->nextsub)->entry.AttrTypeName,FALSE);
			}

			break;
		}
		case 'S': {
			OID	SetOid;

			SetOid = *((OID *)(Object+currAttr->entry.AttributeOffset));

			if (!OID_IS_INVALID(SetOid)) {
				DeleteSet(&SetOid,(currAttr->nextsub)->entry.AttrTypeName);
			}

			break;
		}
		case 'L': {
			OID	ListOid;

			ListOid = *((OID *)(Object+currAttr->entry.AttributeOffset));

			if (!OID_IS_INVALID(ListOid)) {
				DeleteList(&ListOid,(currAttr->nextsub)->entry.AttrTypeName);
			}

			break;
		}
		default : break;

		} // switch

		i++;
		currAItem = currAItem->next;

	} // while

	err = sm_CloseBufferGroup(GrNdx);
	if (err != esmNOERROR) {
		ur->setError( EXODUS_ERROR, 1, "" );
		return FALSE;
	}

	myFree(Object);

	// deallocate LinkedList attrList ???

	return TRUE;

} // DeleteRSL

void DeleteClause::convertToList(AttrList *inAttr, LinkedList *attrList)
{
// Ilker, AttrList daha sonra delete edilebilir. Bu durumda attrList de delete
// edilecek olursa ayni yerler iki defa delete edilirse sorun cikabilir.
// Yuksel
	if (!inAttr) return;

	if (inAttr->entry.AttributeConstructor == 'T')
		convertToList(inAttr->nextsub, attrList);

	if (inAttr->entry.AttributeConstructor != 'T')
		attrList->insert(ATR_PTR, VOID inAttr);

	convertToList(inAttr->next, attrList);

} // convertToList

