
#include <stdio.h>
#include "linklist.h"
#include "keywords.h"
#include "reqroot.h"
#include "mood_catalog.h"


/************************************************************************

 complexExpression := expression compExpPrime
 compExpPrime :=  e | [EACH] [ = | <> | < | > | >= | <= | CONTAINS

************************************************************************/


char *PathItem::setItemName( char *item )
{
        return strcpy(itemName, item );
}
 
void PathItem::printOn()
{
        printf(" %s ", itemName);
	printf("Parameters:\n");
	if( parameters ) parameters->printOn();
}
 
void PathItem::setItemType( char  t )
{
        itemType = t;
}


void PathExpression::printOn()
{
	printf("PATHEXP(");
        pathItems->printOn();
	printf(") ");
}


PathExpression::parseParamList( PathItem *pItem )
{
	Expression	*exp;

	getToken();

	if( tokenType() == RPAR ) return TRUE;

	backtrack(); // parameterless function

	pItem->parameters = new LinkedList();

	do {
		exp =  new Expression( ur );
		
		if( !exp->parse() ) {
			delete exp;
			return FALSE;
		}

		pItem->parameters->insert( FUNCTION_PARAM, VOID exp );

		getToken();
		
	} while ( tokenType() == COMMA );

	if( tokenType() != RPAR ) {
		setError(PARSE_ERROR, MISSING_RP, "");
                return FALSE;
        }

	return TRUE;
	
}


PathExpression::parse()
{
	PathItem	*p;

	do {

		p = new PathItem();

		getToken();

		if( tokenType() == IDENTIFIER ) {

			p->setItemName( tokenValue() );

			p->setItemType( PATH_ATTRIBUTE );

			getToken();

			if( tokenType() == LPAR ) {

				p->setItemType( PATH_FUNCTION );

				parseParamList( p );

			}
			else backtrack();

			pathItems->insert( PATH_ITEM, VOID p );

		}
		else if( tokenType() == STAR ) {

			p->setItemType( FUNCTION_PARAM );

			pathItems->insert( PATH_ITEM, VOID p );

			// must be last item in path exp.
			break;
		}
		else {
			delete p;
			return FALSE;
		}

		getToken();

	} while( tokenType() == DOT );

	backtrack();

	return TRUE; 
}

 
void Expression::push(int itype, void *item)
{
        Stack::push(itype, item);
}


Expression::identifier()
{
/*
	identifier can be one of the following types :
		1. String Constant
		2. Character Constant
		3. Integer
		4. Real
		5. Null Keyword
	 	6. Path expression
*/
#ifdef DEBUG
	printf(" < I ");	
#endif
	getToken();
	switch(tokenType())
	{
		case STRCONS  :
		case CHRCONS  :
			char *p;
			p = (char *) myAlloc( strlen(tokenValue())+1 );
			strcpy(p, tokenValue());
			push( tokenType(), VOID p );
#ifdef DEBUG
			printf(" > ");
#endif
			return TRUE;

		case UNSIGNEDINT :
		{
			unsigned int *p;
			p = (unsigned int *) myAlloc( sizeof(unsigned int));
			*p = atoi(tokenValue());
			push( tokenType(), VOID p );
#ifdef DEBUG
			printf(" > ");
#endif
			return TRUE;
		}

		case UNSIGNEDREAL :
		{
			float *p;
			p = (float *) myAlloc(sizeof(float));
			*p = atof(tokenValue());
			push( tokenType(), VOID p );
#ifdef DEBUG
			printf(" > ");
#endif
			return TRUE;
		}

		case NULLVALUE         :
		{
			push( NULLVALUE, NULL);
#ifdef DEBUG
			printf(" > ");
#endif
			return TRUE;
		}

		case MS_TRUE :
		{
			push( MS_TRUE, NULL );
			return TRUE;
		}

		case MS_FALSE :
		{
			push( MS_FALSE, NULL );
			return TRUE;
		}

		default		  :
		{
			PathExpression *p;
			p  = new PathExpression( ur );

			backtrack();

			if( ! (p->parse()) ) {
				delete p;
				return FALSE;
			}

			push( PATH_EXP, VOID p );

			return TRUE;
		}
	}

}

Expression::factor()
{

	getToken();

#ifdef DEBUG
	printf(" < F --> ");
#endif

	switch(tokenType())
	{
		case EVERY :
#ifdef DEBUG
			printf(" EVERY F ");
#endif
			if(!factor()) return FALSE;
	                push( EVERY, NULL);
#ifdef DEBUG
			printf(" > ");
#endif
       		        return TRUE;

		case PLUS  :
#ifdef DEBUG
			printf(" PLUS F ");
#endif
			if(!factor()) return FALSE; 
                        push( UNARYPLUS, NULL); 
#ifdef DEBUG
			printf(" > ");
#endif
                        return TRUE;
 

		case MINUS :
#ifdef DEBUG
			printf(" MINUS F ");
#endif
			if(!factor()) return FALSE;  
                        push( UNARYMINUS, NULL);  
#ifdef DEBUG
			printf(" > ");
#endif
                        return TRUE;

		case NOT   :
#ifdef DEBUG
			printf(" NOT F ");
#endif
			if(!factor()) return FALSE;
                        push( NOT, NULL);
#ifdef DEBUG
			printf(" > ");
#endif
                        return TRUE;

		case LPAR  : 
#ifdef DEBUG
			printf(" ( CE ) ");
#endif
			if(!complexExpression()) return FALSE;
			CHECK_TOKEN( RPAR, MISSING_RP, "");
#ifdef DEBUG
			printf(" > ");
#endif
			return TRUE;

		default    :
			backtrack();
#ifdef DEBUG
			printf(" I ");
#endif
			if(!identifier()) return FALSE;
#ifdef DEBUG
			printf(" > ");
#endif
			return TRUE;
	}
}

Expression::termPrime()
{
	getToken();

#ifdef DEBUG
	printf(" < TP --> ");
#endif

	switch(tokenType())
	{
		case AND     :
#ifdef DEBUG
			printf(" AND F TP ");
#endif
                	if(!factor()) return FALSE;
                	push( AND, NULL );
                	if(!termPrime()) return FALSE;
#ifdef DEBUG
			printf(" >");
#endif
			return TRUE;

		case STAR    :
			if(!factor()) return FALSE; 
                        push( MUL, NULL ); 
                        if(!termPrime()) return FALSE;
#ifdef DEBUG
			printf(" >");
#endif
			return TRUE;
 
		case SLASH   :
			if(!factor()) return FALSE; 
                        push( DIV, NULL ); 
                        if(!termPrime()) return FALSE; 
#ifdef DEBUG
			printf(" >");
#endif
			return TRUE;

		case PERCENT :
			if(!factor()) return FALSE; 
                        push( MOD, NULL ); 
                        if(!termPrime()) return FALSE; 
#ifdef DEBUG
			printf(" >");
#endif
			return TRUE;

		default      :
			backtrack();
			return TRUE;
	}
}

Expression::term()
{
#ifdef DEBUG
	printf(" < T --> T TP ");
#endif
	if(!factor()) return FALSE;
	if(!termPrime()) return FALSE;
#ifdef DEBUG
	printf(" > ");
#endif
	return TRUE;
}

Expression::expressionPrime()
{

#ifdef DEBUG
	printf("< EP --> "); 
#endif
	getToken();

	switch(tokenType())
	{
		case PLUS  :
#ifdef DEBUG
			printf(" T EP ");
#endif
			if(!term()) return FALSE;
                	push( ADD, NULL );
                	if(!expressionPrime()) return FALSE;
#ifdef DEBUG
			printf(" > ");
#endif
			return TRUE;

		case MINUS :
#ifdef DEBUG
			printf(" T EP ");
#endif
                        if(!term()) return FALSE; 
                        push( SUB, NULL ); 
                        if(!expressionPrime()) return FALSE;
#ifdef DEBUG
			printf(" > ");
#endif
			return TRUE;

		case OR    :
#ifdef DEBUG
			printf(" T EP ");
#endif
                        if(!term()) return FALSE; 
                        push( OR, NULL ); 
                        if(!expressionPrime()) return FALSE;
#ifdef DEBUG
			printf(" > ");
#endif
			return TRUE;

		default    :
#ifdef DEBUG
			printf(" nil >\n");
#endif
			backtrack();
			return TRUE;
	}

}

Expression::expression()
{
#ifdef DEBUG
	printf(" < E --> T EP");
#endif
	if(!term()) return FALSE;
	if(!expressionPrime()) return FALSE;
#ifdef DEBUG
	printf(" >\n ");
#endif
	return TRUE;
}

Expression::compExpPrime()
{

	int eachIndex = 0, operatorType;

#ifdef DEBUG
	printf(" < CP --> ");
#endif

	getToken();

	/*if(tokenType() == EACH) 
	{
		eachIndex = 20;
		getToken();
	} */

	switch(tokenType())
	{
		case EQ		:
		case NE		:
		case GT		:
		case GE		:
		case LT		:
		case LE		:
		case CONTAINS   :
		case ISSUBSET	:
		case IN         :	
			operatorType = eachIndex + tokenType();
			break;

		/*case IS		:
			getToken();
			if(tokenType() == NOT) 
			else backtrack();

		case NOT	:
			getToken();
			if(tokenType() == IN) */

		default		:
#ifdef DEBUG
			printf(" nil > ");
#endif
			backtrack();
			return TRUE;
	}

#ifdef DEBUG
	printf(" OP E ");
#endif
	if(!expression()) return FALSE;

	push( operatorType, NULL);

#ifdef DEBUG
	printf(" >\n");
#endif

	return TRUE;

}

Expression::complexExpression()
{
#ifdef DEBUG
	printf(" < CE --> E CP ");
#endif

	if(!expression()) return FALSE;
	if(!compExpPrime()) return FALSE;

#ifdef DEBUG
	printf(" >\n");
#endif
	return TRUE;
}

Expression::parse()
{
        if(!complexExpression()) return FALSE;
	stack2tree( lastItem );
	firstItem->prev = NULL;
        return TRUE;
}

void Expression::printOn()
{
	Stack::printOn();
	printf("\nBACKWARD\n");
	Stack::bprintOn();
	printf("\nAS TREE\n");
	Stack::traverseAsTree( lastItem );
	printf("\n");
}


void Expression::normalize( AndTerms *orTerms)
{
        firstItem->prev = NULL; 
        extractAndedTerms(orTerms, lastItem );
}




/*********************************************************************

	Code for parsing the expression comes after FROM clause.

**********************************************************************/

void BindListItem::printOn()
{
        printf("\nBITEM alias : %s\n", alias);
        bindExpression->printOn();
        printf("\n");
}
 
char* BindListItem::setBindVariable( char *aName)
{
        return strcpy(alias, aName);
}
 
void BindExpression::printOn()
{
        printf("\nBEXP(");
        LinkedList::printOn();
        printf(")\n");
}


BindListItem* BindExpression::contains( char *bindVar )
{
	ListItem *p = firstItem;

	if( !p ) return NULL;

	while(p)
	{
		if(!strcmp( bindVar, (( BindListItem *)p->item)->alias ))
			return ( (BindListItem * ) p->item);
		p = p->next;
	}

	return NULL;
}

char* BindExpression::getClassName( char *bindVar )
{
	BindListItem	*p=NULL;
	register char	*clsName=NULL;
	p = contains( bindVar );
	if(p)
	{
		clsName = (char *) myAlloc(80);
		strcpy(clsName, (char *) p->classInfo.name );
	}
	return clsName;
} 

 
BindExpression::parse()
{
	BindListItem	*anItem;

	do {

                anItem = new BindListItem( ur ); // create a new bind item

		// parse bind expression
		if(! anItem->bindExpression->parse() ) 
		{
			delete anItem;
			return FALSE;
		}

		CHECK_TOKEN( IDENTIFIER, BIND_VAR_EXPECTED, "");


		anItem->setBindVariable( tokenValue() );

		getToken();

/*   *******************
		BURASI SONRAKI VERIONA KALDI
		getToken();

		if( tokenType() == IDENTIFIER ) // check bind variable 
		{
			anItem->setBindVariable( tokenValue() );

			getToken();
		} 
*/

                insert( BIND_ITEM, VOID anItem ); // insert item


        } while( tokenType() == COMMA);

        backtrack();

        return TRUE;
	
}


/***********************************************************************
		
	parses the statement after the keywords SELECT, GROUP BY,
	DELETE, ORDER BY

************************************************************************/


void ProjectionListItem::printOn()
{
        projExpression->printOn();
}


ProjectionListItem::parse()
{
#ifdef DEBUG
        printf("Projection List Item\n");
#endif
        return projExpression->parse();
}



void ProjectionExpression::printOn()
{
        //projectionList->printOn();
	LinkedList::printOn();
}


ProjectionExpression::parse()
{

#ifdef DEBUG
	printf("Projection Expression\n");
#endif

	ProjectionListItem	*anItem;

	do {
		anItem = new ProjectionListItem(ur);

		if(!anItem->parse() ) 
		{
			delete anItem;
			return FALSE;
		}

		insert( PROJ_ITEM, VOID anItem );

		getToken();

	} while( tokenType() == COMMA );

        backtrack();

        return TRUE;
}


/*


parameterList()
{
        Path *apath;
        GenStack *gs;
#ifdef DEBUG
        printf("Paramater List ");
#endif

        if((token = gettoken()).type != RPAR) {
                backtrack();   
                do{
#ifdef DEBUG
                        printf("parameter i ");
#endif
                        gs = (GenStack *) myMalloc(sizeof(GenStack));
                        if(!Expression(gs)) return FALSE;
                        token = gettoken();
                } while(token.type == COMMA);
        }

        if(token.type != RPAR){
                setError(PARSE_ERROR, MISSING_RP, "");
                return FALSE;  
        }

        return TRUE;
}

variable(  Path *apath)
{
        char varname[TOKENSIZE];
        int  vartype, listlower, listupper;
        void *info;

        if((token=gettoken()).type != IDENTIFIER ) {
                backtrack();   
                return(-1);
        }

        strcpy(varname,token.value);

        token = gettoken();
        if(token.type == LPAR) {
                if(!parameterList()) return FALSE;
                FillPath(apath,FUNCTION,varname,NULL);
                return TRUE;
        }
 
        if(token.type == LB) {
                ListItem *listinfo;
 
                CHECK_TOKEN(UNSIGNEDINT, MISSING_RP, "");
 
                listlower = atoi(token.value);
                vartype = LISTITEM;// list item 
 
                listupper = -1; // set list upper limit to -1 
                token=gettoken();
                if(token.type == COMMA ) {
 
                        CHECK_TOKEN(UNSIGNEDINT, MISSING_RP, "");
 
                        listupper = atoi(token.value);
                        if(listlower>=listupper) {
                                setError(PARSE_ERROR, 2, "");
                                return FALSE;
                        }
                }
                else backtrack();
                token = gettoken();
                if(token.type!=RB) {
                        setError(PARSE_ERROR, 3, "");
                        return FALSE;
                }
                vartype = LIST; // means sublist 
                listinfo = (  ListItem *) myMalloc(sizeof(  ListItem));
                listinfo->lower = listlower;
                listinfo->upper = listupper;
                FillPath(apath,vartype,varname,info);
                return TRUE;
        }
 
        backtrack();
        FillPath(apath,REF,varname,NULL); // either a set ot atomic attribute
        return TRUE;
}
 
SetConstant()
{
        int settype;
        if((token=gettoken()).type==LCB) {
                token=gettoken();
                if(token.type==RCB) return TRUE;
                else {
                        settype = token.type; // Set settype to first item type
                        if((token=gettoken()).type == COMMA){
                                do {
                                        token = gettoken(); // Fetch next item in the set
                                        if(token.type != settype){
                                                setError(PARSE_ERROR, 4, "");
                                                return FALSE;
                                        }
                                        token =gettoken();
                                } while(token.type == COMMA);
                        }
 
                        if(token.type != RCB) {
                                setError(PARSE_ERROR, 20, "");
                                return FALSE;
                        }
                }
        }
        else {
                backtrack();
                return TRUE;
        }
        return TRUE;
}

pathExpression( PathExp **apathexp )
{
        Path *apath;
        *apathexp = (PathExp *) myMalloc(sizeof(PathExp));
        unsigned char flag=1;
        int forvar;

        CHECK_TOKEN(IDENTIFIER, BIND_VAR_EXPECTED, "");
 
        strcpy((*apathexp)->BindVar,token.value);
        if((token=gettoken()).type!=DOT)
        {
                backtrack();
                (*apathexp)->path = NULL;
                return TRUE;
        }
 
        apath = (Path *) myMalloc(sizeof(Path));
        apath->no_of_items = 0;
        (*apathexp)->path = apath;
 
        while( flag ) {
                if((forvar = variable(apath)) == FALSE) return FALSE;
                if(forvar == 1) {
                        if((token=gettoken()).type!=DOT){
                                backtrack();
                                flag=0;
                        }
                }
                else flag=0;
        }
        apath->no_of_items--;
        return TRUE;
}
 



*/
