
#include "opt_defs.h"
#include "dbi_gen.h"
#include "opt.h"
#include "dbi.h"
#include "global.h"
#include "mood_parser.h"
#include "userreq.h"
#include "mood_catalog.h"
#include <stdio.h>
#include "index.h" 

extern displayFID( FID *);

#define MAXTYPENO 10
#define INVLDT	  9

char *moodTypeNames[MAXTYPENO] = { 
	"char",    // 0
	"string",  // 1 
	"float",   // 2
	"int",     // 3
	"boolean", // 4 
	"REF",     // 5 
	"SET",     // 6 
	"LIST",    // 7
	"nulltype",    // 8
	"invalid$type" //9 
};

int comparisonOpRetType[ MAXTYPENO ] [ MAXTYPENO ] = {
/*		0	1	2	3	4	5	6	7	8    9 */
/* 0 */ {	4,	4,	4,	4, INVLDT, INVLDT, INVLDT, INVLDT, 	4, INVLDT  },
/* 1 */ {	4,	4, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, 	4, INVLDT   },
/* 2 */ {       4, INVLDT,	4,	4, INVLDT, INVLDT, INVLDT, INVLDT, 	4, INVLDT  },
/* 3 */ {       4, INVLDT,	4,	4, INVLDT, INVLDT, INVLDT, INVLDT, 	4, INVLDT   },
/* 4 */ {  INVLDT, INVLDT, INVLDT, INVLDT, 	4, INVLDT, INVLDT, INVLDT, 	4, INVLDT  },
/* 5 */ {  INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, 	4, 	4, 	4, 	4, INVLDT  },
/* 6 */ {  INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, 	4, 	4, 	4, 	4, INVLDT  },
/* 7 */ {  INVLDT, INVLDT, INVLDT, INVLDT, INVLDT,	4, 	4, 	4, 	4, INVLDT  },
/* 8 */ {  	4, 	4, 	4, 	4, 	4, 	4, 	4,	 4, 	4, INVLDT  },
/* 9 */ {  INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT  }
};

int arithmeticOpRetType[ MAXTYPENO ] [ MAXTYPENO ] = {
/*		0	1	2	3	4	5	6	7	8     9 */
/* 0 */ {	0, INVLDT,	2,	3, INVLDT, INVLDT, INVLDT, INVLDT, 	8, INVLDT    },
/* 1 */ {  INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, 	8, INVLDT    },
/* 2 */ {       2, INVLDT,	2,	2, INVLDT, INVLDT, INVLDT, INVLDT, 	8, INVLDT    },
/* 3 */ {       3, INVLDT,	2,	3, INVLDT, INVLDT, INVLDT, INVLDT, 	8, INVLDT    },
/* 4 */ {  INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, 	8, INVLDT    },
/* 5 */ {  INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, 	8, INVLDT    },
/* 6 */ {  INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, 	8, INVLDT    },
/* 7 */ {  INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, 	8, INVLDT    },
/* 8 */ {  	8, 	8, 	8, 	8, 	8, 	8, 	8, 	8, 	8, INVLDT    },
/* 9 */ {  INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT, INVLDT    }
};

int findTypeIndex( char *typeName )
{
	int i;
	for( i = 0; i < MAXTYPENO-1; i++ )
		if( !strcmp( moodTypeNames[i], typeName ) ) return i;
	return i;
}

void lowerCase( char *source, char *dest )
{
	while( *source ) {
		if( *source >= 'A' && *source <= 'Z' ) 
			*dest = 'a' + *source - 'A';
		else *dest = *source;
		source++;
		dest++;
	}
	*dest = '\0';
}

char *findComparisonRetType( char *leftTypeName, char *rightTypeName )
{
	int lind, rind;
	lind = findTypeIndex( leftTypeName );
	rind = findTypeIndex( rightTypeName );
	return moodTypeNames[ comparisonOpRetType [lind] [rind] ];
}

char *findArithmeticRetType( char *leftTypeName, char *rightTypeName )
{
	int lind, rind;
	lind = findTypeIndex( leftTypeName );
	rind = findTypeIndex( rightTypeName );
	return moodTypeNames[ arithmeticOpRetType [lind] [rind] ];
}



void addFunctionAtt( RELATION_SCHEMA *rs, ATTRIBUTE *att)
{
	int i;

	// do not add if attribute exists
	for( i = 0; i<rs->arity; i++)
		if( !strcmp( rs->attr[i]->name, att->name ) ) return;


	for( i = rs->arity; i > 0; i-- )
		rs->attr[i] = rs->attr[i-1];
	rs->arity++;
	rs->attr[0] = att;

//	rs->attr[rs->arity] = att;
//	rs->arity++;
}

void copyFunctionAtt( RELATION_SCHEMA *dest, RELATION_SCHEMA *source)
{
	int i;
	ATTRIBUTE *att;

	for( i = 0; i<source->arity; i++) {
		if( source->attr[i]->attr_type == METHOD ) {
			att = (ATTRIBUTE *) myAlloc( sizeof( ATTRIBUTE ) );
			*att = *source->attr[i];
			addFunctionAtt( dest, att );
		}
	}
} 


addFunctionToAll( ATTRIBUTE *funcAtt, ListItem *addTo, RELATION_SCHEMA *currSchema )
{
	RELATION_SCHEMA *prevSchema;
	ATTRIBUTE	*currFuncAtt, *prevAtt;
	char		currItemName[2*MAXSTRLEN];

	if( !addTo ){
		addFunctionAtt( currSchema, funcAtt );
		return TRUE;
	}

	strcpy( currItemName, ((PathItem *) addTo->item)->itemName ); 

	// add function attribute to current schema
	addFunctionAtt( currSchema, funcAtt );

	prevSchema = ((PathItem *) addTo->item)->prevSchema;
	
	// copy input function attribute
	currFuncAtt = (ATTRIBUTE *) myAlloc( sizeof( ATTRIBUTE ) );
	*currFuncAtt = *funcAtt;

	// find the itemName offset in the previous schema
	prevAtt = prevSchema->contains( currItemName);

	// update offset of the function and function name
	if( prevAtt ) {
		currFuncAtt->offset = funcAtt->offset + prevAtt->offset;
		sprintf( currFuncAtt->name, "%s$%s", currItemName, funcAtt->name );
	}

	if( ((PathItem *) addTo->item )->dbType == 1 )
		addFunctionToAll(currFuncAtt,addTo->prev, prevSchema );
	else 
		addFunctionToAll(currFuncAtt, NULL, prevSchema );
	return TRUE;

}




void printSchema()
{
}

void printAtt( AttrList *inAtt )
{
	if( !inAtt ) return;
	printf(" For the Attribute : \n");
	printf(" Name   : %s \n", inAtt->entry.AttributeName );
	printf(" Type   : %s \n", inAtt->entry.AttrTypeName );
	printf(" Offset : %d \n", inAtt->entry.AttributeOffset );
	printf(" Length : %d \n", inAtt->entry.AttributeLength );
	printf(" Cons.  : %c \n", inAtt->entry.AttributeConstructor );
}


void printAllAtt( AttrList *inAtt )
{
	if( !inAtt ) return ;
	printf(" For the Attribute : \n");
	printf(" Name   : %s \n", inAtt->entry.AttributeName );
	printf(" Type   : %s \n", inAtt->entry.AttrTypeName );
	printf(" Offset : %d \n", inAtt->entry.AttributeOffset );
	printf(" Length : %d \n", inAtt->entry.AttributeLength );
	printf(" Cons.  : %c \n", inAtt->entry.AttributeConstructor );
	printf(" NEXT SUB \n ");
	printAllAtt( inAtt->nextsub );
	printf( " NEXT \n ");
	printAllAtt( inAtt->next );
	printf("-----------\n");
	
}

copyConsAtt( ATTRIBUTE *anAtt, char *prefix, char constructor, AttrList *p, UserRequest *ownerReq, int preOffset )
{
	AttrList	*q;

	// get the structure of next element	
	q = p->nextsub;
	
	// This version allows only class/type names in the constructed element 
	if( q->entry.AttributeConstructor != 'N' )
	{
		ownerReq->setError( PARSE_ERROR, VERSION_ERR, "constructor inside constructor");
	  return FALSE;
	}

	strcpy( anAtt->name, prefix );
	strcat( anAtt->name, p->entry.AttributeName );
	anAtt->attr_width = p->entry.AttributeLength;
	anAtt->offset = preOffset + p->entry.AttributeOffset;
	strcpy(anAtt->class_name, q->entry.AttrTypeName);

	switch( constructor )
	{
		case 'S' : anAtt->attr_type = MS_SET; break;
		case 'L' : anAtt->attr_type = LIST; break;
		case 'R' : anAtt->attr_type = REF; break;
	}

	return TRUE;
}

copyAtomicAtt( ATTRIBUTE *anAtt, char *prefix, char *atomicFlag, AttrList *p, UserRequest *ownerReq )
{
	int typeIndex;
	char lowerTypeName[MAXSTRLEN];


	lowerCase( p->entry.AttrTypeName, lowerTypeName );
	typeIndex = findTypeIndex( lowerTypeName );

	if( !strcmp( lowerTypeName, "integer" ) ) typeIndex = 3;
	if( !strcmp( lowerTypeName, "oid" ) ) typeIndex = -1;
	if( !strcmp( lowerTypeName, "fid" ) ) typeIndex = -2;
	if( !strcmp( lowerTypeName, "iid" ) ) typeIndex = -3;

	*atomicFlag = 1;

	switch(  typeIndex )
	{

	case  1 : // string
		anAtt->unique =2000;
		anAtt->attr_type = TEXT;
		break;

	case  2 : // float
		anAtt->domain_min.real_value = 1.00;
		anAtt->domain_max.real_value = 500000.00;
		anAtt->unique =2000;
		anAtt->attr_type = REAL;
		break;

	case  3 : // int
		anAtt->domain_min.int_value = 1;
		anAtt->domain_max.int_value = 500000;
		anAtt->unique =2000;
		anAtt->attr_type = INTEGER;
		break;

	case -1 :
		anAtt->attr_type = MS_OID;
		anAtt->unique =2000;
		break;

	case -2 :
		anAtt->attr_type = MS_FID;
		break;

	case -3 :
		anAtt->attr_type = MS_IID;
		break;

	case  0 : // char
		anAtt->attr_type = INTEGER;
		anAtt->domain_min.int_value = -256;
		anAtt->domain_max.int_value = 255;
		anAtt->unique = 512;
		break;

	case  4 : // boolean
		anAtt->domain_min.int_value = 0;
		anAtt->domain_max.int_value = 1;
		anAtt->unique = 2;
		anAtt->attr_type = INTEGER;
		break;

	/**************	
		ownerReq->setError( PARSE_ERROR, VERSION_ERR, " basic types except String, Float, and Integer");
		return FALSE;
*******************/
	default : // Not a atomic type
		*atomicFlag = 0;
		return TRUE;
	}
	return TRUE;
}



copyToAttribute( 
		RELATION_SCHEMA *relSchema, // Copy to  
		UserRequest *ownerReq, // for error handling
		char *className, // Copy From
		AttrList *inAtt,  // Copy From  
		char *prefix, 
		int  preOffset )
{
	char  		atomicFlag;

	if( !inAtt ) return TRUE;
	if( inAtt->entry.AttributeConstructor == 'T' )
	{
		if(!copyToAttribute( relSchema,ownerReq,className,inAtt->nextsub, prefix, preOffset ))
			return FALSE;
		if(!copyToAttribute( relSchema, ownerReq, className, inAtt->next, prefix, preOffset ))
			return FALSE;
		return TRUE;
	}

	ATTRIBUTE *anAtt;
	AttrList *p, *q;
	p = inAtt;
 
	while(p) 
	{ 
		anAtt = (ATTRIBUTE *) myAlloc( sizeof(ATTRIBUTE));
		anAtt->use = 10;

		switch( p->entry.AttributeConstructor )
		{
		case 'S' : // means set constructor
		case 'L' : // means list constructor
		case 'R' : // means set constructor
			if(!copyConsAtt(anAtt,prefix,p->entry.AttributeConstructor,p,ownerReq, preOffset ))
				return FALSE;
			break;
			
		case 'N' : // means an atomic type or a database typeName

			strcpy( anAtt->name, prefix );
			strcat( anAtt->name, p->entry.AttributeName );
			anAtt->attr_width = p->entry.AttributeLength;
			anAtt->offset = p->entry.AttributeOffset + preOffset;
			strcpy(anAtt->class_name, p->entry.AttrTypeName);

			if(!copyAtomicAtt(anAtt,prefix, &atomicFlag, p, ownerReq))
				return FALSE;

			// if it is not a basic type then it have to be a class name
			if( atomicFlag == 0 ) // not a atomic type
			{
				MoodsTypes	moodsType;
				Catalog		cls;
				AttrList	*att;
				CatBOOLEAN	notFound=0;
				char		currPrefix[MAXSTRLEN], className[MAXSTRLEN];
				strcpy( className,  p->entry.AttrTypeName);
			
				// search type name in catalog
				moodsType.getClassInfo( className, &cls, &att, &notFound );
				if(notFound) {
					ownerReq->setError( PARSE_ERROR, INVLD_CLSNAME_ERR, className );
					return FALSE;
				}

				anAtt->totref = DB_TYPE;
				anAtt->attr_type = DB_TYPE;
				strcpy( currPrefix, prefix );
				strcat( currPrefix, p->entry.AttributeName );
				strcat( currPrefix, "$" );
				if(!copyToAttribute(relSchema,ownerReq,className, att, currPrefix, anAtt->offset ))
				{
					delete att;
					return FALSE;
				}
				delete att;

			}
			break;

		default :
		printf("Never Comes here\n");
		}

		/***********************************
		// later put a right code
		IndexClass	ci(ownerReq);
		char status=1;
		anAtt->btree_avail = 0;
		// TO THE ATTENTION OF YUKSEL 
		//anAtt->index_id = ci.getIID( className, anAtt->name, &status );
		if( !status )
		anAtt->btree_avail = 1;
		*************************************/

		relSchema->attr[relSchema->arity] = anAtt;
		relSchema->arity++;
		anAtt->btree_avail = 0;
		p = p->next;
	}
	return TRUE;

}

copyToRelSchema( RELATION_SCHEMA 	*relSchema, // Copy to  
		 UserRequest 		*ownerReq, // for error handling
                 Catalog 		*catClass, // Copy From
                 AttrList 		*inAtt ) // Copy From  
{
	AttrList	*p = inAtt, *q;
	ATTRIBUTE	*anAtt;
 
	strcpy( relSchema->name, catClass->TypeName );
	//relSchema->object_type = SET_OF_RECORDS; 
	relSchema->fid = catClass->Instance_File;
	//displayFID( &relSchema->fid );
	relSchema->record_width = catClass->Size;

	// put proper values later
	relSchema->cardinality = 20000;
	relSchema->arity = 0;
	relSchema->TypeId = catClass->TypeId;

	// copy attributes now
	if( !copyToAttribute( relSchema, ownerReq, relSchema->name, inAtt,"", 0 ))
		return FALSE;

	return TRUE;
}


/*
BindListItem::evaluateBindExp( UserRequest *userReq, LinkedList )
{
}
*/

BindListItem::checkSemantic( UserRequest *userReq )
{
#ifdef DEBUG
	printf(" BindListItem::checkSemantic \n");
#endif
        char            className[ 80 ];
        Catalog         cls;
        AttrList        *att;
        CatBOOLEAN      notFound;
	MoodsTypes	moodsType;
	PathExpression *pathExp;
 
        if( bindExpression->size > 1 )
        {
                userReq->setError( PARSE_ERROR,
                              VERSION_ERR, " expressions in FROM clause");
                return FALSE;
        }

	//printOn();
 
        // extract className from bind expression
	pathExp =  ( PathExpression *) bindExpression->firstItem->item;
#ifdef DEBUG
	pathExp->printOn();
#endif
        strcpy(className,((PathItem *)pathExp->pathItems->firstItem->item)->itemName) ;

#ifdef DEBUG
	printf(" Input Class Name : %s\n ", className );
#endif
 
        // if class name not found then set a error message
        moodsType.getClassInfo( className, &cls, &att, &notFound );
	//printAllAtt( att );
	//getchar();
 
        /* Fill Class Info */
        if(notFound) {
                userReq->setError( PARSE_ERROR, INVLD_CLSNAME_ERR, className );
                return FALSE;
        }
 
        if( !copyToRelSchema(&classInfo, userReq, &cls, att))
                return FALSE;
#ifdef DEBUG
	classInfo.printOn();
	getchar();
#endif

	delete att;
 
        return TRUE; 
 
}

BindExpression::checkSemantic()
{
#ifdef DEBUG
	printf(" BindExpression::checkSemantic \n");
#endif
        ListItem *p = firstItem;

        if( !p ) return 0;

        while(p)
	{
                if( ! (((BindListItem *) p->item)->checkSemantic( ur )))
                        return 0;
		p = p->next;
	}

        return 1;
}

RELATION_SCHEMA* BindExpression::isDefineClass( char *className )
{
        ListItem *p;
 
 
	for( p = firstItem; p; p=p->next )
                if(!strcmp( className, (( BindListItem *)p->item)->classInfo.name ))
                        return &( (BindListItem * ) p->item)->classInfo;

        return NULL;
}


PathExpression::checkSemantic( BindExpression *bexp )
{
	BindListItem	*container;
	ListItem *p = pathItems->firstItem;
#ifdef DEBUG
	printf(" PathExpression::checkSemantic \n");
#endif
	if( !( container = bexp->contains( ( (PathItem *) p->item )->itemName )))
	{
		ur->setError( PARSE_ERROR, INVLD_BIND_VAR,
                                   ( ( PathItem *) p->item)->itemName);
		return FALSE;
	}	
	// copy class info of bind variable
	( (PathItem *) p->item)->attClass.copy( &container->classInfo );

	( (PathItem *) p->item)->prevSchema = &container->classInfo;

	strcpy(((PathItem *) p->item)->itemSign, ((PathItem *)p->item)->itemName);
	strcpy(((PathItem *) p->item )->retTypeName, "REF" );
	((PathItem *) p->item )->dbType = 0;
	((PathItem *) p->item )->isSetList = 0;

	p = p->next; // Now, get next item
	while(p)
	{
		if( !((PathItem *) p->item )->fillPathItem( bexp, p->prev, ur ))
			return FALSE;
		p = p->next;
	}

	// Now get last item type in path expression
	strcpy( finalType, ((PathItem *) pathItems->lastItem->item)->retTypeName );

	return TRUE;
}

int Expression::checkSemantic(BindExpression *bexp, ListItem *inItem, char *finalType )
{
	char	leftType[30], rightType[30];

	if( !inItem ) return NULL;

	switch( inItem->itemType )
	{
		case NOT :
			if( !checkSemantic( bexp, inItem->left, leftType ))
				return FALSE; 
			if( strcmp( leftType, "boolean" ) ){
				ur->setError( RUNTIME_ERROR, EXP_TYPE_MISMATCH, "for NOT operator"); 
				return FALSE;
			}
			strcpy( finalType, "boolean");
			return TRUE;

		case AND : case OR  :
			if( !checkSemantic( bexp, inItem->left, leftType ))
				return FALSE;
			if( !checkSemantic( bexp, inItem->right, rightType ))
				return FALSE;
			if( strcmp( leftType,  "boolean" ) ||
			    strcmp( rightType, "boolean" ) ) {
				ur->setError( RUNTIME_ERROR, EXP_TYPE_MISMATCH, "for AND/OR operator"); 
				return FALSE;
			}

			strcpy( finalType, "boolean");
			return TRUE;

		case EQ : case NE : case GE : case LT : case LE :
                case GT : case ISOP : case ISNOT :
			if( !checkSemantic( bexp, inItem->left, leftType ))
				return FALSE;
			if( !checkSemantic( bexp, inItem->right, rightType ))
				return FALSE;
			strcpy( finalType, findComparisonRetType( leftType, rightType ));

			if( !strcmp( finalType, "invalid$type" ) ){
				ur->setError( RUNTIME_ERROR, EXP_TYPE_MISMATCH, "for comparison operator"); 
				return FALSE;
			}

			return TRUE;

		case ADD : case MUL : case DIV : case SUB : case MOD :
			if( !checkSemantic( bexp, inItem->left, leftType ))
				return FALSE;
			if( !checkSemantic( bexp, inItem->right, rightType ))
				return FALSE;
			strcpy( finalType, findArithmeticRetType( leftType, rightType ));
			if( !strcmp( finalType, "invalid$type" ) ) {
				ur->setError( RUNTIME_ERROR, EXP_TYPE_MISMATCH, "for arithmetic operator"); 
 				return FALSE;
			}
			return TRUE;

		case UNARYPLUS : case UNARYMINUS :
			if( !checkSemantic( bexp, inItem->left, finalType )){
				ur->setError( RUNTIME_ERROR, EXP_TYPE_MISMATCH, "for unary -/+ operator"); 
				return FALSE;
			}
			return TRUE;

		case PATH_EXP :
			if(!((PathExpression *)inItem->item)->checkSemantic(bexp))
				return FALSE;
			strcpy( finalType, ((PathExpression *) inItem->item)->finalType );	
			return TRUE;

		case CHRCONS : 
			strcpy( finalType, "char"); return TRUE;

		case STRCONS :
			strcpy( finalType, "string"); return TRUE;

		case UNSIGNEDREAL:
			strcpy( finalType, "float"); return TRUE;

		case UNSIGNEDINT :
			strcpy( finalType, "int"); return TRUE;

		case MS_FALSE : 
		case MS_TRUE :
			strcpy( finalType, "boolean"); return TRUE;

		default :
			strcpy( finalType, "invalid$type"); return TRUE;
	}
	return TRUE;
}

isAtomicType( char *typeName )
{
	if( !strcmp( typeName, "string") ||
	    !strcmp( typeName, "int") ||
	    !strcmp( typeName, "float") ||
	    !strcmp( typeName, "boolean") ||
            !strcmp( typeName, "integer" ) ||
	    !strcmp( typeName, "char" ) ) {
               if( !strcmp( typeName, "integer" ) )
                        strcpy( typeName, "int" );

                        return TRUE;
        }

	return FALSE;
}

isVoidType( char *typeName )
{
	if( !strcmp( typeName, "void" ) ) return TRUE;
	return FALSE;
}

determineArgs( LinkedList *parameters, char**arguments, int *argCount, BindExpression *bexp )
{
	ListItem	*p;

	if( parameters ) 
	{
		for( p = parameters->firstItem; p; p = p->next )
		{
			arguments[ *argCount ] = (char * ) myAlloc( 30 );
			if( !((Expression *) p->item)->checkSemantic( 
				bexp, 
				((Expression *)p->item)->lastItem,
				arguments[*argCount] )) 
				return FALSE;
/*****
			if( !strcmp( arguments[*argCount], "invalid$type") )
			{
				
			}
*******/
				(*argCount)++;
		}
	}
	return TRUE;
}

void prepareErrorMsg( char *errMsg, char *cName, char *fName, char **arguments, int argCount )
{
	int i;

	sprintf( errMsg, "%s::%s(", cName, fName );
	for( i = 0; i < argCount-1; i++ ){
		strcat( errMsg, arguments[i] );
		strcat( errMsg, "," );
		myFree( (char *) arguments[i] );
	}
	if( argCount > 0 ) strcat( errMsg, arguments[argCount-1] );
	myFree( (char *) arguments[argCount-1] );
	strcat( errMsg, ")" );
}

void combineWithPrev( ListItem *previous )
{
}

void fillFuncAtt( ATTRIBUTE *funcAtt, FunctionType funcType, int appOffset )
{
	funcAtt->attr_type = METHOD;
	strcpy( funcAtt->name, funcType.signature );
	// add a dummy attribute to the class with offset = 0;
	funcAtt->offset = appOffset;
	funcAtt->attr_width = 0;
	funcAtt->use = 10;
	// This is actual application offset of the function
	funcAtt->notnull = appOffset;		
}

isConstructor( char *typeName )
{
	if( !strcmp( typeName, "SET" ) ||
	    !strcmp( typeName, "LIST" ) ||
	    !strcmp( typeName, "REF" ) )
	    return TRUE;

	return FALSE;
}


PathItem::fillPathItem( BindExpression *bexp, ListItem *previous, UserRequest *userReq )
{
	//RELATION_SCHEMA *prevSchema;
	ATTRIBUTE	*prevAtt;
	char		*prevClsName;
	Catalog		cls;
	AttrList	*att;
	CatBOOLEAN	success;
	MoodsTypes	moodsType;
	FunctionType	funcType;
	ListItem	*p;
	int		i;
	char isDBType;

	if( ( (PathItem *) previous->item )->dbType == 1 &&
	    ( (PathItem *) previous->item )->itemType == PATH_FUNCTION ){
		userReq->setError( RUNTIME_ERROR, R_VERSION_ERROR, 
			"function returning a class/type inside a path expression" );
		return FALSE;
	}

	// item item is function
	if( itemType == PATH_FUNCTION )
	{
		int		argCount = 0, appOffset;
		char		**arguments, fSign[2*MAXSTRLEN], lowerPrevName[MAXSTRLEN];
		ATTRIBUTE	*funcAtt;
		RELATION_SCHEMA *rs;

		arguments = ( char ** ) myAlloc( 10 * sizeof( char * ) );

		// if the previous path item is DB type
		prevSchema   = &(( (PathItem *) previous->item )->attClass);
		isDBType = (( (PathItem *) previous->item )->dbType);

		prevClsName = prevSchema->name;

		//rs = bexp->isDefineClass( prevClsName );

		if( !prevClsName )
		{
			userReq->setError( PARSE_ERROR, INVLD_ATT_NAME , itemName);
			return FALSE;
		}

		determineArgs( parameters, arguments, &argCount, bexp );

		// Check the existence of the function
		if( !moodsType.searchSign(prevClsName, itemName, (char **) arguments,
			argCount, &success, &funcType, &appOffset, 1 ))
			return FALSE;

		if( !success ) { // if function not found
			char	errMsg[120];
			prepareErrorMsg( errMsg, prevClsName, itemName, arguments, argCount );
			userReq->setError( RUNTIME_ERROR, FUNC_NOT_EXIST, errMsg );
			return FALSE;
		}

		for( i = 0; i < argCount; i++ ) myFree( (char *) arguments[i]);

		funcAtt = (ATTRIBUTE *) myAlloc( sizeof(ATTRIBUTE) );
		fillFuncAtt( funcAtt, funcType, appOffset );
		returnType = funcType.retType;
		strcpy( funcAtt->class_name,  funcType.retTypeName );
		strcpy( retTypeName, funcType.retTypeName );

		dbType = FALSE;
		isSetList = FALSE;

		lowerCase( retTypeName, lowerPrevName );

		if( isAtomicType(lowerPrevName)) strcpy(retTypeName,lowerPrevName );
		else if(isVoidType(lowerPrevName)) strcpy(retTypeName,lowerPrevName);
		else {
			// check return type as SET_?, LIST_?, REF_?
			char *underMarker, *tailName;
			char typeConstructor[120];

			strcpy( typeConstructor, retTypeName );
			underMarker = strchr( typeConstructor, '_' );

			if( underMarker ) {

				tailName = underMarker + 1;
				*underMarker = '\0';

				if( isConstructor( typeConstructor ) ){
					strcpy( retTypeName, typeConstructor );
					if( strcmp( typeConstructor, "REF" ) )
						isSetList = TRUE;
				}
				else {
					dbType = TRUE;
					tailName = typeConstructor;
				}

			}
			else {
				dbType = TRUE;
				tailName = typeConstructor;
			}

			strcpy( funcAtt->class_name, tailName );

			// now process tailName
			// if class name not found then set a error message
			if( !moodsType.getClassInfo(tailName, &cls, &att, &success))
				return FALSE;

			/* Fill Class Info */
			if(success) {
				userReq->setError( PARSE_ERROR, INVLD_CLSNAME_ERR, tailName );
				return FALSE;
			}

			if( !copyToRelSchema(&attClass, userReq, &cls, att))
				return FALSE;

			delete att;

		}

		addFunctionToAll( funcAtt, previous, prevSchema );
		//addFunctionAtt( prevSchema, funcAtt );
		// if( rs ) addFunctionAtt( rs, funcAtt );

		if( isDBType ) sprintf( itemSign, "%s%s", "$",  funcType.signature );
		else  strcpy( itemSign, funcType.signature );

	}
	else // that mean it is an atomic or {REF|SET|LIST} constructed attribute
	{
		char lowerPrevName[MAXSTRLEN];

		// Check the existence of the attribute from previous one
		prevSchema = &(( (PathItem *) previous->item )->attClass);
		isDBType = (( (PathItem *) previous->item )->dbType);

		prevAtt = prevSchema->contains(itemName);

		if( !prevAtt )
		{
			userReq->setError( PARSE_ERROR, INVLD_ATT_NAME , itemName);
			return FALSE;
		}

		prevClsName = prevAtt->class_name;

		if( isDBType ) sprintf( itemSign, "%s%s", "$", itemName );
		else strcpy( itemSign, itemName );

		lowerCase( prevClsName, lowerPrevName );

		dbType = FALSE;
		isSetList = FALSE;

		if( isAtomicType( lowerPrevName ) ) 
			strcpy( retTypeName, lowerPrevName );
		else if( isVoidType( lowerPrevName))
			strcpy( retTypeName, lowerPrevName );
		else {

			// make return type "SET" / "REF" / "LIST"
			if( prevAtt->totref == DB_TYPE ){
				strcpy( retTypeName, prevClsName );
				dbType = TRUE;
			}
			else strcpy( retTypeName, "REF" );

			if( prevAtt->attr_type == MS_SET ||
			    prevAtt->attr_type == LIST )
				isSetList = TRUE;

			// if class name not found then set a error message
			if( !moodsType.getClassInfo(prevClsName, &cls, &att, &success))
				return FALSE;

			/* Fill Class Info */
			if(success) {
				userReq->setError( PARSE_ERROR, INVLD_CLSNAME_ERR, prevClsName );
				return FALSE;
			}

			if( !copyToRelSchema(&attClass, userReq, &cls, att))
			  return FALSE;

			delete att;
		}
	}

	return TRUE;
}

Query::checkSemantic()
{
	char finalType[30];
#ifdef DEBUG
	printf("Query::checkSemantic\n");
#endif
	if( bindList ) if( !bindList->checkSemantic()) return FALSE;
	if( whereP ){ 
		if( !whereP->checkSemantic( bindList, whereP->lastItem, finalType)) 
			return FALSE;
		if( strcmp( finalType, "boolean") )
		{
			ur->setError( RUNTIME_ERROR, R_TYPE_MISMATCH,
		 	"\ncondition is not boolean in where clause" ); 
			return FALSE;
		}
	}

	if( projectionList ) if( !projectionList->checkSemantic( bindList)) return FALSE;
	return TRUE;
}

/***************
Expression::checkSemantic( BindExpression *bexp, char *finalType )
{

	ListItem	*p = firstItem;
#ifdef DEBUG
	printf("Expression::checkSemantic\n");
#endif
	while(p)
	{
		if( p->itemType == PATH_EXP )
		  if( !( (PathExpression *) p->item)->checkSemantic( bexp ) )
			return FALSE;
		p = p->next;
	}
	return TRUE;
}
***************/

ProjectionExpression::checkSemantic( BindExpression *bexp )
{
	ListItem	*p = firstItem;
	Expression	*exp;
	char				finalType[80];

#ifdef DEBUG
	printf("ProjectionExpression::checkSemantic\n");
#endif
	while(p)
	{
		exp = ( (ProjectionListItem *) p->item )->projExpression;
		if( !exp->checkSemantic( bexp, exp->lastItem, finalType ) ) 
			return FALSE;
		p = p->next;
	}
	return TRUE;
}
