#include "keywords.h"
#include "mood_parser.h"
#include <Cursor.h>
#include <mood_catalog.h>
#include <memory.h>
#include <dld.h>

extern evaluateExpr(JOIN_ARG *, CNST *);
extern int IncRefCount(OID);

UpdateClause::parseSetClause(LinkedList *SetExp)
{
        Expression *exp;
 
        CHECK_TOKEN(LCB, LCB_EXPECTED, "");     // error "missing {"
 
        getToken();
 
        switch(tokenType()) {
                case MS_SELECT: { // Parse the "select" statement...
                        Query *query;
 
                        backtrack();
                        query = new Query(ur);
                        SetExp->insert(QRY_PTR, VOID query);
                        if (!query->parse()) {
                        //      delete query;
                                return FALSE;
                        }
                        break;
                }
                case NEW: { // Parse the sequence of "new" statements...
                        NewClause *nestedNew;
 
                        backtrack();
 
                        do {
                                nestedNew = new NewClause(ur);
                                SetExp->insert(NEW_PTR, VOID nestedNew);
                                if (!nestedNew->parse()) {
                                        //delete nestedNew;
                                        return FALSE;
                                } // if
 
                                getToken();
 
                        } while (tokenType() == COMMA);
 
                        backtrack();
 
                        break;
                }
                default: {
                        // error "SELECT or NEW keyword expected"
                        ur->setError( PARSE_ERROR, SELNEW_KWD_EXPECTED, "" );
                        return FALSE;
                } // default
        } // switch
 
        CHECK_TOKEN(RCB, RCB_EXPECTED, "");     // error "missing }"
 
        return TRUE;
 
} // parseSetClause


UpdateClause::parseListClause(LinkedList *ListExp)
{
	Expression *exp;
	NewClause *nestedNew;
 
	CHECK_TOKEN(LB, LB_EXPECTED, "");       // error "missing ["
 
	getToken();
 
	switch(tokenType()) {
		case MS_SELECT: { // Parse the "select" statement...
			Query *query;
 
			backtrack();
			query = new Query(ur);
			ListExp->insert(QRY_PTR, VOID query);
			if (!query->parse()) {
			//      delete query;
				return FALSE;
			}
			break;
		}
		case NEW: { // Parse the sequence of "new" statements...
			NewClause *nestedNew;
 
			backtrack();
 
			do {
				nestedNew = new NewClause(ur);
				ListExp->insert(NEW_PTR, VOID nestedNew);
				if (!nestedNew->parse()) {
					//delete nestedNew;
					return FALSE;
				} // if
 
				getToken();
 
			} while (tokenType() == COMMA);
 
			backtrack();
 
			break;
		}
		default: {
			// error "SELECT or NEW keyword expected"
			ur->setError( PARSE_ERROR, SELNEW_KWD_EXPECTED, "" );
			return FALSE;
		} // default
	} // switch
 

	CHECK_TOKEN(RB, MISSING_RB, "");        // error "missing ]"
 
	return TRUE;
 
} // parseListClause

void UpdateClause::printOn()
{
	printf("\n ------- UPDATE STATEMENT ------ \n");
	if( whereP ) {
		printf("Where Predicate : \n");
		whereP->printOn();
	}
	if( bindList ) {
		printf("\n Bind List : \n");
		bindList->printOn();
	}
	if( updateExpression ) {
		printf("\n Set Clause : \n");
		updateExpression->printOn();
	}
	printf("\n ------------------------------- \n");
}
 
UpdateClause::parse()
{
	return updateClause();
}


UpdateClause::updateClause()
{

	connectUserRequest();

	CHECK_TOKEN(UPDATE, UPDATE_KYWD_ERR, ""); // Check UPDATE keyword

	bindList = new BindExpression( ur );

	if( !bindList->parse() ) return FALSE;

	if(! setClause() ) 	 return FALSE;

	if(! whereClause() )     return FALSE;

	return TRUE;
}


UpdateClause::setClause()
{
	Expression	*exp;
	PathExpression	*pexp;

	CHECK_TOKEN(SET, SET_KYWD_ERR, ""); // there must be SET keyword

	do {

		// this is left hand side
		pexp = new PathExpression( ur );
		pexp->parse();
		updateExpression->insert( PATH_EXP, pexp );

		getToken();

		switch( tokenType() )
		{
			case COMMA : 
				// it must be a member function ( for update )
				backtrack(); 
				break;

			case EQ :
				// check token if it is SELECT
				getToken();

				switch( tokenType() )
				{

					case LPAR:
					{
						Query *query;
						CHECK_TOKEN( MS_SELECT, SELECT_KYWD_ERR, "" );
						backtrack();
						query = new Query( ur );
						if( !query->parse() ) return FALSE;
						updateExpression->insert( QRY_PTR, query );
						CHECK_TOKEN(RPAR,MISSING_RP,"");
						break;
					}

					case LB : // set of new clauses
					{
						LinkedList *ListExp;
						backtrack();
						ListExp = new LinkedList();
						//newExpList->insert(LST_PTR, VOID ListExp);
						updateExpression->insert(LST_PTR, VOID ListExp );	
						if (!parseListClause(ListExp)) return FALSE;
						break;
					}

					case LCB :
					{
						LinkedList *SetExp;
						backtrack();
						SetExp = new LinkedList();
						//newExpList->insert(SET_PTR, VOID SetExp);
						updateExpression->insert(SET_PTR, VOID SetExp );
						if (!parseSetClause(SetExp)) return FALSE;
						break;
					}

					case NEW : // a new statement for REF
					{
						NewClause *newStmt;
						backtrack();
						newStmt = new NewClause(ur);
						if( !newStmt->parse() ) return FALSE;
						updateExpression->insert(NEW_PTR, VOID newStmt );
						break;
					}

					default :
						backtrack();
						exp = new Expression( ur );
						if( ! exp->parse() ) // parse an expression
						{
							delete exp;
							return FALSE;
						}
						updateExpression->insert( EXP_PTR, VOID exp );
				}
		}


		// if everything is OK then insert item into the list
		// updateExpression->insert( UPDATE_EXP, VOID exp );

		getToken();

	} while( tokenType() == COMMA );

	backtrack();

	return TRUE;
}

UpdateClause::whereClause()
{
	IS_THIS_KEYWORD(WHERE); // check WHERE keyword

	whereP = new Expression( ur );

	if( !whereP->parse()) return FALSE;

	return TRUE;
}

UpdateClause::checkSemantic()
{
	// if it is not function, top operator have to be "="
}

UpdateClause::convertToSelectClause()
{
}

/************************************************************************************

     simpleUpdate.c

 ***********************************************************************************/

AttrList *UpdateClause::findAttrList( AttrList *attributes, char *attrName )
{
	AttrList 	*p;

	if( attributes == NULL ) return NULL;

	if( strcmp( attributes->entry.AttributeName, attrName ) == 0 ) return attributes;

	if( ( p = findAttrList( attributes->next, attrName ) ) ) return p;
	if( ( p = findAttrList( attributes->nextsub, attrName ) ) ) return p;	

	return NULL;
}


void UpdateClause::freeAttrList( AttrList *attrList )
{
	if( !attrList ) return;

	freeAttrList( attrList->next );
	freeAttrList( attrList->nextsub );

	free( attrList );
}

int UpdateClause::findLinkedList( PathExpression *pe, LinkedList *pathChain )
{
	MoodsTypes	mt;
	Catalog		cat;
	CatBOOLEAN	status = FALSE;
	AttrList	*attrList, *currentAttr, *attrType;
	int		i, size = pe->pathItems->size;
	char		*currentAttrName;

	if( !mt.getClassInfo( className, &cat, &attrList, &status ) || status) 
		return FALSE;

	currentAttr = attrList;

	for( i = 1; i < size; i++ ){
		currentAttrName = (char *)(pe->pathItems->listAt(i)->item);
		// find the attribute inside the currentAttr
		currentAttr = findAttrList(attrList, currentAttrName );
		if( !currentAttr ){
			//printf("Attribute not found\n");
			ur->setError( RUNTIME_ERROR, ATTR_NOT_FOUND, "" );
			freeAttrList( attrList );
			return FALSE;
		}
		
		switch( currentAttr->entry.AttributeConstructor ){
			case 'S' : attrType = new AttrList;
			   memcpy( attrType, &currentAttr->entry, sizeof( AttributeType) );
			   pathChain->insert(ATR_PTR, VOID attrType );
			   currentAttr = currentAttr->nextsub;
			   attrType = new AttrList;
			   memcpy( attrType,  &currentAttr->entry, sizeof( AttributeType) );
			   pathChain->insert(ATR_PTR, VOID attrType );
			   break;
			case 'L' : attrType = new AttrList;
			   memcpy( attrType, &currentAttr->entry, sizeof( AttributeType) );
			   pathChain->insert(ATR_PTR, VOID attrType );
			   currentAttr = currentAttr->nextsub;
			   attrType = new AttrList;
			   memcpy( attrType,  &currentAttr->entry, sizeof( AttributeType) );
		  	   pathChain->insert(ATR_PTR, VOID attrType );
			   break;
			case 'R' : attrType = new AttrList;
		   	   memcpy( attrType, &currentAttr->entry, sizeof( AttributeType) );
			   pathChain->insert(ATR_PTR, VOID attrType );
			   currentAttr = currentAttr->nextsub;
			   attrType = new AttrList;
			   memcpy( attrType,  &currentAttr->entry, sizeof( AttributeType) );
			   pathChain->insert(ATR_PTR, VOID attrType );
			   break;
//			case 'T' : 
			case 'N' : attrType = new AttrList;
			   memcpy( attrType, &currentAttr->entry, sizeof( AttributeType ) );
			   pathChain->insert(ATR_PTR, VOID attrType );
			   break;

		}
	}

	freeAttrList( attrList );
	return TRUE;		
}

int UpdateClause::findLinkedLists()
{
	unsigned 	itemType;
	int		i, size=updateExpression->size;
	LinkedList	*list;
	PathExpression 	*pathExp;

	for( i = 0; i < size; i++ ){
		itemType = updateExpression->listAt(i)->itemType;
		switch( itemType ){
		case PATH_EXP :
			pathExp= (PathExpression *)(updateExpression->listAt(i)->item);
			list = new LinkedList;
			if( !findLinkedList( pathExp, list ) ) return FALSE;
			pathList.insert( LST_PTR, list );	
			break;
		default       :
			pathList.insert( NULLVALUE, (OID *)0 );
		}
	}
	return TRUE;	
}


/*
int UpdateClause::traverseAndUpdate( OID *oid, LinkedList *pathChain, int position, int expPosition )
{
	OID		nextOid, element;
	AttributeType 	*p = (AttributeType *)(pathChain->listAt(position)->item);
	int		err, s_type, s_size, l_type, l_size;
	USERDESC	*UserDesc;
	CatBOOLEAN	status = FALSE;
		
	if( ( ( p->AttributeConstructor != 'N' ) && ( position == (pathChain->size-2) ) ||
	      ( p->AttributeConstructor == 'N' ) && ( position == (pathChain->size-1) ) ){
		assign( oid, pathChain->listAt(position), expPosition );
		return TRUE;
	}

	// read the object corresponding to the oid
	err = sm_ReadObject( BufferGroup, oid, 0, READ_ALL, &UserDesc );
	if( err != esmNOERROR ){
		ur->setError( EXODUS_ERROR, 1, "" );
		return FALSE;
	}
	
	// copy  the corresponding oid
	memcpy( &nextOid, (char *)UserDesc->basePtr+p->AttributeOffset), sizeof(OID));
	sm_ReleaseObject( UserDesc );

	if( !OID_IS_INVALID( nextOid ) ){
		switch(p->AttributeConstructor){
			case 'S' : 
				Set s( &nextOid, &s_type, &s_size, &status );
				if( status ){ 
					ur->setError( INTERNAL_ERROR, 0, _errorMessage );
					return FALSE;
				}
				for( i = 0; i < s_size; i++ ){
					s.getElement( &element, i, &status );
					if( status ) return FALSE;
					if ( !traverseAndUpdate( &element, pathChain, 
								 position+2, expPosition )
						return FALSE;  
				}
				break;
			case 'L' :
                                List l( &nextOid, &l_type, &l_size, &status );
                                if( status ){
                                        ur->setError( INTERNAL_ERROR, 0, _errorMessage );
                                        return FALSE;
                                }
                                for( i = 0; i < l_size; i++ ){
                                        l.getElement( &element, i, &status );
                                        if( status ) return FALSE;
                                        if ( !traverseAndUpdate( &element, pathChain, 
                                                                 position+2, expPosition )
                                                return FALSE;
                                }
                                break;
			case 'R' :
				if ( !traverseAndUpdate( &element, pathChain, 
                                                                 position+2, expPosition )
                                                return FALSE;
                                break;
		}
	}
 
	return TRUE;	
}


int UpdateClause::traverse( PathExpression *pe, char *Object )
{
	int i, size = pe->size - 1;

	for( i = 1; i < size; i++ ){

	} 
}

*/

int UpdateClause::selectForRef( Query *query, OID *oid )
{
        MOODSCursor     ResCur;
        char            Flag;
        int             err;
	USERDESC	*userDesc;

        if( !query->generateAllTree() ) return FALSE;
        if( !query->generatePlan() ) return FALSE;

        OpenCursor(query->solutionFile, &ResCur);

        if (!GetFirstObject(&ResCur,&Flag)) {
                 ur->setError( RUNTIME_ERROR, CANNOT_GETOBJ, "" );
                 return FALSE;
        }
        if( !Flag ){
		memcpy((char *)oid,(char *)ResCur.CurUserdesc->basePtr,sizeof(OID));
                if (!GetNextObject(&ResCur,&Flag)) {
                         ur->setError( RUNTIME_ERROR, CANNOT_GETOBJ, "" );
			 CloseCursor( &ResCur );
                         return FALSE;
                 }

                 if (!Flag) {
                         ur->setError( RUNTIME_ERROR, TOO_MANY_OBJS, "" );
			 CloseCursor( &ResCur );
                         return FALSE;
                 }
        } else INVALIDATE_OID( *oid );

	CloseCursor( &ResCur );

	return TRUE;

}


int UpdateClause::fillOpList( )
{
	int		size = updateExpression->size, i, elementCount, j;
	unsigned	itemType, elementType;
	Query		*query;
	NewClause	*nc;
	OID		*oid, *setOid, *listOid, tempOid;
	Set		*set;
	List		*list;
	MoodsTypes	mt;
	Catalog		cat;
	CatBOOLEAN	status = FALSE;
	LinkedList	*setExp, *listExp;
	AttrList	*attrList;

	for( i = 0; i < size; i++ ){
		itemType=updateExpression->listAt(i)->itemType;
		switch(itemType){
		case QRY_PTR : 
			query = (Query *)(updateExpression->listAt(i)->item);	
			oid = ( OID * )malloc( sizeof( OID ) );
			if( !selectForRef( query, oid ) ) return FALSE;
			operators.insert( OID_PTR, VOID oid );	
			break;

		case NEW_PTR :
			nc = ( NewClause *)(updateExpression->listAt(i)->item);
			oid = (OID *) malloc( sizeof( OID ) );
			if( !nc->execute( oid ) ) return FALSE;
			operators.insert( OID_PTR, VOID oid );
			break;

		case SET_PTR :
			if( !mt.getClassInfo( className, &cat, &attrList, &status ) || status )
				return FALSE;
			freeAttrList( attrList );

			set = new Set( &cat.Instance_File, NEAR_FIRST_PHYSICAL, NULL, &status );
			if( status ) return FALSE;

			setOid = (OID *)malloc(sizeof(OID));
			*setOid = set->getSetOid(&status);
			if( status ) return FALSE;

			setExp = (LinkedList *)(updateExpression->listAt(i)->item);
			elementCount = setExp->size;
			
			for( j = 0; j < elementCount; j++ ){
				elementType=setExp->listAt(j)->itemType;
				switch( elementType ){
				case QRY_PTR:
					query = (Query *)(setExp->listAt(j)->item);
					if( !selectSet( set, query ) ) return FALSE;
					break;
				case NEW_PTR:
					nc = ( NewClause *)(setExp->listAt(j)->item);
					if( !nc->execute( &tempOid ) ) return FALSE;
					set->insertToSet( &tempOid, &status );
					if( status ) return FALSE;
					break;
				}	
			}

			operators.insert( OID_PTR, VOID setOid );
			break;

		case LST_PTR :
                        if(!mt.getClassInfo(className,&cat,&attrList,&status)||status)
                                return FALSE;
			freeAttrList( attrList );

                        list = new List( &cat.Instance_File, NEAR_FIRST_PHYSICAL, NULL, 20, &status);
                        if( status ) return FALSE;

			listOid = (OID *)malloc(sizeof(OID));
                        *listOid = list->getListOid(&status);
                        if( status ) return FALSE;

                        listExp = (LinkedList *)(updateExpression->listAt(i)->item);
                        elementCount = listExp->size;

                        for( j = 0; j < elementCount; j++ ){
                                elementType=listExp->listAt(j)->itemType;
                                switch( elementType ){
                                case QRY_PTR:
                                        query = (Query *)(listExp->listAt(j)->item);
                                        if( !selectList( list, query ) ) return FALSE;
                                        break;
                                case NEW_PTR:
                                        nc = ( NewClause *)(listExp->listAt(j)->item);
                                        if( !nc->execute( &tempOid ) ) return FALSE;
                                        list->insertToList( &tempOid, &status );
                                        if( status ) return FALSE;
                                        break;
                                }
                        }

                        operators.insert( OID_PTR, VOID listOid );

			break;
		default : 
			operators.insert( NULLVALUE, VOID oid );
		}	
	}
	return TRUE;
} 


int UpdateClause::updateRef( USERDESC *ud, int offset, int pos, char *typeName )
{
	OID		*newOid, *oldOid;
	int		err;
	DeleteClause	del(ur);

	oldOid = (OID *)((char *)ud->basePtr+offset);
	newOid = (OID *)(operators.listAt(pos)->item);

	if( !OID_IS_INVALID( *oldOid ) ){
		if( !del.DeleteObject( oldOid, typeName, FALSE ) ){
			ur->setError( RUNTIME_ERROR, CANNOT_DELETE_OBJ, "" );
			return FALSE;
		}
	}

	if (!IncRefCount(*newOid)) {
		ur->userError = catalogError;
		ur->operationType = HANDLE_ERROR;
		ur->operationType = HANDLE_ERROR;
		return FALSE;
	}

	// copy the new oid in place of the old one
	err = sm_WriteObject( BufferGroup, offset, sizeof(OID), (char *)newOid,ud,TRUE );
	if( err != esmNOERROR ){
		ur->setError( EXODUS_ERROR, err, "" );
		return FALSE;
	}

	return TRUE;
}


int UpdateClause::selectSet( Set *set, Query *query )
{
	MOODSCursor     ResCur;
	char            Flag;
	OID             newOid;
	int             err;
	CatBOOLEAN      status = FALSE;

	if( !query->generateAllTree() ) return FALSE;

	if( !query->generatePlan() ) return FALSE;

	OpenCursor(query->solutionFile, &ResCur);

	if (!GetFirstObject(&ResCur,&Flag)) {
		ur->setError( RUNTIME_ERROR, CANNOT_GETOBJ, "" );
		return FALSE;
	}

	while( !Flag ){
		memcpy( (char *)&newOid,
			(char *)ResCur.CurUserdesc->basePtr,
			sizeof(OID));

		// insert it to the set.
		set->insertToSet(&newOid,&status);
		if (status != 0){
			CloseCursor( &ResCur );
			return FALSE;
		}

		if (!IncRefCount(newOid)) {
			ur->userError = catalogError;
			ur->operationType = HANDLE_ERROR;
			CloseCursor( &ResCur );
                  	return FALSE;
		}

		if (!GetNextObject(&ResCur,&Flag)){
			CloseCursor( &ResCur );
			return FALSE;
		}
	} // while

	CloseCursor(&ResCur);

	return TRUE;
}


int UpdateClause::selectList( List *list, Query *query )
{
	MOODSCursor     ResCur;
	char            Flag;
	OID             newOid;
	int             err;
	CatBOOLEAN      status = FALSE;

	if( !query->generateAllTree() ) return FALSE;

	if( !query->generatePlan() ) return FALSE;

	OpenCursor(query->solutionFile, &ResCur);

	if (!GetFirstObject(&ResCur,&Flag)) {
		ur->setError( RUNTIME_ERROR, CANNOT_GETOBJ, "" );
		return FALSE;
	}

	while( !Flag ){
		memcpy( (char *)&newOid,
			(char *)ResCur.CurUserdesc->basePtr,
			sizeof(OID));

		// insert it to the set.
		list->insertToList(&newOid,&status);
		if (status != 0){
			CloseCursor( &ResCur );
			return FALSE;
		}

		if (!IncRefCount(newOid)) {
			ur->userError = catalogError;
			ur->operationType = HANDLE_ERROR;
			CloseCursor( &ResCur );
			return FALSE;
		}

          	if (!GetNextObject(&ResCur,&Flag)){
			CloseCursor( &ResCur );
			return FALSE;
		}
	} // while

	CloseCursor(&ResCur);

	return TRUE;
}


int UpdateClause::updateSet( USERDESC *ud, int offset, int pos, char *typeName )
{
	OID             *setOid, *oldOid;
	int             err;
	CatBOOLEAN	status = FALSE;
	DeleteClause	del( ur );

	oldOid = (OID *)((char *)ud->basePtr+offset);
	setOid = (OID *)(operators.listAt(pos)->item);

	if( !OID_IS_INVALID( *oldOid ) ){
		if( !del.DeleteSet( oldOid, typeName ) ){
			ur->setError( RUNTIME_ERROR, CANNOT_DELETE_OBJ, "" );
			return FALSE;
		}
	}

	err = sm_WriteObject( BufferGroup, offset, sizeof(OID), (char *)setOid, ud,TRUE );
	if( err != esmNOERROR ){
		ur->setError( EXODUS_ERROR, err, "" );
		return FALSE;
	}


	return TRUE;
}


int UpdateClause::updateList( USERDESC  *ud, int offset, int pos, char *typeName )
{
        OID             *listOid, *oldOid;
        int             err;
        CatBOOLEAN      status = FALSE;
        DeleteClause    del( ur );

        oldOid = (OID *)((char *)ud->basePtr+offset);
        listOid = (OID *)(operators.listAt(pos)->item);

        if( !OID_IS_INVALID( *oldOid ) ){
                if( !del.DeleteList( oldOid, typeName ) ){
                        ur->setError( RUNTIME_ERROR, CANNOT_DELETE_OBJ, "" );
                        return FALSE;
                }
        }

        err = sm_WriteObject( BufferGroup, offset, sizeof(OID), (char *)listOid, ud,TRUE );
        if( err != esmNOERROR ){
                ur->setError( EXODUS_ERROR, err, "" );
                return FALSE;
        }


        return TRUE;

}


int UpdateClause::pathUpdate( OID *oid, int pos )
{
	LinkedList	*list;
	AttributeType	*attribute;
	USERDESC	*UserDesc;
	int		err, offset;
	char		*Object, errorMsg[MAXSTRLEN];
	Expression	*exp;
	JOIN_ARG	*jExp;
	CNST		retVal;
	OID		*tempOid;
	static	int	statI=0;

	list = (LinkedList *)(pathList.listAt(pos)->item);

	 // read the object corresponding to the oid
        err = sm_ReadObject( BufferGroup, oid, 0, READ_ALL, &UserDesc );
        if( err != esmNOERROR ){
                ur->setError( EXODUS_ERROR, 1, "" );
		//free(retVal );
                return FALSE;
        }

	attribute = (AttributeType *)(list->listAt(0)->item);
	Object = UserDesc->basePtr;
	offset = attribute->AttributeOffset;

	switch( attribute->AttributeConstructor ){
	case 'N' : exp = (Expression *) (updateExpression->listAt(pos+1)->item);
		   jExp = exp->convertToIlker( exp->lastItem );
		   evaluateExpr( jExp, &retVal );
		   switch(retVal.kind){
		   case INTEGER :  
			if (strcmp(attribute->AttrTypeName,"INTEGER")){
				sprintf( errorMsg, "(%s was expected)",
				attribute->AttrTypeName);
				ur->setError(RUNTIME_ERROR,R_TYPE_MISMATCH,errorMsg);
				sm_ReleaseObject( UserDesc );
				return FALSE;
			}
			err = sm_WriteObject(BufferGroup,offset,sizeof(int),(char *)&retVal.value.int_value, UserDesc, TRUE );
			if( err != esmNOERROR ){
				ur->setError( EXODUS_ERROR, err, "" );
				return FALSE;
			}
			break;
                   case REAL:
                        if (strcmp(attribute->AttrTypeName,"FLOAT")){
                                sprintf( errorMsg, "(%s was expected)",
                                attribute->AttrTypeName);
                                ur->setError(RUNTIME_ERROR,R_TYPE_MISMATCH,errorMsg);
				sm_ReleaseObject( UserDesc );
                                return FALSE;
                        }
			sm_WriteObject(BufferGroup,offset,sizeof(float),(char *)&retVal.value.real_value, UserDesc, TRUE );
			if( err != esmNOERROR ){
				ur->setError( EXODUS_ERROR, err, "" );
				return FALSE;
			}
                        break;
		   case TEXT :
			if (strcmp(attribute->AttrTypeName,"STRING")) {
			  	sprintf( errorMsg, "(%s was expected)",
				attribute->AttrTypeName);
				ur->setError( RUNTIME_ERROR, R_TYPE_MISMATCH, errorMsg);
				sm_ReleaseObject( UserDesc );
				return FALSE;
			}
			sm_WriteObject(BufferGroup,offset,attribute->AttributeLength,(char *)&retVal.value.text_value, UserDesc, TRUE );
			if( err != esmNOERROR ){
				ur->setError( EXODUS_ERROR, err, "" );
				return FALSE;
			}
			break;
		   }
		   break;
	case 'R' : attribute = (AttributeType *)(list->listAt(1)->item); 
		   if( !updateRef( UserDesc, offset, pos+1, attribute->AttrTypeName ) ){
			sm_ReleaseObject( UserDesc );
			return FALSE;
		   }
		   
		   break;
	case 'S' : attribute = (AttributeType *)(list->listAt(1)->item); 
		   if( !updateSet( UserDesc, offset, pos+1, attribute->AttrTypeName ) ){
			sm_ReleaseObject( UserDesc );
			return FALSE;
		   }
		   break; 
	case 'L' : attribute = (AttributeType *)(list->listAt(1)->item); 
		   if( !updateList( UserDesc, offset, pos+1, attribute->AttrTypeName ) ){
			sm_ReleaseObject( UserDesc );
			return FALSE;
		   }
		   break; 

	}		

	return TRUE;

}


int UpdateClause::update(OID *oid, int &position )
{
	unsigned	itemType = updateExpression->listAt(position)->itemType;

	switch( itemType ){
		case PATH_EXP : if( !pathUpdate( oid, position ) ) return FALSE;
				position+=2;
				return TRUE;
		// case FUNCTION : lhfdkjhfdgkjhfdkj;
	}
	
	return TRUE;
}


int UpdateClause::update( OID *oid )
{
	char 	flag = 1;
	int	size = updateExpression->size, position = 0;

	while( position < size ) if( !update( oid, position ) ) return FALSE;

	return TRUE;
}

/*
int UpdateClause::checkTypes( int pos )
{
	unsigned	rvalueType = operators.listAt(pos+1).itemType;
	LinkedList	*path = pathList.listAt(pos)->item;
	int		pathSize = path->size, i;
	AttributeType	*attribute = ( AttributeType *)(path->listAt(pathSize-1)->item),
			*prevAttr  = ( AttributeType *)(path->listAt(pathSize-2)->item);

	if( !strcmp( attribute->TypeName, "STRING" ) ){
		if( rvalueType != STRCONS ){
			ur->setError( PARSE_ERROR, TYPE_MISMATCH );
			return FALSE;
		}
		return TRUE;
	}

        if( !strcmp( attribute->TypeName, "INTEGER" ) ){
                if( rvalueType != UNSIGNEDINT ){
                        ur->setError( PARSE_ERROR, TYPE_MISMATCH );
                        return FALSE;
                }
                return TRUE;
        }

        if( !strcmp( attribute->TypeName, "FLOAT" ) ){
                if( rvalueType != UNSIGNEDREAL ){
                        ur->setError( PARSE_ERROR, TYPE_MISMATCH );
                        return FALSE;
                }
                return TRUE;
        }


	switch( prevAttr->AttributeConstructor ){
	case 'R' : // if it is a ref assignment 
		if( rvalueType != QRY_PTR && rvalueType != NEW_PTR ){
			ur->setError( PARSE_ERROR, TYPE_MISMATCH );
			return FALSE;
		}
		break;
	case 'S' : // if it is a set assignment
		if( rvalue != SET_PTR ){
			ur->setError( PARSE_ERROR, TYPE_MISMATCH );
			return FALSE;
		}
		break;
	case 'L' : // if it is a list assignment
		if( rvalue != LST_PTR ){
			ur->setError( PARSE_ERROR, TYPE_MISMATCH );
			return FALSE;
		}
		break;
	}	
	return TRUE;
}

int UpdateClause::semanticCheck()
{
	int		i, size = updateExpression->size;
	unsigned	itemType;

	for( i = 0; i < size; i++ ){
		itemType = updateExpression->listAt(i)->itemType;
		if( itemType == PATH_EXP && !checkTypes( i ) ) return FALSE;
			
	}	
}
*/

char *UpdateClause::getBindVar()
{
	BindListItem 	*bli;

	bli = ( BindListItem *)(bindList->firstItem->item);
	return bli->alias;
}

char *UpdateClause::getClassName()
{
        BindListItem    *bli;

        bli = ( BindListItem *)(bindList->firstItem->item);
        return bli->classInfo.name;
}

int UpdateClause::execute()
{
	Query           *query;
	OID             oid;
	UserRequest	*userreq;
	char 		*bindVar, Flag;
	MOODSCursor	ResCur;

	// Convert UPDATE Clause to a SELECT Clause and execute it...
	bindList->checkSemantic();
	bindVar = getBindVar();
	className = getClassName();

	if( !fillOpList() ) return FALSE;
	if( !findLinkedLists() ) return FALSE;
//	if( !semanticCheck() ) return FALSE;


	query = new Query(ur);

	userreq = new UserRequest( bindVar );
	projectionList = new ProjectionExpression( userreq );
	projectionList->parse();
	delete userreq;

	query->projectionList = projectionList;
	query->bindList = bindList;
	query->whereP = whereP;

	if( !query->generateAllTree() ) return FALSE;
	if( !query->generatePlan() ) return FALSE;

	OpenCursor(query->solutionFile,&ResCur);

	if (!GetFirstObject(&ResCur,&Flag))
		return(FALSE);

	while( !Flag ){

		memcpy((char *)&oid, (char *)ResCur.CurUserdesc->basePtr, sizeof(OID));
		if( !update( &oid ) ){
			CloseCursor( &ResCur );
			return FALSE;
		}
		// Get next object from the cursor...
		if (!GetNextObject(&ResCur,&Flag)){
			CloseCursor( &ResCur );
			return FALSE;
		}

	}	

	if( !CloseCursor( &ResCur ) ) return FALSE;

	return TRUE;
}
